{*******************************************************************************
  Author      : Steve Pinneo (steveyvr@yahoo.ca)
  Updated     : June 10 2003
  Version     : 0.9

 LICENSE:

  The contents of this file are subject to the Mozilla Public License
  Version 1.1 (the "License"); you may not use this file except in
  compliance with the License. You may obtain a copy of the License
  at http://www.mozilla.org/MPL/

  Software distributed under the License is distributed on an "AS IS"
  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
  the License for the specific language governing rights and
  limitations under the License.

  Alternatively, the contents of this file may be used under the terms
  of the GNU General Public License Version 2 or later (the "GPL"), in
  which case the provisions of the GPL are applicable instead of those
  above. If you wish to allow use of your version of this file only
  under the terms of the GPL and not to allow others to use your
  version of this file under the MPL, indicate your decision by
  deleting the provisions above and replace them with the notice and
  other provisions required by the GPL. If you do not delete the
  provisions above, a recipient may use your version of this file
  under either the MPL or the GPL.

*******************************************************************************}

unit TBXTabReg;

interface

{$I TB2Ver.inc}

uses
  Windows, Classes, Controls, SysUtils,
  {$IFDEF JR_D6} DesignIntf, DesignEditors, {$ELSE} DsgnIntf, {$ENDIF}
  TBXImageTabSet;

procedure Register;

type
  TTBXImageTabSetEditor = class(TDefaultEditor)
  protected
{$IFDEF JR_D6}
    procedure GetTabsProp(const Prop: IProperty);
{$ELSE}
    procedure GetTabsProp(Prop: TPropertyEditor);
{$ENDIF}
  public
    procedure Edit; override;
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;

implementation

uses
  Forms, TBXThemes, TBXUtils, TypInfo;

{ TTBXImageTabSetEditor }

procedure TTBXImageTabSetEditor.Edit;
var
{$IFDEF JR_D6}
  Components: IDesignerSelections;
{$ELSE}
  {$IFDEF JR_D5}
  Components: TDesignerSelectionList;
  {$ELSE}
  Components: TComponentList;
  {$ENDIF}
{$ENDIF}
begin
{$IFDEF JR_D6}
  Components := CreateSelectionList;
{$ELSE}
  {$IFDEF JR_D5}
  Components := TDesignerSelectionList.Create;
  {$ELSE}
  Components := TComponentList.Create;
  {$ENDIF}
{$ENDIF}
  try
    Components.Add(Component);
    GetComponentProperties(Components, [tkClass], Designer, GetTabsProp);
  finally
{$IFNDEF JR_D6}
    Components.Free;
{$ENDIF}
  end;
end;

procedure TTBXImageTabSetEditor.ExecuteVerb(Index: Integer);
begin
  if Index = 0 then Edit;
end;

function TTBXImageTabSetEditor.GetVerb(Index: Integer): string;
begin
  if Index = 0 then Result := '&Tabs Editor...';
end;

function TTBXImageTabSetEditor.GetVerbCount: Integer;
begin
  Result := 1;
end;

{$IFDEF JR_D6}
procedure TTBXImageTabSetEditor.GetTabsProp(const Prop: IProperty);
begin
  if SameText(Prop.GetName, 'Items') then Prop.Edit;
end;
{$ELSE}
procedure TTBXImageTabSetEditor.GetTabsProp(Prop: TPropertyEditor);
begin
  if CompareText(Prop.GetName, 'Items') = 0 then Prop.Edit;
end;
{$ENDIF}

procedure Register;
begin
  RegisterComponents('Toolbar2000', [TTBXImageTabSet]);
  RegisterComponentEditor(TTBXImageTabSet, TTBXImageTabSetEditor);
end;

end.
