unit TBXCustomControl;

{$I mxs.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, TBX, TBXThemes;

type
  TTBXCustomControl = class(TCustomControl)
  protected
   FMouseOver: boolean;
   procedure TBMThemeChange(var Message: TMessage); message TBM_THEMECHANGE;
   procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
   procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
   procedure WMEraseBkgnd(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
   procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
   //procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
   //procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
   //procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
   procedure ThemeChanged; virtual;
  public
   constructor Create(AOwner: TComponent); override;
   destructor Destroy; override;
   procedure SetItemInfo(state: string; var IInfo: TTBXItemInfo);
  end;

implementation

{=========================}
{=== TTBXCustomControl ===}
{=========================}

constructor TTBXCustomControl.Create(AOwner: TComponent);
begin
 inherited;
 AddThemeNotification(Self);
 DoubleBuffered := true;
 ControlStyle := ControlStyle + [csOpaque];
 FMouseOver := false;
end;

destructor TTBXCustomControl.Destroy;
begin
 RemoveThemeNotification(Self);
 inherited;
end;

procedure TTBXCustomControl.TBMThemeChange(var Message: TMessage);
begin
 if Message.WParam = TSC_VIEWCHANGE then
  ThemeChanged;
end;

procedure TTBXCustomControl.ThemeChanged;
begin
 Invalidate;
end;

procedure TTBXCustomControl.CMMouseEnter(var Message: TMessage);
begin
 inherited;
 if not FMouseOver then
  begin
   FMouseOver := true;
   Invalidate;
  end;
end;

procedure TTBXCustomControl.CMMouseLeave(var Message: TMessage);
begin
 inherited;
 if FMouseOver then
  begin
   FMouseOver := false;
   Invalidate;
  end;
end;

procedure TTBXCustomControl.WMEraseBkgnd(var Message: TWMEraseBkgnd);
begin
 Message.Result := 1;
end;

procedure TTBXCustomControl.SetItemInfo(state: string; var IInfo: TTBXItemInfo);
begin
   FillChar(IInfo, SizeOf(TTBXItemInfo), 0);
   IInfo.ViewType := TVT_NORMALTOOLBAR;
   IInfo.ItemOptions := IO_APPACTIVE or IO_TOOLBARSTYLE;
   IInfo.Enabled := Enabled;
   IInfo.IsVertical := false;

   if state = 'normal' then
   begin
     IInfo.Pushed := false;
     IInfo.Selected := false;
     IInfo.HoverKind := hkNone;
   end;
   if state = 'hot' then
   begin
     IInfo.Pushed := false;
     IInfo.Selected := false;
     IInfo.HoverKind := hkMouseHover;
   end;
   if state = 'down' then
   begin
     IInfo.Pushed := true;
     IInfo.Selected := false;
     IInfo.HoverKind := hkMouseHover;
   end;
   if state = 'checked' then
   begin
     IInfo.Pushed := false;
     IInfo.Selected := true;
     IInfo.HoverKind := hkNone;
   end;
   if state = 'checked-hot' then
   begin
     IInfo.Pushed := false;
     IInfo.Selected := true;
     IInfo.HoverKind := hkMouseHover;
   end;
   if state = 'checked-down' then
   begin
     IInfo.Pushed := true;
     IInfo.Selected := true;
     IInfo.HoverKind := hkMouseHover;
   end;
end;

procedure TTBXCustomControl.CMEnabledChanged(var Message: TMessage);
begin
 inherited;
 Invalidate;
end;

end.
