unit mbTBXArrowButton;

{$I mxs.inc}

interface

uses
  Messages, SysUtils, Classes, Controls, StdCtrls, Windows, TBX, TBXUtils, TBXThemes,
  Graphics, ImgList, Menus;

type
  TMouseLoc = (mlNone, mlOver, mlDown);
  TArrowStyle = (asLeft, asRight, asUp, asDown, asNone, asImage);

  TmbTBXCustomArrowButton = class(TCustomControl)
  private
   FMouseLoc: TMouseLoc;
   FMouseOver: boolean;
   FImages: TCustomImageList;
   FImageChangeLink: TChangeLink;
   FImageIndex: TImageIndex;
   FMenu: TPopupMenu;
   FArr: TArrowStyle;
   FJust: boolean;
   FOnDrop, FOnClose: TNotifyEvent;
   FHotTrack: boolean;

   procedure SetHotTrack(Value: boolean);
   procedure SetJust(j: boolean);
   procedure SetArr(d: TArrowStyle);
   procedure SetImageIndex(i: TImageIndex);
   procedure ImageListChange(Sender: TObject);
   Procedure SetImages(i: TCustomImageList);
   procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
   procedure TBMThemeChange(var Message: TMessage); message TBM_THEMECHANGE;
   procedure SetItemInfo(state: string; var IInfo: TTBXItemInfo);
  protected
   procedure CMLostFocus(var Message: TCMLostFocus); message CM_EXIT;
   procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
   procedure PaintArrow(c: TCanvas; arrs: TArrowStyle);
   procedure WMEraseBkgnd(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
   procedure Notification(AComponent: TComponent; Operation: TOperation); override;
   procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
   procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
   procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
   procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
   procedure KeyPress(var Key: Char); override;
  public
   constructor Create(AOwner: TComponent); override;
   destructor Destroy; override;

   property JustPaintArrow: boolean read FJust write SetJust default false;
   property ArrowStyle: TArrowStyle read FArr write SetArr default asRight;
   property DropMenu: TPopupMenu read FMenu write FMenu;
   property Images: TCustomImageList read FImages write SetImages;
   property ImageIndex: TImageIndex read FImageIndex write SetImageIndex default -1;
   property OnBeforeDropMenu: TNotifyEvent read FOnDrop write FOnDrop;
   property OnAfterDropMenu: TNotifyEvent read FOnClose write FOnClose;
   property HotTracking: boolean read FHotTrack write SetHotTrack default true;
  end;

  TmbTBXArrowButton = class(TmbTBXCustomArrowButton)
  published
   property DropMenu;
   property Images;
   property ImageIndex default -1;
   property Anchors;
   property Constraints;
   property DragCursor;
   property DragKind;
   property DragMode;
   property Enabled;
   property Font;
   property ParentFont;
   property ParentShowHint;
   property PopupMenu;
   property ShowHint;
   property TabOrder;
   property TabStop default true;
   property Visible;
   property Hint;
   property ArrowStyle;
   property JustPaintArrow;
   property HotTracking;

   property OnBeforeDropMenu;
   property OnAfterDropMenu;
   property OnClick;
   property OnDblClick;
   property OnContextPopup;
   property OnDragDrop;
   property OnDragOver;
   property OnEndDock;
   property OnEndDrag;
   property OnKeyDown;
   property OnKeyPress;
   property OnKeyUp;
   property OnStartDock;
   property OnStartDrag;
  end;

implementation

constructor TmbTBXCustomArrowButton.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
 ControlStyle := ControlStyle + [csOpaque, csDoubleClicks] - [csAcceptsControls];
 AddThemeNotification(Self);
 FMouseLoc := mlNone;
 FMouseOver := false;
 DoubleBuffered := true;
 FImageIndex := -1;
 FImageChangeLink := TChangeLink.Create;
 FImageChangeLink.OnChange := ImageListChange;
 TabStop := true;
 FArr := asRight;
 Height := 20;
 Width := 9;
 FJust := false;
 FHotTrack := true;
end;

destructor TmbTBXCustomArrowButton.Destroy;
begin
 RemoveThemeNotification(Self);
 FImageChangeLink.Free;
 inherited Destroy;
end;

procedure TmbTBXCustomArrowButton.WMPaint(var Message: TWMPaint);
var
  R, IR: TRect;
  ii: TTBXItemInfo;
  rgn: HRGN;
begin
  inherited;
  R := ClientRect;
  Canvas.Brush.Color := Color;
  Canvas.FillRect(R);
  if not FJust then
   begin
    //paint frame in case of transparent buttons
    SetItemInfo('over', ii);
    CurrentTheme.PaintFrame(Canvas, R, ii);
    // set info
    if (csDesigning in ComponentState) or (Enabled = false) then FMouseLoc := mlNone;
    case FMouseLoc of
     mlNone: SetItemInfo('normal', ii);
     mlOver:
      if FHotTrack then
       SetItemInfo('over', ii)
      else
       SetItemInfo('normal', ii);
     mlDown: SetItemInfo('down', ii);
    end;
    ii.Enabled := Enabled;

    InflateRect(R, -1, -1);

    rgn := CreateRectRgnIndirect(R);
    SelectClipRgn(Canvas.Handle, rgn);

    InflateRect(R, 10, 10);
    CurrentTheme.PaintBackgnd(canvas, R, R, R, CurrentTheme.GetViewColor(TVT_NORMALTOOLBAR), false, TVT_NORMALTOOLBAR);
    CurrentTheme.PaintDock(canvas, R, R, DP_TOP);
    if CurrentTheme.PaintDockBackground then
     CurrentTheme.PaintBackgnd(canvas, R, R, R, CurrentTheme.GetViewColor(TVT_NORMALTOOLBAR), false, TVT_NORMALTOOLBAR)
    else
     CurrentTheme.PaintDock(canvas, R, R, DP_TOP);
    InflateRect(R, -10, -10);
    InflateRect(R, 1, 1);
    CurrentTheme.PaintButton(Canvas, R, ii);

    DeleteObject(rgn);
   end;
  //draw the image
  if FJust and (CurrentTheme.GetItemTextColor(ii) = Color) then
   begin
    Canvas.Pen.Color := clWindowText;
    Canvas.Brush.Color := clWindowText;
   end
  else
   begin
    Canvas.Pen.Color := CurrentTheme.GetItemTextColor(ii);
    Canvas.Brush.Color := CurrentTheme.GetItemTextColor(ii);
   end;
  case FArr of
   asLeft, asRight, asUp, asDown: PaintArrow(Canvas, FArr);
   asImage:
    if (FImages <> nil) and (FImageIndex > -1) then
     begin
      with IR do
       begin
        Left := (Width - FImages.Width) div 2;
        Right := Left + FImages.Width;
        Top := (Height - FImages.Height) div 2;
        Bottom := Top + FImages.Height;
       end;
      CurrentTheme.PaintImage(Canvas, ir, ii, FImages, FImageIndex);
     end;
  end;
end;

procedure TmbTBXCustomArrowButton.PaintArrow(c: TCanvas; arrs: TArrowStyle);
var
  X, Y: Integer;
begin
 X := (ClientRect.Left + ClientRect.Right) div 2;
 Y := (ClientRect.Top + ClientRect.Bottom) div 2 - 1;
 if FMouseLoc = mlDown then Inc(Y);
 case arrs of
  asLeft: c.Polygon([Point(X, Y + 2), Point(X, Y - 2), Point(X - 2, Y)]);
  asRight: c.Polygon([Point(X, Y - 2), Point(X, Y + 2), Point(X + 2, Y)]);
  asUp: c.Polygon([Point(X + 2, Y), Point(X - 2, Y), Point(X, Y - 2)]);
  asDown: c.Polygon([Point(X - 2, Y), Point(X + 2, Y), Point(X, Y + 2)]);
 end;
end;

procedure TmbTBXCustomArrowButton.SetItemInfo(state: string; var IInfo: TTBXItemInfo);
begin
   FillChar(IInfo, SizeOf(TTBXItemInfo), 0);
   IInfo.ViewType := TVT_NORMALTOOLBAR;
   IInfo.ItemOptions := IO_APPACTIVE or IO_TOOLBARSTYLE;
   IInfo.Enabled := True;
   IInfo.IsVertical := false;

   if state = 'normal' then
    begin
     IInfo.Pushed := false;
     IInfo.Selected := false;
     IInfo.HoverKind := hkNone;
    end;
   if state = 'over' then
    begin
     IInfo.Pushed := false;
     IInfo.Selected := false;
     IInfo.HoverKind := hkMouseHover;
    end;
   if state = 'down' then
    begin
     IInfo.Pushed := true;
     IInfo.Selected := false;
     IInfo.HoverKind := hkMouseHover;
    end;
end;

procedure TmbTBXCustomArrowButton.TBMThemeChange(var Message: TMessage);
begin
if Message.WParam = TSC_VIEWCHANGE then
 begin
  FMouseLoc := mlNone;
  Invalidate;
 end;
end;

procedure TmbTBXCustomArrowButton.CMMouseEnter(var Message: TMessage);
begin
 FMouseOver := true;
 FMouseLoc := mlOver;
 if FHotTrack then
 Invalidate;
end;

procedure TmbTBXCustomArrowButton.CMMouseLeave(var Message: TMessage);
begin
 FMouseOver := false;
 FMouseLoc := mlNone;
 if FHotTrack then
 Invalidate;
end;

procedure TmbTBXCustomArrowButton.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
 p: TPoint;
begin
 if Button = mbLeft then
  begin
   SetFocus;
   FMouseLoc := mlDown;
   Invalidate;
   if FMenu <> nil then
    begin
     case FArr of
      asLeft:
       begin
        FMenu.Alignment := paRight;
        p := Point(0, 0);
       end;
      asRight:
       begin
        FMenu.Alignment := paLeft;
        p := Point(Width, 0);
       end;
      asUp:
       begin
        FMenu.Alignment := paLeft;
        p := Point(0, 0);
       end;
      asDown:
       begin
        FMenu.Alignment := paLeft;
        p := Point(0, Height);
       end;
     end;
     if Assigned(FOnDrop) then FOnDrop(Self);
     FMenu.Popup(ClientToScreen(p).x, ClientToScreen(p).y);
     if Assigned(FOnClose) then FOnClose(Self);
     if FmouseOver then
      FMouseLoc := mlOver
     else
      FMouseLoc := mlNone;
    end;
   Invalidate;
  end;
end;

procedure TmbTBXCustomArrowButton.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 if FMouseOver then
  FMouseLoc := mlOver
 else
  FMouseLoc := mlNone;
 Invalidate;
end;

procedure TmbTBXCustomArrowButton.CMEnabledChanged(var Message: TMessage);
begin
 inherited;
 Invalidate;
end;

procedure TmbTBXCustomArrowButton.Notification(AComponent: TComponent; Operation: TOperation);
begin
 inherited Notification(AComponent, Operation);
 if Operation = opRemove then
  if AComponent = Images then
   SetImages(nil);
end;

procedure TmbTBXCustomArrowButton.SetImages(i: TCustomImageList);
begin
 if FImages <> nil then
  FImages.UnRegisterChanges(FImageChangeLink);
 FImages := i;
 if FImages <> nil then
  begin
   FImages.RegisterChanges(FImageChangeLink);
   FImages.FreeNotification(Self);
  end;
 Invalidate;
end;

procedure TmbTBXCustomArrowButton.ImageListChange(Sender: TObject);
begin
 if (Images <> nil) and (Sender = Images) then
  Invalidate;
end;

procedure TmbTBXCustomArrowButton.SetImageIndex(i: TImageIndex);
begin
 FImageIndex := i;
 Invalidate;
end;

procedure TmbTBXCustomArrowButton.KeyPress(var Key: Char);
var
 p: TPoint;
begin
 if key = ' ' then
  begin
   FMouseLoc := mlDown;
   Invalidate;
   if FMenu <> nil then
    begin
     case FArr of
      asLeft:
       begin
        FMenu.Alignment := paRight;
        p := Point(0, 0);
       end;
      asRight:
       begin
        FMenu.Alignment := paLeft;
        p := Point(Width, 0);
       end;
      asUp:
       begin
        FMenu.Alignment := paLeft;
        p := Point(0, 0);
       end;
      asDown:
       begin
        FMenu.Alignment := paLeft;
        p := Point(0, Height);
       end;
     end;
     if Assigned(FOnDrop) then FOnDrop(Self);
     FMenu.Popup(ClientToScreen(p).x, ClientToScreen(p).y);
     if Assigned(FOnClose) then FOnClose(Self);
    end;
   if FmouseOver then
    FMouseLoc := mlOver
   else
    FMouseLoc := mlNone;
   Invalidate;
  end;
 inherited KeyPress(Key);
end;

procedure TmbTBXCustomArrowButton.SetArr(d: TArrowStyle);
begin
 FArr := d;
 Invalidate;
end;

procedure TmbTBXCustomArrowButton.WMEraseBkgnd(var Message: TWMEraseBkgnd);
begin
 Message.Result := 1;
end;

procedure TmbTBXCustomArrowButton.CMLostFocus(var Message: TCMLostFocus);
begin
 inherited;
 if FMouseOver then
  FMouseLoc := mlOver
 else
  FMouseLoc := mlNone;
 Invalidate;
end;

procedure TmbTBXCustomArrowButton.SetJust(j: boolean);
begin
 FJust := j;
 Invalidate;
end;

procedure TmbTBXCustomArrowButton.SetHotTrack(Value: boolean);
begin
 if FHotTrack <> Value then
  begin
   FHotTrack := Value;
   Invalidate;
  end;
end;

end.
