unit mbTBXBevel;

{$I mxs.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Graphics, TBX, TBXThemes, TBXUtils;

type
  TBevelStyle = (bsBox, bsLeftLine, bsTopLine, bsRightLine, bsBottomLine);
  TmbTBXBevel = class(TCustomControl)
  private
   FStyle: TBevelStyle;
   FTransparent: boolean;

   procedure SetTransparent(t: boolean);
   procedure SetStyle(s: TBevelStyle);
  protected
   procedure Paint; override;
   procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
   procedure WMEraseBkgnd(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
   procedure TBMThemeChange(var Message: TMessage); message TBM_THEMECHANGE;
  public
   constructor Create(AOwner: TComponent); override;
   destructor Destroy; override;
  published
   property Align;
   property Anchors;
   property Constraints;
   property Visible;
   property Enabled;
   property Style: TBevelStyle read FStyle write SetStyle default bsBox;
   property Transparent: boolean read FTransparent write SetTransparent default false;
   property Color;
   property ParentColor;
   property ParentShowHint;
  end;

implementation

constructor TmbTBXBevel.Create(AOwner: TComponent);
begin
 inherited;
 AddThemeNotification(Self);
 DoubleBuffered := true;
 ControlStyle := COntrolStyle - [csAcceptsControls] + [csOpaque];
 FStyle := bsBox;
 Width := 50;
 Height := 50;
 FTransparent := false;
end;

destructor TmbTBXBevel.Destroy;
begin
 RemoveThemeNotification(Self);
 inherited;
end;

procedure TmbTBXBevel.TBMThemeChange(var Message: TMessage);
begin
if Message.WParam = TSC_VIEWCHANGE then
 Invalidate;
end;

procedure TmbTBXBevel.Paint; 
var
 IInfo: TTBXItemInfo;
begin
 with Canvas do
  begin
   IInfo.ViewType := TVT_NORMALTOOLBAR;
   IInfo.ItemOptions := IO_APPACTIVE or IO_TOOLBARSTYLE;
   IInfo.Enabled := Enabled;
   IInfo.Pushed := false;
   IInfo.Selected := false;
   IInfo.HoverKind := hkNone;
   Pen.Color := Blend(CurrentTheme.GetItemColor(IInfo), clBtnShadow, 50);
   Brush.Color := Color;
   FillRect(ClientRect);
   case FStyle of
    bsBox: Rectangle(ClientRect);
    bsLeftLine:
     begin
      MoveTo(0, 0);
      LineTo(0, Height);
     end;
    bsTopLine:
     begin
      MoveTo(0, 0);
      LineTo(Width, 0);
     end;
    bsRightLine:
     begin
      MoveTo(Width - 1, 0);
      LineTo(Width - 1, Height - 1);
     end;
    bsBottomLine:
     begin
      MoveTo(0, Height - 1);
      LineTo(Width - 1, Height - 1);
     end;
   end;
  end;
end;

procedure TmbTBXBevel.SetStyle(s: TBevelStyle);
begin
 FStyle := s;
 invalidate;
end;

procedure TmbTBXBevel.SetTransparent(t: boolean);
begin
 FTransparent := t;
 invalidate;
end;

procedure TmbTBXBevel.CMEnabledChanged(var Message: TMessage);
begin
 inherited;
 invalidate;
end;

procedure TmbTBXBevel.WMEraseBkgnd(var Message: TWMEraseBkgnd);
begin
 if not FTransparent then
  Message.Result := 1
 else
  inherited;
end;

end.
