unit mbTBXBooleanCombo;

{$I mxs.inc}

interface

uses
  Messages, SysUtils, Classes, Controls, ExtCtrls, Windows, Graphics, StdCtrls,
  mbTBXComboBox, TBX, TBXThemes, TBXUtils;

type
  TmbTBXBooleanCombo = class(TmbTBXComboBox)
  private
   FPaintHotBox: boolean;
   FDrawLabel: TDrawItemLabelEvent;

   procedure SetPaintHot(h: boolean);
  protected
   procedure DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
   procedure CreateWnd; override;
   function GetSelection: boolean;
   procedure SetSelection(s: boolean);
   procedure EnabledChanged; 
  public
   constructor Create(AOwner: TComponent); override;
   destructor Destroy; override;
  published
   property Color;
   property DragMode;
   property DragCursor;
   property Enabled;
   property Font;
   property ItemHeight;
   property MaxLength;
   property ParentFont;
   property ParentColor default false;
   property ParentShowHint;
   property PopupMenu;
   property ShowHint;
   property TabOrder;
   property TabStop;
   property Visible;
   property PaintHotBox: boolean read FPaintHotBox write SetPaintHot default false;
   property Value: boolean read GetSelection write SetSelection default true;

   property OnDrawItemLabel: TDrawItemLabelEvent read FDrawLabel write FDrawLabel;
   property OnChange;
   property OnClick;
   property OnDblClick;
   property OnDragDrop;
   property OnDragOver;
   property OnDrawItem;
   property OnDropDown;
   property OnEndDrag;
   property OnEnter;
   property OnExit;
   property OnKeyDown;
   property OnKeyPress;
   property OnKeyUp;
   property OnMeasureItem;
   property OnStartDrag;
  end;

implementation

constructor TmbTBXBooleanCombo.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
ParentColor := false;
 DoubleBuffered := true;
 Style := csOwnerDrawFixed;
 FPaintHotBox := false;
end;

destructor TmbTBXBooleanCombo.Destroy;
begin
 inherited Destroy;
end;

function TmbTBXBooleanCombo.GetSelection: Boolean;
begin
 Result := (Items.Strings[ItemIndex] = 'True');
end;

procedure TmbTBXBooleanCombo.SetSelection(s: boolean);
begin
 if s then
  ItemIndex := 0
 else
  ItemIndex := 1;
end;

procedure TmbTBXBooleanCombo.DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
 R, IR: TRect;
 ii: TTBXItemInfo;
 ed: TTBXEditInfo;
begin
 Canvas.Font := Font;
 Canvas.Brush.Style := bsSolid;
 if odSelected in State then
  begin
   SetItemInfo('over', ii, ed);
   if FThemedSelect then
    begin
     Canvas.Pen.color := CurrentTheme.GetItemTextColor(ii);
     Canvas.Brush.Color := CurrentTheme.GetItemColor(ii);
     Canvas.Font.Color := CurrentTheme.GetItemTextColor(ii);
    end
   else
    begin
     Canvas.Brush.Color := clHighlight;
     Canvas.Pen.Color := clHighlightText;
     Canvas.Font.Color := clHighlightText;
    end;
  end
 else
  begin
   SetItemInfo('normal', ii, ed);
   Canvas.Brush.Color := Color;
   if FThemedSelect then
    begin
     Canvas.Pen.color := CurrentTheme.GetItemTextColor(ii);
     Canvas.Font.Color := CurrentTheme.GetItemTextColor(ii);
    end
   else
    begin
     Canvas.Pen.Color := clWindowText;
     Canvas.Font.Color := clWindowText;
    end;
  end;
 Canvas.FillRect(Rect);
 // set checkmark rect
 R := Rect;
 Inc(R.Left, 1);
 Inc(R.Top, 1);
 Dec(R.Bottom, 1);
 R.Right := R.Left + (R.Bottom - R.Top);
 InflateRect(R, -1, -1);
 if FPaintHotBox then
  begin
   if Enabled then
    case Index of
     0: CurrentTheme.PaintFrameControl(Canvas, R, PFC_CHECKBOX, PFS_CHECKED or PFS_HOT, nil);
     1: CurrentTheme.PaintFrameControl(Canvas, R, PFC_CHECKBOX, PFS_HOT, nil);
    end
   else
    CurrentTheme.PaintFrameControl(Canvas, R, PFC_CHECKBOX, PFS_DISABLED or PFS_HOT, nil);
  end
 else
  begin
   if Enabled then
    case Index of
     0: CurrentTheme.PaintFrameControl(Canvas, R, PFC_CHECKBOX, PFS_CHECKED, nil);
     1: CurrentTheme.PaintFrameControl(Canvas, R, PFC_CHECKBOX, 0, nil);
    end
   else
    CurrentTheme.PaintFrameControl(Canvas, R, PFC_CHECKBOX, PFS_DISABLED, nil);
  end;
 // draw text
 Canvas.Brush.Style := bsClear;
 IR := Rect;
 IR.Left := R.Right + 6;
 if Assigned(FDrawLabel) then FDrawLabel(Self, Index, Canvas.Font, State);
 DrawText(Canvas.Handle, PChar(Items.Strings[index]), Length(Items.Strings[index]), IR, DT_VCENTER or DT_SINGLELINE);
 if Assigned(OnDrawItem) then
  OnDrawItem(Self, Index, Rect, State);
end;

procedure TmbTBXBooleanCombo.CreateWnd;
begin
 inherited CreateWnd;
 Items.Clear;
 Items.Add('True');
 Items.Add('False');
 ItemIndex := 0;
end;

procedure TmbTBXBooleanCombo.SetPaintHot(h: boolean);
begin
 FPaintHotBox := h;
 Invalidate;
end;

procedure TmbTBXBooleanCombo.EnabledChanged;
begin
 inherited;
 Invalidate;
end;

end.
