unit mbTBXBrushStyleCombo;

{$I mxs.inc}

interface

uses
  Messages, SysUtils, Classes, Controls, ExtCtrls, Windows, Graphics, StdCtrls,
  mbTBXComboBox, TBX, TBXThemes;

type
  TmbTBXCustomBrushStyleCombo = class(TmbTBXComboBox)
  private
   FDrawLabel: TDrawItemLabelEvent;
  protected
   procedure DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
   procedure CreateWnd; override;
   function GetSelection: TBrushStyle;
   procedure SetSelection(Value: TBrushStyle);
   procedure EnabledChanged;
  public
   constructor Create(AOwner: TComponent); override;
   destructor Destroy; override;
   property OnDrawItemLabel: TDrawItemLabelEvent read FDrawLabel write FDrawLabel;
  end;

  TmbTBXBrushStyleCombo = class(TmbTBXCustomBrushStyleCombo)
  published
   property Color;
   property DragMode;
   property DragCursor;
   property DropDownCount;
   property Enabled;
   property Font;
   property ItemHeight;
   property MaxLength;
   property ParentColor default false;
   property ParentFont;
   property ParentShowHint;
   property PopupMenu;
   property ShowHint;
   property TabOrder;
   property TabStop;
   property Text;
   property Visible;
   property Selection: TBrushStyle read GetSelection write SetSelection default bsSolid;

   property OnDrawItemLabel;
   property OnChange;
   property OnClick;
   property OnDblClick;
   property OnDragDrop;
   property OnDragOver;
   property OnDrawItem;
   property OnDropDown;
   property OnEndDrag;
   property OnEnter;
   property OnExit;
   property OnKeyDown;
   property OnKeyPress;
   property OnKeyUp;
   property OnMeasureItem;
   property OnStartDrag;
  end;

implementation

const BrushStyleArray: array[0..7] of string = ('Solid', 'Clear', 'Horizontal', 'Vertical', 'Diagonal Right', 'Diagonal Left', 'Cross', 'Diagonal Cross');

constructor TmbTBXCustomBrushStyleCombo.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
ParentColor := false;
 DoubleBuffered := true;
 Style := csOwnerDrawFixed;
end;

destructor TmbTBXCustomBrushStyleCombo.Destroy;
begin
 inherited Destroy;
end;

function TmbTBXCustomBrushStyleCombo.GetSelection: TBrushStyle;
begin
 Result := TBrushStyle(ItemIndex);
end;

procedure TmbTBXCustomBrushStyleCombo.SetSelection(Value: TBrushStyle);
var
 i: integer;
begin
 for i:=0 to 7 do
  if TBrushStyle(i) = Value then
   begin
    ItemIndex := i;
    Exit;
   end;
end;

procedure TmbTBXCustomBrushStyleCombo.DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
 R, IR: TRect;
 ii: TTBXItemInfo;
 ed: TTBXEditInfo;
 tc: TColor;
begin
 Canvas.Font := Font;
 if odSelected in State then
  begin
   SetItemInfo('over', ii, ed);
   if FThemedSelect then
    begin
     Canvas.Pen.color := CurrentTheme.GetItemTextColor(ii);
     Canvas.Brush.Color := CurrentTheme.GetItemColor(ii);
     Canvas.Font.Color := CurrentTheme.GetItemTextColor(ii);
    end
   else
    begin
     Canvas.Brush.Color := clHighlight;
     Canvas.Pen.Color := clHighlightText;
     Canvas.Font.Color := clHighlightText;
    end;
  end
 else
  begin
   SetItemInfo('normal', ii, ed);
   Canvas.Brush.Color := Color;
   if FThemedSelect then
    begin
     Canvas.Pen.color := CurrentTheme.GetItemTextColor(ii);
     Canvas.Font.Color := CurrentTheme.GetItemTextColor(ii);
    end
   else
    begin
     Canvas.Pen.Color := clWindowText;
     Canvas.Font.Color := clWindowText;
    end;
  end;
 Canvas.FillRect(Rect);
 // set icon rect
 R := Rect;
 Inc(R.Left, 1);
 Inc(R.Top, 1);
 Dec(R.Bottom, 1);
 R.Right := R.Left + (R.Bottom - R.Top);
 // draw icon
 if odSelected in state then
  SetItemInfo('over', ii, ed)
 else
  SetItemInfo('normal', ii, ed);
 Canvas.Brush.Color := CurrentTheme.GetItemTextColor(ii);
 Canvas.Brush.Style := TBrushStyle(Index);
 tc := Canvas.Font.Color;
 if Assigned(FDrawLabel) then FDrawLabel(Self, Index, Canvas.Font, State);
 if Canvas.Font.Color <> tc then
  Canvas.Brush.Color := Canvas.Font.Color;
 Canvas.Rectangle(R);
 // draw text
 Canvas.Brush.Style := bsClear;
 IR := Rect;
 IR.Left := R.Right + 4;
 DrawText(Canvas.Handle, PChar(Items.Strings[index]), Length(Items.Strings[index]), IR, DT_VCENTER or DT_SINGLELINE);
 if Assigned(OnDrawItem) then
  OnDrawItem(Self, Index, Rect, State);
end;

procedure TmbTBXCustomBrushStyleCombo.CreateWnd;
var
 c: integer;
begin
 inherited CreateWnd;
 Items.Clear;
 for c := 0 to High(BrushStyleArray) do
  Items.Add(BrushStyleArray[c]);
 SetSelection(bsSolid);
end;

procedure TmbTBXCustomBrushStyleCombo.EnabledChanged;
begin
 inherited;
 Invalidate;
end;

end.
