unit mbTBXButton;

{$I mxs.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Graphics, TBX, TBXThemes, ActnList,
  ImgList, TBXUtils, Math, StdCtrls, Forms;

type
  TMouseLoc = (mlNone, mlOver, mlDown, mlChecked, mlCheckedHot);
  TGlyphLayout = (glyLeft, glyRight, glyTop, glyBottom);
  TEllipsisStyle = (esNone, esEndEllipsis, esPathEllipsis);


  TmbTBXButton = class(TButton)
  private
   FChecked: boolean;
   FGroupIndex: integer;
   FImages: TImageList;
   FImageChangeLink: TChangeLink;
   FImageIndex: TImageIndex;
   FOnChange: TNotifyEvent;
   FAutoSize: boolean;
   FLay: TGlyphLayout;
   FSpacing: integer;
   FEllipsis: TEllipsisStyle;
   FCheckable: boolean;
   FFramed: boolean;
   FAllowAllUp: boolean;
   canvas: TControlCanvas;
   FThemed: boolean;
   FHotTrack: boolean;
   FExtraFrame: boolean;

   procedure SetExtraFrame(Value: boolean);
   procedure SetHotTrack(Value: boolean);
   procedure SetThemed(Value: boolean);
   procedure SetFramed(f: boolean);
   procedure SetEllipsis(e: TEllipsisStyle);
   procedure SetSpacing(s: integer);
   procedure SetLayout(l: TGlyphLayout);
   procedure SetIRAutoSize(a: boolean);
   procedure SetImageIndex(i: TImageIndex);
   procedure ImageListChange(Sender: TObject);
   Procedure SetImages(i: TImageList);
   procedure SetGroupIndex(Value: Integer);
   procedure SetIsChecked(c: boolean);
   procedure SetShowFocus(s: boolean);
  protected
   FMouseLoc: TMouseLoc;
   FMouseOver: boolean;
   FMouseDown: boolean;
   FShowFocus: boolean;

   procedure UpdateExclusive;
   procedure Resize; override;
   procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
   procedure DrawImage(ii: TTBXItemInfo);
   procedure WMEraseBkgnd(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
   procedure WMSetText(var Msg: TWMSetText); message WM_SETTEXT;
   procedure TBMThemeChange(var Message: TMessage); message TBM_THEMECHANGE;
   procedure ActionChange(Sender: TObject; CheckDefaults: Boolean); override;
   procedure CMButtonPressed(var Message: TMessage); message CM_BUTTONPRESSED;
   procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
   procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
   procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
   procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
   procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
   procedure CMGotFocus(var Message: TCMGotFocus); message CM_ENTER;
   procedure CMLostFocus(var Message: TCMLostFocus); message CM_EXIT;
   procedure CMFocusChanged(var Message: TCMFocusChanged); message CM_FOCUSCHANGED;
   procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
   procedure Notification(AComponent: TComponent; Operation: TOperation); override;
   procedure KeyDown(var Key: Word; Shift: TShiftState); override;
   procedure KeyUp(var Key: Word; Shift: TShiftState); override;
   procedure WMUpdateUIState(var Message: TMessage); message WM_UPDATEUISTATE;
  public
   constructor Create(AOwner: TComponent); override;
   destructor Destroy; override;
  published
   property Checkable: boolean read FCheckable write FCheckable default false;
   property AutoSize: boolean read FAutoSize write SetIRAutoSize default false;
   property GroupIndex: integer read FGroupIndex write SetGroupIndex default 0;
   property Checked: boolean read FChecked write SetIsChecked default false;
   property Images: TImageList read FImages write SetImages;
   property ImageIndex: TImageIndex read FImageIndex write SetImageIndex default -1;
   property GlyphLayout: TGlyphLayout read FLay write SetLayout default glyLeft;
   property GlyphSpacing: integer read FSpacing write SetSpacing default 4;
   property EllipsisStyle: TEllipsisStyle read FEllipsis write SetEllipsis default esNone;
   property ShowFocus: boolean read FShowFocus write SetShowFocus default true;
   property Framed: boolean read FFramed write SetFramed default true;
   property AllowAllUp: boolean read FAllowAllUp write FAllowAllUp default false;
   property TabStop default true;
   property Themed: boolean read FThemed write SetThemed default true;
   property HotTracking: boolean read FHotTrack write SetHotTrack default true;
   property ExtraFrame: boolean read FExtraFrame write SetExtraFrame default false;

   property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

implementation

{ TmbTBXButton }

constructor TmbTBXButton.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
 AddThemeNotification(Self);
 canvas := TControlCanvas.Create;
 canvas.Control := self;
 DoubleBuffered := true;
 ControlStyle := ControlStyle - [csAcceptsControls] + [csOpaque];
 Height := 25;
 Width := 85;
 TabStop := true;
 FChecked := false;
 FImageIndex := -1;
 FImageChangeLink := TChangeLink.Create;
 FImageChangeLink.OnChange := ImageListChange;
 FAutoSize := false;
 FMouseOver := false;
 FMouseLoc := mlNone;
 Flay := glyLeft;
 FSpacing := 4;
 FEllipsis := esNone;
 FCheckable := false;
 FMouseDown := false;
 FShowFocus := true;
 FFramed := true;
 FAllowAllUp := false;
 FThemed := true;
 FHotTrack := true;
 FExtraFrame := false;
end;

destructor TmbTBXButton.Destroy;
begin
 RemoveThemeNotification(Self);
 FImageChangeLink.Free;
 Canvas.Free;
 inherited;
end;

procedure TmbTBXButton.TBMThemeChange(var Message: TMessage);
begin
if Message.WParam = TSC_VIEWCHANGE then
 Invalidate;
end;

procedure SetItemInfo(state: string; var IInfo: TTBXItemInfo);
begin
   FillChar(IInfo, SizeOf(TTBXItemInfo), 0);
   IInfo.ViewType := TVT_NORMALTOOLBAR;
   IInfo.ItemOptions := IO_APPACTIVE or IO_TOOLBARSTYLE;
   IInfo.Enabled := True;
   IInfo.IsVertical := false;

   if state = 'normal' then
   begin
     IInfo.Pushed := false;
     IInfo.Selected := false;
     IInfo.HoverKind := hkNone;
   end;
   if state = 'over' then
   begin
     IInfo.Pushed := false;
     IInfo.Selected := false;
     IInfo.HoverKind := hkMouseHover;
   end;
   if state = 'down' then
   begin
     IInfo.Pushed := true;
     IInfo.Selected := false;
     IInfo.HoverKind := hkMouseHover;
   end;
   if state = 'checked' then
   begin
     IInfo.Pushed := false;
     IInfo.Selected := true;
     IInfo.HoverKind := hkNone;
   end;
   if state = 'checkedhot' then
   begin
     IInfo.Pushed := false;
     IInfo.Selected := true;
     IInfo.HoverKind := hkMouseHover;
   end;
end;

procedure TmbTBXButton.WMPaint(var Message: TWMPaint);
var
 R: TRect;
 ii: TTBXItemInfo;
 rgn: hrgn;
begin
 inherited;
 if not FThemed then Exit;
 R := ClientRect;
 DrawParentBackground(Self, canvas.Handle, R);
 if csDesigning in ComponentState then
  begin
   FMouseDown := false;
   FMouseOver := false;
  end;
 if FFramed then
  begin
   SetItemInfo('over', ii);
   CurrentTheme.PaintFrame(canvas, R, ii);
  end;
 // set info
 if FShowFocus then
  if Focused and (FMouseLoc <> mlDown) and not FMouseDown then FMouseLoc := mlOver;
 if FChecked then
  begin
   if FMouseOver then
    begin
     if not (csDesigning in ComponentState) then
      begin
       if FHotTrack then
        FMouseLoc := mlCheckedHot
       else
        FMouseLoc := mlChecked;
      end;
    end
   else
    FMouseLoc := mlChecked;
  end
 else
  if (csDesigning in ComponentState) or (Enabled = false) then
   FMouseLoc := mlNone;
 case FMouseLoc of
  mlNone: SetItemInfo('normal', ii);
  mlOver:
   if FHotTrack or (Focused and FShowFocus) then
    SetItemInfo('over', ii)
   else
    SetItemInfo('normal', ii);
  mlDown: SetItemInfo('down', ii);
  mlChecked: SetItemInfo('checked', ii);
  mlCheckedHot:
   if FHotTrack then
    SetItemInfo('checkedhot', ii)
   else
    SetItemInfo('checked', ii);
 end;
 ii.Enabled := Enabled;
 //paint
 CurrentTheme.PaintFrame(canvas, R, ii);
 if FFramed then
  InflateRect(R, -1, -1);

 rgn := CreateRectRgnIndirect(R);
 SelectClipRgn(Canvas.Handle, rgn);

 InflateRect(R, 10, 10);
 CurrentTheme.PaintBackgnd(canvas, R, R, R, CurrentTheme.GetViewColor(TVT_NORMALTOOLBAR), false, TVT_NORMALTOOLBAR);
 CurrentTheme.PaintDock(canvas, R, R, DP_TOP);
 if CurrentTheme.PaintDockBackground then
  CurrentTheme.PaintBackgnd(canvas, R, R, R, CurrentTheme.GetViewColor(TVT_NORMALTOOLBAR), false, TVT_NORMALTOOLBAR)
 else
  CurrentTheme.PaintDock(canvas, R, R, DP_TOP);
 InflateRect(R, -10, -10);

 DeleteObject(rgn);

 if FFramed then
  InflateRect(R, 1, 1);
 CurrentTheme.PaintButton(canvas, R, ii);

 DrawImage(ii);

 if FExtraFrame then
  begin
   InflateRect(R, -2, -2);
   ExcludeClipRect(Canvas.Handle, R.Left, R.Top, R.Right, R.Bottom);
   InflateRect(R, 2, 2);
   CurrentTheme.PaintFrame(canvas, R, ii);
  end;
end;

procedure TmbTBXButton.DrawImage(ii: TTBXItemInfo);
var
 GR, TR, R: TRect;
begin
 R := ClientRect;
 InflateRect(R, -1, -1);
 if (FImages <> nil) and (FImageIndex > -1) then
  begin
   //Set the GlyphRect
   case FLay of
    glyLeft:
     begin
      with GR do
       begin
        Left := (Width - (FImages.Width + FSpacing + canvas.TextWidth(Caption))) div 2;
        Right := Left + FImages.Width;
        Top := (Height - FImages.Height) div 2;
        Bottom := Top + FImages.Height;
       end;
      with TR do
       begin
        Left := GR.Left + FSpacing + FImages.Width;
        Top := (Height - canvas.TextHeight(Caption)) div 2;
        Right := Left + canvas.TextWidth(Caption);
        Bottom := Top + canvas.TextHeight(Caption);
       end;
     end;
    glyRight:
     begin
      with GR do
       begin
        Left := (Width - (FImages.Width + FSpacing + canvas.TextWidth(Caption))) div 2 + FSpacing + canvas.TextWidth(Caption);
        Right := Left + FImages.Width;
        Top := (Height - FImages.Height) div 2;
        Bottom := Top + FImages.Height;
       end;
      with TR do
       begin
        Left := (Width - (FImages.Width + FSpacing + canvas.TextWidth(Caption))) div 2;
        Top := (Height - canvas.TextHeight(Caption)) div 2;
        Right := Left + canvas.TextWidth(Caption);
        Bottom := Top + canvas.TextHeight(Caption);
       end;
     end;
    glyTop:
     begin
      with GR do
       begin
        Left := (Width - FImages.Width) div 2;
        Right := Left + FImages.Width;
        Top := (Height - (FImages.Height + FSpacing + canvas.TextHeight(Caption))) div 2;
        Bottom := Top + FImages.Height;
       end;
      with TR do
       begin
        Left := (Width - canvas.TextWidth(Caption)) div 2;
        Top := GR.Bottom + FSpacing;
        Right := Left + canvas.TextWidth(Caption);
        Bottom := Top + canvas.TextHeight(Caption);
       end;
     end;
    glyBottom:
     begin
      with GR do
       begin
        Left := (Width - FImages.Width) div 2;
        Right := Left + FImages.Width;
        Top := (Height - (FImages.Height + FSpacing + canvas.TextHeight(Caption))) div 2 +  FSpacing + canvas.TextHeight(Caption);
        Bottom := Top + FImages.Height;
       end;
      with TR do
       begin
        Left := (Width - canvas.TextWidth(Caption)) div 2;
        Top := GR.Top - (FSpacing + canvas.TextHeight(Caption));
        Right := Left + canvas.TextWidth(Caption);
        Bottom := Top + canvas.TextHeight(Caption);
       end;
     end;
   end;
   //draw the image
   CurrentTheme.PaintImage(canvas, GR, ii, FImages, FImageIndex);
  end
 else
  begin
   //Set the TextRect
   TR := R;
   InflateRect(TR, -2, -2);
  end;
 //Draw the text
 canvas.Brush.Style := bsClear;
 Font.Color := CurrentTheme.GetItemTextColor(ii);
 canvas.Font := Font;
 case FEllipsis of
  esNone: DrawText(canvas.Handle, PChar(Caption), Length(Caption), TR, DT_VCENTER or DT_CENTER or DT_SINGLELINE);
  esEndEllipsis: DrawText(canvas.Handle, PChar(Caption), Length(Caption), TR, DT_VCENTER or DT_LEFT or DT_SINGLELINE or DT_END_ELLIPSIS);
  esPathEllipsis: DrawText(canvas.Handle, PChar(Caption), Length(Caption), TR, DT_VCENTER or DT_LEFT or DT_SINGLELINE or DT_PATH_ELLIPSIS);
 end;
end;

procedure TmbTBXButton.Resize;
begin
 inherited;
 SetIRAutoSize(FAutoSize);
 invalidate;
end;

procedure TmbTBXButton.CMMouseEnter(var Message: TMessage);
begin
 FMouseOver := true;
 if not FMouseDown then
  FMouseLoc := mlOver
 else
  FMouseLoc := mlDown;
 inherited;
 Invalidate;
end;

procedure TmbTBXButton.CMMouseLeave(var Message: TMessage);
begin
 FMouseOver := false;
 if not FMouseDown then
  FMouseLoc := mlNone
 else
  FMouseLoc := mlDown;
 inherited;
 Invalidate;
end;

procedure TmbTBXButton.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
 inherited;
 if not (FMouseLoc in [mlOver, mlDown, mlCheckedHot]) or not FMouseOver then
  begin
   FMouseOver := true;
   FMouseLoc := mlOver;
   Invalidate;
  end;
end;

procedure TmbTBXButton.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 FMouseDown := Button = mbLeft;
 if Button = mbLeft then
  begin
   SetFocus;
   FMouseLoc := mlDown;
  end
 else
  FMouseLoc := mlOver;
 Invalidate;
 UpdateExclusive;
 inherited;
end;

procedure TmbTBXButton.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 FMouseDown := false;
 if Button = mbLeft then
  begin
   if FMouseOver then
    FMouseLoc := mlOver
   else
    FMouseLoc := mlNone;
   if PtInRect(ClientRect, Point(x, y)) then
    SetIsChecked(true); 
   Invalidate;
  end;
 UpdateExclusive;
 inherited;
end;

procedure TmbTBXButton.CMGotFocus(var Message: TCMGotFocus);
begin
 inherited;
 if FShowFocus then
  FMouseLoc := mlOver;
 Invalidate;
end;

procedure TmbTBXButton.CMLostFocus(var Message: TCMLostFocus);
begin
 inherited;
 if not FMouseOver then
  FMouseLoc := mlNone
 else
  FMouseLoc := mlOver;
 Invalidate;
end;

procedure TmbTBXButton.CMFocusChanged(var Message: TCMFocusChanged);
begin
 inherited;
 Invalidate;
end;

procedure TmbTBXButton.CMEnabledChanged(var Message: TMessage);
begin
 inherited;
 Invalidate;
end;

procedure TmbTBXButton.SetShowFocus(s: boolean);
begin
 if FShowFocus <> s then
  begin
   FShowFocus := s;
   Invalidate;
  end;
end;

procedure TmbTBXButton.SetIsChecked(c: boolean);
begin
 if FAllowAllUp then
  if FChecked and c then
   c := false;
 if FCheckable then
  begin
   FChecked := c;
   if not FChecked then
    begin
     if FMouseOver then
      FMouseLoc := mlOver
     else
      FMouseLoc := mlNone;
    end;
   if (Action is TAction) then TAction(Action).Checked := c;
  end;
  invalidate;
  UpdateExclusive;
  if Assigned(FOnChange) then FOnChange(Self);
end;

procedure TmbTBXButton.SetGroupIndex(Value: Integer);
begin
 if FGroupIndex <> Value then
  begin
   FGroupIndex := Value;
   UpdateExclusive;
  end;
end;

procedure TmbTBXButton.UpdateExclusive;
var
 Msg: TMessage;
begin
 if (FGroupIndex <> 0) and (Parent <> nil) then
  begin
   Msg.Msg := CM_BUTTONPRESSED;
   Msg.WParam := FGroupIndex;
   Msg.LParam := Longint(Self);
   Msg.Result := 0;
   Parent.Broadcast(Msg);
  end;
 invalidate;
end;

procedure TmbTBXButton.CMButtonPressed(var Message: TMessage);
var
 Sender: TmbTBXButton;
begin
 if Message.WParam = FGroupIndex then
  begin
   Sender := TmbTBXButton(Message.LParam);
   if (Sender <> Self) and Sender.Checked and FChecked then
    SetIsChecked(false);
  end;
end;

procedure TmbTBXButton.Notification(AComponent: TComponent; Operation: TOperation);
begin
 inherited Notification(AComponent, Operation);
 if Operation = opRemove then
  if AComponent = Images then
   SetImages(nil);
end;

procedure TmbTBXButton.SetImages(i: TImageList);
begin
 if FImages <> nil then
  FImages.UnRegisterChanges(FImageChangeLink);
 FImages := i;
 if FImages <> nil then
  begin
   FImages.RegisterChanges(FImageChangeLink);
   FImages.FreeNotification(Self);
  end;
 SetIRAutoSize(FAutoSize);
 Invalidate;
end;

procedure TmbTBXButton.ImageListChange(Sender: TObject);
begin
 if Sender = Images then
  begin
   SetIRAutoSize(FAutoSize);
   Invalidate;
  end;
end;

procedure TmbTBXButton.SetImageIndex(i: TImageIndex);
begin
 if FImageIndex <> i then
  begin
   FImageIndex := i;
   SetIRAutoSize(FAutoSize);
   Invalidate;
 end;
end;

procedure TmbTBXButton.ActionChange(Sender: TObject; CheckDefaults: Boolean);
begin
 inherited;
 {$IFDEF DELPHI_7_UP}
 if Sender is TAction then
  begin
   if CheckDefaults or (GroupIndex = 0) then
    GroupIndex := TAction(Sender).GroupIndex;
  end;
 {$ENDIF} 
end;

procedure TmbTBXButton.SetIRAutoSize(a: boolean);
begin
 FAutoSize := a;
 if a then
  begin
   if (FImages <> nil) and (FImageIndex > -1) then
    case FLay of
     glyLeft, glyRight:
      begin
       Width := FImages.Width + Canvas.TextWidth(Caption) + FSpacing + 2*6;
       Height := Max(FImages.Height, Canvas.TextHeight(Caption)) + 2*6;
      end;
     glyTop, glyBottom:
      begin
       Width := Max(FImages.Width, Canvas.TextWidth(Caption)) + 2*6;
       Height := FImages.Height + Canvas.TextHeight(Caption) + FSpacing + 2*6;
      end;
     end
   else
    begin
     Width := Canvas.TextWidth(Caption) + 2*6;
     Height := Canvas.TextHeight(Caption) + 2*6;
    end;
  end;
 invalidate;
end;

procedure TmbTBXButton.KeyDown(var Key: Word; Shift: TShiftState);
begin
 if key = VK_SPACE then
  begin
  if FMouseLoc <> mlDown then
   begin
    FMouseDown := true;
    FMouseLoc := mlDown;
   end;
   SetIsChecked(true);
   Invalidate;
  end;
 UpdateExclusive;
 inherited;
end;

procedure TmbTBXButton.KeyUp(var Key: Word; Shift: TShiftState);
begin
if Key = VK_SPACE then
 begin
  FMouseDown := false;
  if not FMouseOver then
   FMouseLoc := mlNone
  else
   FMouseLoc := mlOver;
 end;
Invalidate;
inherited;
end;

procedure TmbTBXButton.SetLayout(l: TGlyphLayout);
begin
 FLay := l;
 SetIRAutoSize(FAutoSize);
 Invalidate;
end;

procedure TmbTBXButton.WMSetText(var Msg: TWMSetText);
begin
 inherited;
 SetIRAutoSize(FAutoSize);
 Invalidate;
end;

procedure TmbTBXButton.SetEllipsis(e: TEllipsisStyle);
begin
 FEllipsis := e;
 Invalidate;
end;

procedure TmbTBXButton.SetSpacing(s: integer);
begin
 FSpacing := s;
 SetIRAutoSize(FAutoSize);
 Invalidate;
end;

procedure TmbTBXButton.SetFramed(f: boolean);
begin
 FFramed := f;
 Invalidate;
end;

procedure TmbTBXButton.WMEraseBkgnd(var Message: TWMEraseBkgnd);
begin
 Message.Result := 1;
end;

procedure TmbTBXButton.SetThemed(Value: boolean);
begin
 FThemed := Value;
 Invalidate;
end;

procedure TmbTBXButton.SetHotTrack(Value: boolean);
begin
 if FHotTrack <> Value then
  begin
   FHotTrack := Value;
   Invalidate;
  end;
end;

procedure TmbTBXButton.SetExtraFrame(Value: boolean);
begin
 if FExtraFrame <> Value then
  begin
   FExtraFrame := Value;
   Invalidate;
  end;
end;

procedure TmbTBXButton.WMUpdateUIState(var Message: TMessage);
begin
 inherited;
 invalidate;
end;

end.
