unit mbTBXClipartBox;

{$I mxs.inc}

interface

uses
  Messages, SysUtils, Classes, Controls, StdCtrls, Windows, TBX, TBXUtils, TBXThemes,
  Graphics, ImgList, Menus;

type
  TMouseLoc = (mlNone, mlOver, mlDown);

  TmbTBXCustomClipartBox = class(TCustomControl)
  private
   FMouseLoc: TMouseLoc;
   FMouseOver: boolean;
   FImages: TCustomImageList;
   FImageChangeLink: TChangeLink;
   FImageIndex: TImageIndex;
   FAutoSize: boolean;
   FCenter: boolean;
   FOnBtnClick: TNotifyEvent;
   FMenu: TPopupMenu;
   FHotTrack: boolean;

   procedure SetHotTrack(Value: boolean);
   procedure SetFAutoSize(a: boolean);
   procedure SetCenter(c: boolean);
   procedure SetImageIndex(i: TImageIndex);
   procedure ImageListChange(Sender: TObject);
   Procedure SetImages(i: TCustomImageList);
   procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
   procedure TBMThemeChange(var Message: TMessage); message TBM_THEMECHANGE;
   procedure SetItemInfo(state: string; var IInfo: TTBXItemInfo; var EInfo: TTBXEditInfo);
  protected
   procedure WMEraseBkgnd(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
   procedure CMGotFocus(var Message: TCMGotFocus); message CM_ENTER;
   procedure CMLostFocus(var Message: TCMLostFocus); message CM_EXIT;
   procedure Notification(AComponent: TComponent; Operation: TOperation); override;
   procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
   procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
   procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
   procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
   procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
   procedure KeyPress(var Key: Char); override;
  public
   constructor Create(AOwner: TComponent); override;
   destructor Destroy; override;

   property DropMenu: TPopupMenu read FMenu write FMenu;
   property AutoSize: boolean read FAutoSize write SetFAutoSize default true;
   property Center: boolean read FCenter write SetCenter default true;
   property Images: TCustomImageList read FImages write SetImages;
   property ImageIndex: TImageIndex read FImageIndex write SetImageIndex default -1;
   property OnBtnClick: TNotifyEvent read FOnBtnClick write FOnBtnClick;
   property HotTracking: boolean read FHotTrack write SetHotTrack default true;
  end;

  TmbTBXClipartBox = class(TmbTBXCustomClipartBox)
  published
   property DropMenu;
   property Images;
   property ImageIndex default -1;
   property Center default true;
   property AutoSize default true;
   property Anchors;
   property Constraints;
   property Color;
   property DragCursor;
   property DragKind;
   property DragMode;
   property Enabled;
   property Font;
   property ParentFont;
   property ParentColor default false;
   property ParentShowHint;
   property PopupMenu;
   property ShowHint;
   property TabOrder;
   property TabStop default true;
   property Visible;
   property Hint;
   property HotTracking;

   property OnContextPopup;
   property OnDragDrop;
   property OnDragOver;
   property OnEndDock;
   property OnEndDrag;
   property OnKeyDown;
   property OnKeyPress;
   property OnKeyUp;
   property OnStartDock;
   property OnStartDrag;
   property OnBtnClick;
  end;

implementation

constructor TmbTBXCustomClipartBox.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
 ControlStyle := ControlStyle + [csOpaque];
 ParentColor := false;
 AddThemeNotification(Self);
 FMouseLoc := mlNone;
 FMouseOver := false;
 DoubleBuffered := true;
 FImageIndex := -1;
 FImageChangeLink := TChangeLink.Create;
 FImageChangeLink.OnChange := ImageListChange;
 FCenter := true;
 FAutoSize := true;
 TabStop := true;
 Width := 145;
 Height := 25;
 FHotTrack := true;
end;

destructor TmbTBXCustomClipartBox.Destroy;
begin
 RemoveThemeNotification(Self);
 FImageChangeLink.Free;
 inherited Destroy;
end;

procedure TmbTBXCustomClipartBox.WMPaint(var Message: TWMPaint);
var
  R, I: TRect;
  ii: TTBXItemInfo;
  ed: TTBXEditInfo;
begin
  inherited;
  //check size first
  if FAutoSize and (FImages <> nil) then
  begin
   width := 6 + GetSystemMetrics(SM_CXHSCROLL) + FImages.Width;
   Height := 6 + FImages.Height;
  end;
  //paint
  GetWindowRect(Handle, R);
  OffsetRect(R, -R.Left, -R.Top);
  Canvas.Brush.Color := Color;
  Canvas.FillRect(ClientRect);
  // set info
  case FMouseLoc of
   mlNone: SetItemInfo('normal', ii, ed);
   mlOver:
    if FHotTrack then
    SetItemInfo('over', ii, ed)
   else
    SetItemInfo('normal', ii, ed);
   mlDown: SetItemInfo('down', ii, ed);
  end;
  if Focused and (FMouseLoc <> mlDown) then SetItemInfo('over', ii, ed);
  if (csDesigning in ComponentState) or (Enabled = false) then SetItemInfo('normal', ii, ed);
  ii.Enabled := Enabled;
  //draw the image
  if (FImages <> nil) and (FImageIndex > -1) then
   begin
    I := ClientRect;
    with I do
     begin
      if FCenter then
       begin
        I.Left := I.Left + (((I.Right - GetSystemMetrics(SM_CXHSCROLL)) - I.Left) - FImages.Width) div 2;
        I.Right := I.Left + FImages.Width;
        I.Top := I.Top + ((I.Bottom - I.Top) - FImages.Height) div 2;
        I.Bottom := I.Top + FImages.Height;
       end
      else
       begin
        I.Left := 3;
        I.Top := 3;
        if I.Left + FImages.Width > Width - (3 + GetSystemMetrics(SM_CXHSCROLL)) then
         I.Right := Width - (3 + GetSystemMetrics(SM_CXHSCROLL))
        else
         I.Right := I.Left + FImages.Width;
        if I.Top + FImages.Height > Height - 3 then
         I.Bottom := Height - 3
        else
         I.Bottom := I.Top + FImages.Height;
        end;
     end;
    CurrentTheme.PaintImage(Canvas, I, ii, FImages, FImageIndex);
    // exclude image rect
    ExcludeClipRect(Canvas.Handle, I.Left, I.Top, width - (GetSystemMetrics(SM_CXHSCROLL) + 3), Height - 3);
   end;
  // paint the border and button
  CurrentTheme.PaintFrame(Canvas, R, ii);
  CurrentTheme.PaintBackgnd(Canvas, R, R, R, CurrentTheme.GetViewColor(TVT_NORMALTOOLBAR), false, TVT_NORMALTOOLBAR);
  CurrentTheme.PaintDock(Canvas, R, R, DP_TOP);
  if CurrentTheme.PaintDockBackground then
   CurrentTheme.PaintBackgnd(Canvas, R, R, R, CurrentTheme.GetViewColor(TVT_NORMALTOOLBAR), false, TVT_NORMALTOOLBAR)
  else
   CurrentTheme.PaintDock(Canvas, R, R, DP_TOP);
  CurrentTheme.PaintEditFrame(Canvas, R, ii, ed);
end;

procedure TmbTBXCustomClipartBox.SetItemInfo(state: string; var IInfo: TTBXItemInfo; var EInfo: TTBXEditInfo);
begin
   FillChar(IInfo, SizeOf(TTBXItemInfo), 0);
   IInfo.ViewType := TVT_NORMALTOOLBAR;
   IInfo.ItemOptions := IO_APPACTIVE or IO_TOOLBARSTYLE;
   IInfo.Enabled := True;

   FillChar(EInfo, SizeOf(TTBXEditInfo), 0);
   EInfo.RightBtnWidth := GetSystemMetrics(SM_CXHSCROLL);
   EInfo.RightBtnInfo.ButtonType := EBT_DROPDOWN;

   if state = 'normal' then
   begin
     IInfo.Pushed := false;
     IInfo.Selected := false;
     IInfo.HoverKind := hkNone;
     EInfo.RightBtnInfo.ButtonState := 0;
   end;
   if state = 'over' then
   begin
     IInfo.Pushed := false;
     IInfo.Selected := false;
     IInfo.HoverKind := hkMouseHover;
     EInfo.RightBtnInfo.ButtonState := EBDS_HOT;
   end;
   if state = 'down' then
   begin
     IInfo.Pushed := true;
     IInfo.Selected := false;
     IInfo.HoverKind := hkMouseHover;
     EInfo.RightBtnInfo.ButtonState := EBDS_PRESSED;
   end;
end;

procedure TmbTBXCustomClipartBox.TBMThemeChange(var Message: TMessage);
begin
if Message.WParam = TSC_VIEWCHANGE then
 begin
  FMouseLoc := mlNone;
  Invalidate;
 end;
end;

procedure TmbTBXCustomClipartBox.CMMouseEnter(var Message: TMessage);
begin
FMouseOver := true;
 if not Focused then
  begin
   FMouseLoc := mlOver;
   if FHotTrack then
   Invalidate;
  end;
end;

procedure TmbTBXCustomClipartBox.CMMouseLeave(var Message: TMessage);
begin
FMouseOver := false;
 if not Focused then
  begin
   FMouseLoc := mlNone;
   if FHotTrack then
   Invalidate;
  end;
end;

procedure TmbTBXCustomClipartBox.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
 R: TRect;
 p: TPoint;
begin
 if Button = mbLeft then
  begin
   SetFocus;
   R := Rect(width - (GetSystemMetrics(SM_CXHSCROLL) - 2), 0, width, height);
   if PtInRect(R, Point(x, y)) then
    begin
     FMouseLoc := mlDown;
     Invalidate;
     if FMenu <> nil then
      begin
       p := Point(width - (GetSystemMetrics(SM_CXHSCROLL) + 3), Height);
       FMenu.Popup(ClientToScreen(p).x, ClientToScreen(p).y);
      end;
     if FmouseOver then
      FMouseLoc := mlOver
     else
      FMouseLoc := mlNone;
     Invalidate;
    end;
  end;
end;

procedure TmbTBXCustomClipartBox.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 if FMouseLoc = mlDown then
  if Assigned(FOnBtnClick) then
   FOnBtnClick(Self);
 if not Focused then
  begin
   if FMouseOver then
    FMouseLoc := mlOver
   else
    FMouseLoc := mlNone;
   Invalidate;
  end;
end;

procedure TmbTBXCustomClipartBox.CMEnabledChanged(var Message: TMessage);
begin
 inherited;
 Invalidate;
end;

procedure TmbTBXCustomClipartBox.Notification(AComponent: TComponent; Operation: TOperation);
begin
 inherited Notification(AComponent, Operation);
 if Operation = opRemove then
  if AComponent = Images then
   SetImages(nil);
end;

procedure TmbTBXCustomClipartBox.SetImages(i: TCustomImageList);
begin
 if FImages <> nil then
  FImages.UnRegisterChanges(FImageChangeLink);
 FImages := i;
 if FImages <> nil then
  begin
   FImages.RegisterChanges(FImageChangeLink);
   FImages.FreeNotification(Self);
  end;
 Invalidate;
end;

procedure TmbTBXCustomClipartBox.ImageListChange(Sender: TObject);
begin
 if Sender = Images then
  Invalidate;
end;

procedure TmbTBXCustomClipartBox.SetImageIndex(i: TImageIndex);
begin
 FImageIndex := i;
 Invalidate;
end;

procedure TmbTBXCustomClipartBox.SetFAutoSize(a: boolean);
begin
 FAutoSize := a;
 Invalidate;
end;

procedure TmbTBXCustomClipartBox.SetCenter(c: boolean);
begin
 FCenter := c;
 Invalidate;
end;

procedure TmbTBXCustomClipartBox.CMGotFocus(var Message: TCMGotFocus);
begin
 inherited;
 FMouseLoc := mlOver;
 Invalidate;
end;

procedure TmbTBXCustomClipartBox.CMLostFocus(var Message: TCMLostFocus);
begin
 inherited;
 if FMouseOver then
  FMouseLoc := mlOver
 else
  FMouseLoc := mlNone;
 Invalidate;
end;

procedure TmbTBXCustomClipartBox.KeyPress(var Key: Char);
var
 p: TPoint;
begin
 if key = ' ' then
  begin
   FMouseLoc := mlDown;
   Invalidate;
   if FMenu <> nil then
    begin
     p := Point(width - (GetSystemMetrics(SM_CXHSCROLL) + 3), Height);
     FMenu.Popup(ClientToScreen(p).x, ClientToScreen(p).y);
    end;
   if FmouseOver then
    FMouseLoc := mlOver
   else
    FMouseLoc := mlNone;
   Invalidate;
  end;
 inherited KeyPress(Key);
end;

procedure TmbTBXCustomClipartBox.WMEraseBkgnd(var Message: TWMEraseBkgnd);
begin
 Message.Result := 1;
end;

procedure TmbTBXCustomClipartBox.SetHotTrack(Value: boolean);
begin
 if FHotTrack <> Value then
  begin
   FHotTrack := Value;
   Invalidate;
  end;
end;

end.
