unit mbTBXClipartScrollBox;

{$I mxs.inc}

interface

uses
  Messages, SysUtils, Classes, Controls, StdCtrls, Windows, TBX, TBXUtils, TBXThemes,
  Graphics, ImgList, ExtCtrls;

type
  TMouseLoc = (mlNone, mlOver, mlDown);
  TSpinButState = (ssDown, ssUp, ssNone);

  TmbTBXCustomClipartScrollBox = class(TCustomControl)
  private
   FMouseLoc: TMouseLoc;
   FMouseOver: boolean;
   FImages: TCustomImageList;
   FImageChangeLink: TChangeLink;
   FImageIndex: TImageIndex;
   FAutoSize: boolean;
   FCenter: boolean;
   FSBut: TSpinButState;
   FTimer: TTimer;
   FOnUpClick: TNotifyEvent;
   FOnDownClick: TNotifyEvent;
   FMouseDown: boolean;
   FInterval: integer;
   FInverse: boolean;
   Floop: boolean;
   FHotTrack: boolean;

   procedure SetHotTrack(Value: boolean);
   procedure SetInterval(i: integer);
   procedure SetFAutoSize(a: boolean);
   procedure SetCenter(c: boolean);
   procedure SetImageIndex(i: TImageIndex);
   procedure ImageListChange(Sender: TObject);
   Procedure SetImages(i: TCustomImageList);
   procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
   procedure TBMThemeChange(var Message: TMessage); message TBM_THEMECHANGE;
   procedure SetItemInfo(state: string; var IInfo: TTBXItemInfo; var EInfo: TTBXEditInfo);
  protected
   procedure WMEraseBkgnd(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
   procedure KeyDown(var Key: Word; Shift: TShiftState); override;
   procedure DoTimer(Sender: TObject);
   procedure UpClick;
   procedure DownClick;
   procedure CMGotFocus(var Message: TCMGotFocus); message CM_ENTER;
   procedure CMLostFocus(var Message: TCMLostFocus); message CM_EXIT;
   procedure Notification(AComponent: TComponent; Operation: TOperation); override;
   procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
   procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
   procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
   procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
   procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
   procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
  public
   constructor Create(AOwner: TComponent); override;
   destructor Destroy; override;

   property AutoSize: boolean read FAutoSize write SetFAutoSize default true;
   property Center: boolean read FCenter write SetCenter default true;
   property Images: TCustomImageList read FImages write SetImages;
   property ImageIndex: TImageIndex read FImageIndex write SetImageIndex default -1;
   property ScrollInterval: integer read FInterval write SetInterval default 200;
   property InverseScroll: boolean read FInverse write FInverse default false;
   property Loop: boolean read Floop write Floop default false;
   property OnDownClick: TNotifyEvent read FOnDownClick write FOnDownClick;
   property OnUpClick: TNotifyEvent read FOnUpClick write FOnUpClick;
   property HotTracking: boolean read FHotTrack write SetHotTrack default true;
  end;

  TmbTBXClipartScrollBox = class(TmbTBXCustomClipartScrollBox)
  published
   property Images;
   property ImageIndex default -1;
   property Center default true;
   property AutoSize default true;
   property Anchors;
   property Constraints;
   property Color;
   property DragCursor;
   property DragKind;
   property DragMode;
   property Enabled;
   property Font;
   property ParentFont;
   property ParentColor default false;
   property ParentShowHint;
   property PopupMenu;
   property ShowHint;
   property TabOrder;
   property TabStop default true;
   property Visible;
   property Hint;
   property ScrollInterval default 200;
   property InverseScroll default false;
   property Loop default false;
   property HotTracking;

   property OnContextPopup;
   property OnDragDrop;
   property OnDragOver;
   property OnEndDock;
   property OnEndDrag;
   property OnKeyDown;
   property OnKeyPress;
   property OnKeyUp;
   property OnStartDock;
   property OnStartDrag;
   property OnDownClick;
   property OnUpClick;
  end;

implementation

constructor TmbTBXCustomClipartScrollBox.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
 ControlStyle := ControlStyle + [csOpaque];
 ParentColor := false;
 AddThemeNotification(Self);
 FMouseLoc := mlNone;
 FMouseOver := false;
 DoubleBuffered := true;
 FImageIndex := -1;
 FImageChangeLink := TChangeLink.Create;
 FImageChangeLink.OnChange := ImageListChange;
 FCenter := true;
 FAutoSize := true;
 TabStop := true;
 FSBut := ssNone;
 FTimer := TTimer.Create(Self);
 FTimer.OnTimer := DoTimer;
 FTimer.Interval := 200;
 FTimer.Enabled := false;
 FMouseDown := false;
 FInterval := 200;
 FInverse := false;
 FLoop := false;
 Width := 145;
 Height := 25;
 FHotTrack := true;
end;

destructor TmbTBXCustomClipartScrollBox.Destroy;
begin
 FTimer.Free;
 RemoveThemeNotification(Self);
 FImageChangeLink.Free;
 inherited Destroy;
end;

procedure TmbTBXCustomClipartScrollBox.WMPaint(var Message: TWMPaint);
var
  R, I: TRect;
  ii: TTBXItemInfo;
  ed: TTBXEditInfo;
begin
  inherited;
  //check size first
  if FAutoSize and (FImages <> nil) then
  begin
   width := 6 + GetSystemMetrics(SM_CXHSCROLL) + FImages.Width;
   Height := 6 + FImages.Height;
  end;
  //paint
  GetWindowRect(Handle, R);
  OffsetRect(R, -R.Left, -R.Top);
  Canvas.Brush.Color := Color;
  Canvas.FillRect(ClientRect);
  // set info
  case FMouseLoc of
   mlNone: SetItemInfo('normal', ii, ed);
   mlOver:
    if FHotTrack then
    SetItemInfo('over', ii, ed)
   else
    SetItemInfo('normal', ii, ed);
   mlDown: SetItemInfo('down', ii, ed);
  end;
  if Focused and (FMouseLoc <> mlDown) then SetItemInfo('over', ii, ed);
  if (csDesigning in ComponentState) or (Enabled = false) then SetItemInfo('normal', ii, ed);
  ii.Enabled := Enabled;
  //draw the image
  if (FImages <> nil) and (FImageIndex > -1) then
   begin
    I := ClientRect;
    with I do
     begin
      if FCenter then
       begin
        I.Left := I.Left + (((I.Right - GetSystemMetrics(SM_CXHSCROLL)) - I.Left) - FImages.Width) div 2;
        I.Right := I.Left + FImages.Width;
        I.Top := I.Top + ((I.Bottom - I.Top) - FImages.Height) div 2;
        I.Bottom := I.Top + FImages.Height;
       end
      else
       begin
        I.Left := 3;
        I.Top := 3;
        if I.Left + FImages.Width > Width - (3 + GetSystemMetrics(SM_CXHSCROLL)) then
         I.Right := Width - (3 + GetSystemMetrics(SM_CXHSCROLL))
        else
         I.Right := I.Left + FImages.Width;
        if I.Top + FImages.Height > Height - 3 then
         I.Bottom := Height - 3
        else
         I.Bottom := I.Top + FImages.Height;
        end;
     end;
    CurrentTheme.PaintImage(Canvas, I, ii, FImages, FImageIndex);
    // exclude image rect
    ExcludeClipRect(Canvas.Handle, I.Left, I.Top, width - (GetSystemMetrics(SM_CXHSCROLL) + 3), Height - 3);
   end;
  // paint the border and button
  CurrentTheme.PaintFrame(Canvas, R, ii);
  CurrentTheme.PaintBackgnd(Canvas, R, R, R, CurrentTheme.GetViewColor(TVT_NORMALTOOLBAR), false, TVT_NORMALTOOLBAR);
  CurrentTheme.PaintDock(Canvas, R, R, DP_TOP);
  if CurrentTheme.PaintDockBackground then
   CurrentTheme.PaintBackgnd(Canvas, R, R, R, CurrentTheme.GetViewColor(TVT_NORMALTOOLBAR), false, TVT_NORMALTOOLBAR)
  else
   CurrentTheme.PaintDock(Canvas, R, R, DP_TOP);
  CurrentTheme.PaintEditFrame(Canvas, R, ii, ed);
end;

procedure TmbTBXCustomClipartScrollBox.SetItemInfo(state: string; var IInfo: TTBXItemInfo; var EInfo: TTBXEditInfo);
begin
   FillChar(IInfo, SizeOf(TTBXItemInfo), 0);
   IInfo.ViewType := TVT_NORMALTOOLBAR;
   IInfo.ItemOptions := IO_APPACTIVE or IO_TOOLBARSTYLE;
   IInfo.Enabled := True;

   FillChar(EInfo, SizeOf(TTBXEditInfo), 0);
   EInfo.RightBtnWidth := GetSystemMetrics(SM_CXHSCROLL);
   EInfo.RightBtnInfo.ButtonType := EBT_SPIN;

   if state = 'normal' then
    begin
     IInfo.Pushed := false;
     IInfo.Selected := false;
     IInfo.HoverKind := hkNone;
     EInfo.RightBtnInfo.ButtonState := 0;
    end;
   if state = 'over' then
    begin
     IInfo.Pushed := false;
     IInfo.Selected := false;
     IInfo.HoverKind := hkMouseHover;
     EInfo.RightBtnInfo.ButtonState := EBSS_HOT;
    end;
   if state = 'down' then
    begin
     IInfo.Pushed := true;
     IInfo.Selected := false;
     IInfo.HoverKind := hkMouseHover;
     case FSbut of
      ssDown: EInfo.RightBtnInfo.ButtonState := EBSS_DOWN;
      ssUp: EInfo.RightBtnInfo.ButtonState := EBSS_UP;
      ssNone: EInfo.RightBtnInfo.ButtonState := 0;
     end;
    end;
end;

procedure TmbTBXCustomClipartScrollBox.TBMThemeChange(var Message: TMessage);
begin
if Message.WParam = TSC_VIEWCHANGE then
 begin
  FMouseLoc := mlNone;
  Invalidate;
 end;
end;

procedure TmbTBXCustomClipartScrollBox.CMMouseEnter(var Message: TMessage);
begin
FMouseOver := true;
 if (not Focused) and (not FMouseDown) then
  begin
   FMouseLoc := mlOver;
   if FHotTrack then
    Invalidate;
  end;
end;

procedure TmbTBXCustomClipartScrollBox.CMMouseLeave(var Message: TMessage);
begin
FMouseOver := false;
 if (not Focused) and (not FMouseDown) then
  begin
   FMouseLoc := mlNone;
   if FHotTrack then
   Invalidate;
  end;
end;

procedure TmbTBXCustomClipartScrollBox.MouseMove(Shift: TShiftState; X, Y: Integer);
var
 R: TRect;
begin
 FMouseOver := true;
 R := Rect(width - (GetSystemMetrics(SM_CXHSCROLL) - 2), 0, width, height);
 if FMouseLoc <> mlOver then
  begin
   FMouseLoc := mlOver;
    if FHotTrack then
   Invalidate;
  end;
 if PtInRect(R, Point(x, y)) or FMouseDown then
  begin
   if y > height div 2 then
    FSBut := ssDown
   else
    FSBut := ssUp;
   FTimer.Enabled := (ssLeft in Shift) and FMouseDown;
   if (ssLeft in Shift) then
    FMouseLoc := mlDown;
  end
 else
  FSBut := ssNone;
end;

procedure TmbTBXCustomClipartScrollBox.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
 R: TRect;
begin
 if Button = mbLeft then
  begin
   SetFocus;
   FMouseDown := true;
   FMouseLoc := mlDown;
   R := Rect(width - (GetSystemMetrics(SM_CXHSCROLL) - 2), 0, width, height);
   if PtInRect(R, Point(x, y)) then
    begin
     if y > height div 2 then
      FSBut := ssDown
     else
      FSBut := ssUp;
     if not FTimer.Enabled then
      FTimer.Enabled := true;
    end
   else
    FSBut := ssNone;
   Invalidate;
  end;
end;

procedure TmbTBXCustomClipartScrollBox.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
 R: TRect;
begin
 if Button = mbLeft then
  begin
   FMouseDown := false;
   R := Rect(width - (GetSystemMetrics(SM_CXHSCROLL) - 2), 0, width, height);
   if not Focused then
    begin
     if FMouseOver then
      FMouseLoc := mlOver
     else
      FMouseLoc := mlNone;
    end;
   if PtInRect(R, Point(x, y)) then
    begin
     if y > Height div 2 then
      DownClick
     else
      UpClick;
    end;
   FTimer.Enabled := false;
   FSbut := ssNone;
   Invalidate;
  end;
end;

procedure TmbTBXCustomClipartScrollBox.UpClick;
begin
 if FImages <> nil then
  begin
   if not FInverse then
    begin
     if Floop then
      if FImageIndex = FImages.Count - 1 then
       FImageIndex := 0;
     if FImageIndex < FImages.Count - 1 then
      Inc(FImageIndex, 1);
    end
   else
    begin
     if Floop then
      if FImageIndex = 0 then
       FImageIndex := FImages.Count - 1;
     if FImageIndex > 0 then
      Dec(FImageIndex, 1);
    end;
  end;
 if Assigned(FOnUpClick) then
  FOnUpClick(self);
 Invalidate;
end;

procedure TmbTBXCustomClipartScrollBox.DownClick;
begin
 if FImages <> nil then
  begin
   if not FInverse then
    begin
     if Floop then
      if FImageIndex = 0 then
       FImageIndex := FImages.Count - 1;
     if FImageIndex > 0 then
      Dec(FImageIndex, 1);
    end
   else
    begin
     if Floop then
      if FImageIndex = FImages.Count - 1 then
       FImageIndex := 0;
     if FImageIndex < FImages.Count - 1 then
      Inc(FImageIndex, 1);
    end;
  end;
 if Assigned(FOnDownClick) then
  FOnDownClick(self);
 Invalidate;
end;

procedure TmbTBXCustomClipartScrollBox.DoTimer(Sender: TObject);
begin
 case FSbut of
  ssUp:
   if FImageIndex < FImages.Count - 1 then
    UpClick
   else
    if FLoop then
     UpClick;
  ssDown:
   if FImageIndex > 0 then
    DownClick
   else
    if FLoop then
     DownClick;
 end;
end;

procedure TmbTBXCustomClipartScrollBox.CMEnabledChanged(var Message: TMessage);
begin
 inherited;
 Invalidate;
end;

procedure TmbTBXCustomClipartScrollBox.Notification(AComponent: TComponent; Operation: TOperation);
begin
 inherited Notification(AComponent, Operation);
 if Operation = opRemove then
  if AComponent = Images then
   SetImages(nil);
end;

procedure TmbTBXCustomClipartScrollBox.SetImages(i: TCustomImageList);
begin
 if FImages <> nil then
  FImages.UnRegisterChanges(FImageChangeLink);
 FImages := i;
 if FImages <> nil then
  begin
   FImages.RegisterChanges(FImageChangeLink);
   FImages.FreeNotification(Self);
  end;
 Invalidate;
end;

procedure TmbTBXCustomClipartScrollBox.ImageListChange(Sender: TObject);
begin
 if Sender = Images then
  Invalidate;
end;

procedure TmbTBXCustomClipartScrollBox.SetImageIndex(i: TImageIndex);
begin
 FImageIndex := i;
 Invalidate;
end;

procedure TmbTBXCustomClipartScrollBox.SetFAutoSize(a: boolean);
begin
 FAutoSize := a;
 Invalidate;
end;

procedure TmbTBXCustomClipartScrollBox.SetCenter(c: boolean);
begin
 FCenter := c;
 Invalidate;
end;

procedure TmbTBXCustomClipartScrollBox.CMGotFocus(var Message: TCMGotFocus);
begin
 inherited;
 FMouseLoc := mlOver;
 Invalidate;
end;

procedure TmbTBXCustomClipartScrollBox.CMLostFocus(var Message: TCMLostFocus);
begin
 inherited;
 if FMouseOver then
  FMouseLoc := mlOver
 else
  FMouseLoc := mlNone;
 Invalidate;
end;

procedure TmbTBXCustomClipartScrollBox.SetInterval(i: integer);
begin
 FInterval := i;
 FTimer.Interval := FInterval;
end;

procedure TmbTBXCustomClipartScrollBox.KeyDown(var Key: Word; Shift: TShiftState);
begin
 if Key = VK_DOWN then
  FSBut := ssDown;
 if Key = VK_UP then
  FSBut := ssUp;
 Invalidate;
 if not FTimer.Enabled then
  FTimer.Enabled := true;
 inherited KeyDown(Key, Shift);
end;

procedure TmbTBXCustomClipartScrollBox.WMEraseBkgnd(var Message: TWMEraseBkgnd);
begin
 Message.Result := 1;
end;

procedure TmbTBXCustomClipartScrollBox.SetHotTrack(Value: boolean);
begin
 if FHotTrack <> Value then
  begin
   FHotTrack := Value;
   Invalidate;
  end;
end;

end.
