unit mbTBXColorBox;

{$I mxs.inc}

interface

uses
  Messages, SysUtils, Classes, Controls, Windows, TBX, TBXUtils, TBXThemes,
  Graphics, ExtCtrls;

type
  TMouseLoc = (mlNone, mlOver, mlDown);

  TmbTBXCustomColorBox = class(TCustomColorBox)
  private
   FMouseLoc: TMouseLoc;
   FMouseOver: boolean;
   FThemed: boolean;
   FHotTrack: boolean;

   procedure SetHotTrack(Value: boolean);
   procedure SetThemed(Value: boolean);
   procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
   procedure TBMThemeChange(var Message: TMessage); message TBM_THEMECHANGE;
   procedure SetItemInfo(state: string; var IInfo: TTBXItemInfo; var EInfo: TTBXEditInfo);
  protected
   procedure CMGotFocus(var Message: TCMGotFocus);
   procedure CMLostFocus(var Message: TCMLostFocus);
   procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
   procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
   procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
   procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
   procedure DropDown; override;
   procedure CloseUp; override;
   procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
  public
   constructor Create(AOwner: TComponent); override;
   destructor Destroy; override;

   property Themed: boolean read FThemed write SetThemed default true;
   property HotTracking: boolean read FHotTrack write SetHotTrack default true;
  end;

  TmbTBXColorBox = class(TmbTBXCustomColorBox)
  published
   property AutoComplete;
   property AutoDropDown;
   property DefaultColorColor;
   property NoneColorColor;
   property Selected;
   property Style;
   property Anchors;
   property BiDiMode;
   property Color;
   property Constraints;
   property DropDownCount;
   property Enabled;
   property Font;
   property ItemHeight;
   property ParentBiDiMode;
   property ParentFont;
   property ParentShowHint;
   property ParentColor default false;
   property PopupMenu;
   property ShowHint;
   property TabOrder;
   property TabStop;
   property Visible;
   property Themed;
   property HotTracking;

   property OnChange;
   property OnCloseUp;
   property OnClick;
   property OnContextPopup;
   property OnDblClick;
   property OnDragDrop;
   property OnDragOver;
   property OnDropDown;
   property OnEndDock;
   property OnEndDrag;
   property OnEnter;
   property OnExit;
   property OnKeyDown;
   property OnKeyPress;
   property OnKeyUp;
   property OnSelect;
   property OnStartDock;
   property OnStartDrag;
  end;

implementation

constructor TmbTBXCustomColorBox.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
 ParentColor := false;
 AddThemeNotification(Self);
 FMouseLoc := mlNone;
 FMouseOver := false;
 FThemed := true;
 FHotTrack := true;
end;

destructor TmbTBXCustomColorBox.Destroy;
begin
 inherited Destroy;
 RemoveThemeNotification(Self);
end;

procedure TmbTBXCustomColorBox.WMPaint(var Message: TWMPaint);
function ColorToBorderColor(AColor: TColor): TColor;
  type
    TColorQuad = record
      Red, Green, Blue, Alpha: Byte;
    end;
  begin
    if (TColorQuad(AColor).Red > 192) or (TColorQuad(AColor).Green > 192) or (TColorQuad(AColor).Blue > 192) then
     Result := clBlack
    else
     Result := AColor;
  end;
var
  R: TRect;
  ii: TTBXItemInfo;
  ed: TTBXEditInfo;
begin
  inherited;
  if not FThemed then Exit;
  GetWindowRect(Handle, R);
  OffsetRect(R, -R.Left, -R.Top);
  ExcludeClipRect(Canvas.Handle, 2, 2, R.Right - (2 + GetSystemMetrics(SM_CXHSCROLL)), R.Bottom - 2);
  // set info
  case FMouseLoc of
   mlNone: SetItemInfo('normal', ii, ed);
   mlOver:
    if FHotTrack then
    SetItemInfo('over', ii, ed)
   else
    SetItemInfo('normal', ii, ed);
   mlDown: SetItemInfo('down', ii, ed);
  end;
  if Focused and (FMouseLoc <> mlDown) then SetItemInfo('over', ii, ed);
  if (csDesigning in ComponentState) or (Enabled = false) then SetItemInfo('normal', ii, ed);
  ii.Enabled := Enabled;
  // paint the border and button
  CurrentTheme.PaintFrame(Canvas, R, ii);
  CurrentTheme.PaintBackgnd(Canvas, R, R, R, CurrentTheme.GetViewColor(TVT_NORMALTOOLBAR), false, TVT_NORMALTOOLBAR);
  CurrentTheme.PaintDock(Canvas, R, R, DP_TOP);
  if CurrentTheme.PaintDockBackground then
   CurrentTheme.PaintBackgnd(Canvas, R, R, R, CurrentTheme.GetViewColor(TVT_NORMALTOOLBAR), false, TVT_NORMALTOOLBAR)
  else
   CurrentTheme.PaintDock(Canvas, R, R, DP_TOP);
  CurrentTheme.PaintEditFrame(Canvas, R, ii, ed);
end;

procedure TmbTBXCustomColorBox.SetItemInfo(state: string; var IInfo: TTBXItemInfo; var EInfo: TTBXEditInfo);
begin
   FillChar(IInfo, SizeOf(TTBXItemInfo), 0);
   IInfo.ViewType := TVT_NORMALTOOLBAR;
   IInfo.ItemOptions := IO_APPACTIVE or IO_TOOLBARSTYLE;
   IInfo.Enabled := True;

   FillChar(EInfo, SizeOf(TTBXEditInfo), 0);
   EInfo.RightBtnWidth := GetSystemMetrics(SM_CXHSCROLL);
   EInfo.RightBtnInfo.ButtonType := EBT_DROPDOWN;

   if state = 'normal' then
   begin
     IInfo.Pushed := false;
     IInfo.Selected := false;
     IInfo.HoverKind := hkNone;
     EInfo.RightBtnInfo.ButtonState := 0;
   end;
   if state = 'over' then
   begin
     IInfo.Pushed := false;
     IInfo.Selected := false;
     IInfo.HoverKind := hkMouseHover;
     EInfo.RightBtnInfo.ButtonState := EBDS_HOT;
   end;
   if state = 'down' then
   begin
     IInfo.Pushed := true;
     IInfo.Selected := false;
     IInfo.HoverKind := hkMouseHover;
     EInfo.RightBtnInfo.ButtonState := EBDS_PRESSED;
   end;
end;

procedure TmbTBXCustomColorBox.TBMThemeChange(var Message: TMessage);
begin
if Message.WParam = TSC_VIEWCHANGE then
 begin
  FMouseLoc := mlNone;
  Invalidate;
 end;
end;

procedure TmbTBXCustomColorBox.CMMouseEnter(var Message: TMessage);
begin
FMouseOver := true;
 if (not DroppedDown) and (not Focused) then
  begin
   FMouseLoc := mlOver;
   if FHotTrack then
   Invalidate;
  end;
end;

procedure TmbTBXCustomColorBox.CMMouseLeave(var Message: TMessage);
begin
FMouseOver := false;
 if (not DroppedDown) and (not Focused) then
  begin
   FMouseLoc := mlNone;
   if FHotTrack then
   Invalidate;
  end;
end;

procedure TmbTBXCustomColorBox.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 FMouseLoc := mlDown;
 Invalidate;
end;

procedure TmbTBXCustomColorBox.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 if (not DroppedDown) and (not Focused) then
  begin
   if FMouseOver then
    FMouseLoc := mlOver
   else
    FMouseLoc := mlNone;
   Invalidate;
  end;
end;

procedure TmbTBXCustomColorBox.DropDown;
begin
 inherited;
 if FMouseLoc <> mlDown then
  begin
   FMouseLoc := mlDown;
   Invalidate;
  end;
 if Assigned(OnDropDown) then
  OnDropDown(Self);
end;

procedure TmbTBXCustomColorBox.CloseUp;
begin
 inherited;
 if FMouseOver then
  FMouseLoc := mlOver
 else
  FMouseLoc := mlNone;
 Invalidate;
end;

procedure TmbTBXCustomColorBox.CMGotFocus(var Message: TCMGotFocus);
begin
 inherited;
 FMouseLoc := mlOver;
 Invalidate;
end;

procedure TmbTBXCustomColorBox.CMLostFocus(var Message: TCMLostFocus);
begin
 inherited;
 FMouseLoc := mlNone;
 Invalidate;
end;

procedure TmbTBXCustomColorBox.CMEnabledChanged(var Message: TMessage);
begin
 inherited;
 Invalidate;
end;

procedure TmbTBXCustomColorBox.SetThemed(Value: boolean);
begin
 FThemed := Value;
 Invalidate;
end;

procedure TmbTBXCustomColorBox.SetHotTrack(Value: boolean);
begin
 if FHotTrack <> Value then
  begin
   FHotTrack := Value;
   Invalidate;
  end;
end;

end.
