unit mbTBXColorPalette;

{$I mxs.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Graphics, TBX, TBXThemes, Forms,
  StrUtils,
 {$IFDEF DELPHI_7_UP}
  Themes,
 {$ENDIF}
  RGBHSLUtilsLite;

type
  TMouseLoc = (mlNone, mlOver, mlDown);
  TColorCellState = (ccsNone, ccsOver, ccsDown, ccsChecked, ccsCheckedHover);
  TMoveDirection = (mdLeft, mdRight, mdUp, mdDown);
  TSortMode = (smRed, smGreen, smBlue, smHue, smSaturation, smLuminance, smNone);
  TSortOrder = (soAscending, soDescending);

  TmbTBXColorPalette = class(TCustomControl)
  private
   FMouseLoc: TMouseLoc;
   FMouseOver, FMouseDown, FAutoHeight: boolean;
   FColCount, FRowCount, FTop, FLeft, FIndex, FCheckedIndex, FCellSize, FTotalCells: integer;
   FTempBmp, PBack: TBitmap;
   FState: TColorCellState;
   FColors: TStrings;
   FPalette: TFileName;
   FHintFormat: string;
   NB, OB, DB, CB, CHB, EB: TBitmap;
   FOnChange, FOnColorsChange: TNotifyEvent;
   FMinColors, FMaxColors: integer;
   FSort: TSortMode;
   FOrder: TSortOrder;
   FOld: TColor;

   function GetMoveCellIndex(move: TMoveDirection): integer;
   function GetSelColor: TColor;
   function FormatHint(c: TColor): string;
   procedure SetCellSize(s: integer);
   procedure SetSortMode(s: TSortMode);
   procedure SetSortOrder(s: TSortOrder);
   procedure SetMinColors(m: integer);
   procedure SetMaxColors(m: integer);
   procedure SetAutoHeight(auto: boolean);
   procedure ProcessPalette(p: TStrings);
   procedure LoadPalette(FileName: TFileName);
   procedure SetStrings(s: TStrings);
   procedure SetSelColor(k: TColor);
  protected
   procedure ColorsChange(Sender: TObject);
   procedure Paint; override;
   procedure DrawCell(c: TColor);
   procedure TBMThemeChange(var Message: TMessage); message TBM_THEMECHANGE;
   procedure WMEraseBkgnd(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
   procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
   procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
   procedure CNKeyDown(var Message: TWMKeyDown); message CN_KEYDOWN;
   procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
   procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
   procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
   procedure CMGotFocus(var Message: TCMGotFocus); message CM_ENTER;
   procedure CMLostFocus(var Message: TCMLostFocus); message CM_EXIT;
   procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
   procedure CMHintShow(var Message: TMessage); message CM_HINTSHOW;
   procedure Resize; override;
   procedure SelectCell(i: integer);
   {$IFDEF DELPHI_7_UP}
   procedure PaintParentBack;
   {$ENDIF}
   procedure PrepareBmps;
   procedure SortColors;
   procedure CreateWnd; override;
  public
   constructor Create(AOwner: TComponent); override;
   destructor Destroy; override;
   function GetColorUnderCursor: TColor;
   function ColorToHex(AColor: TColor): string;
   function HexToTColor(s: string): TColor;
   function GetSelectedCellRect: TRect;

   property RowCount: integer read FRowCount;
   property ColCount: integer read FColCount;
   procedure SaveColorsAsPalette(FileName: TFileName);
   procedure GeneratePalette(BaseColor: TColor);
   procedure GenerateGradientPalette(Colors: array of TColor);
  published
   property Align;
   property Anchors;
   property Enabled;
   property SortMode: TSortMode read FSort write SetSortMode default smNone;
   property SortOrder: TSortOrder read FOrder write SetSortOrder default soAscending;
   property MinColors: integer read FMinColors write SetMinColors default 0;
   property MaxColors: integer read FMaxColors write SetMaxColors default 0;
   property SelectedCell: integer read FCheckedIndex write SelectCell default -1;
   property SelectedColor: TColor read GetSelColor write SetSelColor default clNone;
   property Colors: TStrings read FColors write SetStrings;
   property Palette: TFileName read FPalette write LoadPalette;
   property HintFormat: string read FHintFormat write FHintFormat;
   property AutoHeight: boolean read FAutoHeight write SetAutoHeight default false;
   property CellSize: integer read FCellSize write SetCellSize default 18;
   {$IFDEF DELPHI_7_UP}
   property ParentBackground default true;
   {$ENDIF}
   property TabStop default true;
   property TabOrder;
   property ShowHint default false;
   property Constraints;
   property Color;
   property ParentColor;
   property ParentShowHint default true;
   property PopupMenu;
   property Visible;

   property OnSelColorChange: TNotifyEvent read FOnChange write FOnChange;
   property OnColorsChange: TNotifyEvent read FOnColorsChange write FOnColorsChange;
   property OnContextPopup;
   property OnMouseMove;
   property OnMouseDown;
   property OnMouseUp;
   property OnKeyDown;
   property OnKeyUp;
   property OnKeyPress;
   property OnResize;
  end;

implementation

procedure SetItemInfo(state: string; var IInfo: TTBXItemInfo);
begin
   FillChar(IInfo, SizeOf(TTBXItemInfo), 0);
   IInfo.ViewType := VT_TOOLBAR or TVT_EMBEDDED;
   IInfo.ItemOptions := IO_TOOLBARSTYLE or IO_APPACTIVE;
   IInfo.Enabled := True;

   if state = 'normal' then
    begin
     IInfo.Pushed := false;
     IInfo.Selected := false;
     IInfo.HoverKind := hkNone;
    end;
   if state = 'over' then
    begin
     IInfo.Pushed := false;
     IInfo.Selected := false;
     IInfo.HoverKind := hkMouseHover;
    end;
   if state = 'down' then
    begin
     IInfo.Pushed := true;
     IInfo.Selected := true;
     IInfo.HoverKind := hkMouseHover;
    end;
   if state = 'checked' then
    begin
     IInfo.Pushed := false;
     IInfo.Selected := true;
     IInfo.HoverKind := hkNone;
    end;
   if state = 'checked-hover' then
    begin
     IInfo.Pushed := false;
     IInfo.Selected := true;
     IInfo.HoverKind := hkMouseHover;
    end;
end;

constructor TmbTBXColorPalette.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
 AddThemeNotification(Self);
 {$IFDEF DELPHI_7_UP}
 ControlStyle := ControlStyle - [csAcceptsControls] + [csOpaque, csParentBackground];
 {$ELSE}
 ControlStyle := ControlStyle - [csAcceptsControls] + [csOpaque];
 {$ENDIF}
 DoubleBuffered := true;
 TabStop := true;
 ParentShowHint := true;
 ShowHint := false;
 FMouseLoc := mlNone;
 FMouseOver := false;
 FMouseDown := false;
 FColCount := 0;
 FRowCount := 0;
 FIndex := -1;
 FCheckedIndex := -1;
 FTop := 0;
 FLeft := 0;
 FCellSize := 18;
 FState := ccsNone;
 FColors := TStringList.Create;
 (FColors as TStringList).OnChange := ColorsChange;
 FTotalCells := 0;
 FHintFormat := 'RGB(%r, %g, %b)'#13'Hex: %h';
 FAutoHeight := false;
 FMinColors := 0;
 FMaxColors := 0;
 FSort := smNone;
 FOrder := soAscending;
 FOld := clNone;
 PrepareBmps;
 height:=50;
 Width:=50;
 PBack := TBitmap.Create;
 PBack.PixelFormat := pf32bit;
end;

destructor TmbTBXColorPalette.Destroy;
begin
 NB.Free;
 OB.Free;
 DB.Free;
 CB.Free;
 CHB.Free;
 EB.Free;
 RemoveThemeNotification(Self);
 PBack.Free;
 FColors.Free;
 inherited Destroy;
end;

procedure TmbTBXColorPalette.CreateWnd;
begin
 inherited;
 FColCount := Width div FCellSize;
 if FAutoHeight and (FColCount <> 0) then
  begin
   if FColors.Count mod FColCount > 0 then
    Height := (FColors.Count div FColCount + 1) * FCellSize
   else
    Height := (FColors.Count div FColCount) * FCellSize;
  end;
 if Height = 0 then Height := FCellSize;
 FRowCount := Height div FCellSize;
 Width := FColCount * FCellSize;
 PrepareBmps;
 {$IFDEF DELPHI_7_UP}
 PaintParentBack;
 {$ENDIF}
 Invalidate;
end;

procedure TmbTBXColorPalette.PrepareBmps;
var
 ii: TTBXItemInfo;
begin
 //prepare bitmaps here to avoid calling PaintButton for every cell, this is faster
 //create BMPs
 NB := TBitmap.Create;
 OB := TBitmap.Create;
 DB := TBitmap.Create;
 CB := TBitmap.Create;
 CHB := TBitmap.Create;
 EB := TBitmap.Create;
 //set size
 NB.Width := FCellSize;
 NB.Height := FCellSize;
 OB.Width := FCellSize;
 OB.Height := FCellSize;
 DB.Width := FCellSize;
 DB.Height := FCellSize;
 CB.Width := FCellSize;
 CB.Height := FCellSize;
 CHB.Width := FCellSize;
 CHB.Height := FCellSize;
 EB.Width := FCellSize;
 EB.Height := FCellSize;
 //set pixel format
 NB.PixelFormat := pf24bit;
 OB.PixelFormat := pf24bit;
 DB.PixelFormat := pf24bit;
 CB.PixelFormat := pf24bit;
 CHB.PixelFormat := pf24bit;
 EB.PixelFormat := pf24bit;
 //draw
 ii.Enabled := true;
 SetItemInfo('normal', ii);
 CurrentTheme.PaintButton(NB.Canvas, NB.Canvas.ClipRect, ii);
 SetItemInfo('over', ii);
 CurrentTheme.PaintButton(OB.Canvas, OB.Canvas.ClipRect, ii);
 SetItemInfo('down', ii);
 CurrentTheme.PaintButton(DB.Canvas, DB.Canvas.ClipRect, ii);
 SetItemInfo('checked', ii);
 CurrentTheme.PaintButton(CB.Canvas, CB.Canvas.ClipRect, ii);
 SetItemInfo('checked-hover', ii);
 CurrentTheme.PaintButton(CHB.Canvas, CHB.Canvas.ClipRect, ii);
 ii.Enabled := false;
 SetItemInfo('normal', ii);
 CurrentTheme.PaintButton(EB.Canvas, EB.Canvas.ClipRect, ii); 
end;

procedure TmbTBXColorPalette.TBMThemeChange(var Message: TMessage);
begin
if Message.WParam = TSC_VIEWCHANGE then
 begin
  PrepareBmps;
  Invalidate;
 end;
end;

{$IFDEF DELPHI_7_UP}
procedure TmbTBXColorPalette.PaintParentBack;
var
 MemDC: HDC;
 OldBMP: HBITMAP;
begin
 if PBack = nil then
  begin
   PBack := TBitmap.Create;
   PBack.PixelFormat := pf32bit;
  end;
 PBack.Width := Width;
 PBack.Height := Height;
 PBack.Canvas.Brush.Color := Color;
 PBack.Canvas.FillRect(PBack.Canvas.ClipRect);
 if ParentBackground then
  with ThemeServices do
   if ThemesEnabled then
    begin
     MemDC := CreateCompatibleDC(0);
     OldBMP := SelectObject(MemDC, PBack.Handle);
     DrawParentBackground(Handle, MemDC, nil, False);
     if OldBMP <> 0 then SelectObject(MemDC, OldBMP);
     if MemDC <> 0 then DeleteDC(MemDC);
    end;
end;
{$ENDIF}

procedure TmbTBXColorPalette.Paint;
var
 i: integer;
 ii: TTBXItemInfo;
begin
  // ATTN:
  {$IFDEF DELPHI_7_UP}
   PaintParentBack;
  {$ENDIF}
 //make bitmap
 FTempBmp := TBitmap.Create;
 try
  FTempBmp.PixelFormat := pf32bit;
  FTempBmp.Width := Width;
  FTempBmp.Height := Height;
  SetItemInfo('normal', ii);
  FTotalCells := FColors.Count - 1;
  //reset counters
  FTop := 0;
  FLeft := 0;

  //Fill back
  {$IFDEF DELPHI_7_UP}
  if ParentBackground then
   FTempBmp.Canvas.Draw(0, 0, PBack)
  else
  {$ENDIF}
   begin
    FTempBmp.Canvas.Brush.Color := Color;
    FTempBmp.Canvas.FillRect(FTempBmp.Canvas.ClipRect);
   end;
  //draw the cells
  for i := 0 to FColors.Count - 1 do
   begin
    if FColors.Strings[i] <> '' then
    DrawCell(StringToColor(FColors.Strings[i]));
    Inc(FLeft);
   end;
  //draw the result
  Canvas.Draw(0, 0, FTempBmp);
  //csDesiginng border
  if csDesigning in ComponentState then
   begin
    Canvas.Brush.Style := bsClear;
    Canvas.Pen.Style := psDot;
    Canvas.Pen.Color := clBtnShadow;
    Canvas.Rectangle(ClientRect);
    Canvas.Brush.Style := bsSolid;
    Canvas.Pen.Style := psSolid;
   end;
 finally
  FTempBmp.Free;
 end;
end;

procedure TmbTBXColorPalette.DrawCell(c: TColor);
var
 R: Trect;
 FCurrentIndex: integer;
begin
 // set props
 if (FLeft + 1) * FCellSize > FTempBmp.width then
  begin
   Inc(FTop);
   FLeft := 0;
  end;
 FCurrentIndex := FTop * FColCount + FLeft;
 R := Rect(FLeft * FCellSize, FTop * FCellSize, (FLeft + 1) * FCellSize, (FTop + 1) * FCellSize);
 //start drawing
 with FTempBmp.Canvas do
  begin
   Brush.Color := Color;
   //get current state
   if FCurrentIndex = FCheckedIndex then
    begin
     if FCheckedIndex = FIndex then
      begin
       if FMouseDown then
        FState := ccsDown
       else
        FState := ccsCheckedHover;
      end
     else
      FState := ccsChecked;
    end
   else
    if FIndex = FCurrentIndex then
     case FMouseLoc of
      mlNone: FState := ccsNone;
      mlOver: FState := ccsOver;
     end
    else
     FState := ccsNone;
   //paint
   if Enabled then
    begin
     if csDesigning in ComponentState then
      FTempBmp.Canvas.Draw(R.Left, R.Top, NB)
     else
      case FState of
       ccsNone: FTempBmp.Canvas.Draw(R.Left, R.Top, NB);
       ccsOver: FTempBmp.Canvas.Draw(R.Left, R.Top, OB);
       ccsDown: FTempBmp.Canvas.Draw(R.Left, R.Top, DB);
       ccsChecked: FTempBmp.Canvas.Draw(R.Left, R.Top, CB);
       ccsCheckedHover: FTempBmp.Canvas.Draw(R.Left, R.Top, CHB);
      end;
    end
   else
    FTempBmp.Canvas.Draw(R.Left, R.Top, EB);
   // draw fill
   InflateRect(R, -3, -3);
   if Enabled then
    begin
     Brush.Color := c;
     Pen.Color := clBtnShadow;
    end
   else
    begin
     Brush.Color := clGray;
     Pen.Color := clGray;
    end;
   Rectangle(R);
  end;
end;

procedure TmbTBXColorPalette.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
 inherited;
 if FIndex <> (y div FCellSize)* FColCount + (x div FCellSize) then
  begin
   FIndex := (y div FCellSize)* FColCount + (x div FCellSize);
   if FIndex > FTotalCells then FIndex := -1;
   Invalidate;
  end;
end;

procedure TmbTBXColorPalette.Resize;
begin
 inherited;
 FColCount := Width div FCellSize;
 if FAutoHeight and (FColCount <> 0)  then
  begin
   if FColors.Count mod FColCount > 0 then
    Height := (FColors.Count div FColCount + 1) * FCellSize
   else
    Height := (FColors.Count div FColCount) * FCellSize;
  end;
 if Height = 0 then Height := FCellSize;
 FRowCount := Height div FCellSize;
 Width := FColCount * FCellSize;
 {$IFDEF DELPHI_7_UP}
 PaintParentBack;
 {$ENDIF}
 Invalidate;
end;

procedure TmbTBXColorPalette.CMMouseEnter(var Message: TMessage);
begin
 FMouseOver := true;
 FMouseLoc := mlOver;
 Invalidate;
 inherited;
end;

procedure TmbTBXColorPalette.CMMouseLeave(var Message: TMessage);
begin
 FMouseOver := false;
 FMouseLoc := mlNone;
 FIndex := -1;
 Invalidate;
 inherited;
end;

procedure TmbTBXColorPalette.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
if Button = mbLeft then
 begin
  FMouseLoc := mlDown;
  FMouseDown := true;
  if (y div FCellSize)* FColCount + (x div FCellSize) <= FTotalCells then
   if FCheckedIndex <> (y div FCellSize)* FColCount + (x div FCellSize) then
    FCheckedIndex := (y div FCellSize)* FColCount + (x div FCellSize);
  SetFocus;
  Invalidate;
 end;
inherited;
end;

procedure TmbTBXColorPalette.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 FMouseDown := false;
 if FMouseOver then
  FMouseLoc := mlOver
 else
  FMouseLoc := mlNone;
 Invalidate;
 inherited;
 if Assigned(FOnChange) then FOnChange(Self);
end;

procedure TmbTBXColorPalette.CMGotFocus(var Message: TCMGotFocus);
begin
 inherited;
 Invalidate;
end;

procedure TmbTBXColorPalette.CMLostFocus(var Message: TCMLostFocus);
begin
 inherited;
 if FMouseOver then
  FMouseLoc := mlOver
 else
  FMouseLoc := mlNone;
 Invalidate;
end;

procedure TmbTBXColorPalette.CMEnabledChanged(var Message: TMessage);
begin
 inherited;
 Invalidate;
end;

procedure TmbTBXColorPalette.WMEraseBkgnd(var Message: TWMEraseBkgnd);
begin
 Message.Result := 1;
end;

procedure TmbTBXColorPalette.SelectCell(i: integer);
begin
 if i < FColors.Count - 1 then
  FCheckedIndex := i
 else
  FCheckedIndex := -1;
 Invalidate;
 if Assigned(FOnChange) then FOnChange(Self);
end;

function TmbTBXColorPalette.GetSelColor: TColor;
begin
 if FCheckedIndex > -1 then
  Result := StringToColor(FColors.Strings[FCheckedIndex])
 else
  Result := FOld;
end;

function TmbTBXColorPalette.GetColorUnderCursor: TColor;
begin
 Result := clNone;
 if FIndex > -1 then
  if FIndex < FColors.Count then
   Result := StringToColor(FColors.Strings[FIndex]);
end;

procedure TmbTBXColorPalette.SetSelColor(k: TColor);
var
 s: string;
 i: integer;
begin
 s := ColorToString(k);
 for i:= 0 to FColors.Count - 1 do
  if SameText(s, FColors.Strings[i]) then
   begin
    FCheckedIndex := i;
    Break;
   end
  else
   FCheckedIndex := -1;
 Invalidate;
 FOld := k;
 if Assigned(FOnChange) then FOnChange(Self);
end;

procedure TmbTBXColorPalette.SetStrings(s: TStrings);
var
 i: integer;
begin
 FColors.Clear;
 FColors.AddStrings(s);
 if FColors.Count < FMinColors then
  for i := 0 to FMinColors - FColors.Count - 1 do
   FColors.Add('clNone');
 if (FColors.Count > FMaxColors) and (FMaxColors > 0) then
  for i := FColors.Count - 1 downto FMaxColors do
   FColors.Delete(i);
 if FAutoHeight and (FColCount <> 0) then
  begin
   if FColors.Count mod FColCount > 0 then
    Height := (FColors.Count div FColCount + 1) * FCellSize
   else
    Height := (FColors.Count div FColCount) * FCellSize;
  end;
 if Height = 0 then Height := FCellSize;
 SortColors;
 Invalidate;
end;

procedure TmbTBXColorPalette.LoadPalette(FileName: TFileName);
var
 p: TStrings;
begin
 FPalette := FileName;
 if FileName <> '' then
  begin
   p := TStringList.Create;
   p.LoadFromFile(FPalette);
   ProcessPalette(p);
   if FAutoHeight and (FColCount <> 0) then
    begin
     if FColors.Count mod FColCount > 0 then
      Height := (FColors.Count div FColCount + 1) * FCellSize
     else
      Height := (FColors.Count div FColCount) * FCellSize;
    end;
   if Height = 0 then Height := FCellSize;
   SortColors;
   Invalidate;
  end;
end;

procedure TmbTBXColorPalette.ProcessPalette(p: TStrings);
var
 i: integer;
 t: TStrings;
begin
 FColors.Clear;
 t := TStringList.Create;
 for i := 0 to p.Count - 1 do
  if p.strings[i] <> '' then
   begin
    t.Clear;
    ExtractStrings([' '], [], PChar(p.strings[i]), t);
    if t.Count = 3 then
     FColors.Add(ColorToString(RGB(StrToInt(t.strings[0]), StrToInt(t.strings[1]), StrToInt(t.strings[2]))));
   end;
 if FColors.Count < FMinColors then
  for i := 0 to FMinColors - FColors.Count - 1 do
   FColors.Add('clNone');
 if (FColors.Count > FMaxColors) and (FMaxColors > 0) then
  for i := FColors.Count - 1 downto FMaxColors do
   FColors.Delete(i);
 t.Free;
end;

function TmbTBXColorPalette.GetMoveCellIndex(move: TMoveDirection): integer;
var
 FBefore: integer;
begin
 Result := -1;
 case move of
  mdLeft:
   if FCheckedIndex -1 < 0 then
    Result := FTotalCells
   else
    Result := FCheckedIndex - 1;
  mdRight:
   if FCheckedIndex + 1 > FTotalCells then
    Result := 0
   else
    Result := FCheckedIndex + 1;
  mdUp:
   if FCheckedIndex - FColCount < 0 then
    begin
     FBefore := (FTotalcells div FColCount) * FColCount;
     if FBefore + FCheckedIndex - 1 > FTotalCells then Dec(FBefore, FColCount);
     Result := FBefore + FCheckedIndex - 1;
    end
   else
    Result := FCheckedIndex - FColCount;
  mdDown:
   if FCheckedIndex + FColCount > FTotalCells then
    Result := FCheckedIndex mod FColCount + 1
   else
    Result := FCheckedIndex + FColCount;
 end;
end;

procedure TmbTBXColorPalette.CNKeyDown(var Message: TWMKeyDown);
var
 FInherited: boolean;
 Shift: TShiftState;
begin
 Shift := KeyDataToShiftState(Message.KeyData);
 Finherited := false;
 case Message.CharCode of
  VK_LEFT: FCheckedIndex := GetMoveCellIndex(mdLeft);
  VK_RIGHT: FCheckedIndex := GetMoveCellIndex(mdRight);
  VK_UP: FCheckedIndex := GetMoveCellIndex(mdUp);
  VK_DOWN: FCheckedIndex := GetMoveCellIndex(mdDown);
 else
  begin
   FInherited := true;
   inherited;
  end;
 end;
 if not FInherited then
  begin
   Invalidate;
   if Assigned(OnKeyDown) then OnKeyDown(Self, Message.CharCode, Shift);
   if Assigned(FOnChange) then FOnChange(Self);
  end;
end;

procedure TmbTBXColorPalette.CMHintShow(var Message: TMessage);
begin
if (Colors.Count > 0) and (FIndex > -1) then
 with TCMHintShow(Message) do
  begin
   if not ShowHint then
    Message.Result := 1
   else
    begin
     with HintInfo^ do
      begin
       // show that we want a hint
       Result := 0;
       ReshowTimeout := 1;
       HideTimeout := 5000;
       HintStr := FormatHint(GetColorUnderCursor);
      end;
    end;
  end;
end;

function TmbTBXColorPalette.ColorToHex(AColor: TColor): string;
begin
 if AColor <> $ then
  Result := IntToHex(GetRValue(AColor), 2) + IntToHex(GetGValue(AColor), 2) + IntToHex(GetBValue(AColor), 2)
 else
  Result := '000000';
end;

function TmbTBXColorPalette.HexToTColor(s: string): TColor;
begin
 {$IFDEF DELPHI_7_UP}
 AnsiReplaceText(s, '#', '');
 {$ELSE}
 Stringreplace(s, '#', '', [rfReplaceAll]);
 {$ENDIF}
 if s <> '' then
  Result := RGB(StrToInt('$'+Copy(S, 1, 2)), StrToInt('$'+Copy(S, 3, 2)), StrToInt('$'+Copy(S, 5, 2)))
 else
  Result := clNone;
end;

function TmbTBXColorPalette.FormatHint(c: TColor): string;
var
 s: string;
begin
 Result := '';
 {$IFDEF DELPHI_7_UP}
 s := AnsiReplaceText(FHintFormat, '%r', IntToStr(GetRValue(c)));
 s := AnsiReplaceText(s, '%g', IntToStr(GetGValue(c)));
 s := AnsiReplaceText(s, '%b', IntToStr(GetBValue(c)));
 s := AnsiReplaceText(s, '%h', ColorToHex(c));
 {$ELSE}
 s := Stringreplace(FHintFormat, '%r', IntToStr(GetRValue(c)),[rfReplaceAll]);
 s := Stringreplace(s, '%g', IntToStr(GetGValue(c)),[rfReplaceAll]);
 s := Stringreplace(s, '%b', IntToStr(GetBValue(c)),[rfReplaceAll]);
 s := Stringreplace(s, '%h', ColorToHex(c),[rfReplaceAll]);
 {$ENDIF}
 Result := s;
end;

procedure TmbTBXColorPalette.SaveColorsAsPalette(FileName: TFileName);
var
 i: integer;
 p: TStrings;
 c: TColor;
begin
 p := TStringList.Create;
 try
  p.Add('JASC-PAL');
  p.Add('0100');
  p.Add('256');
  for i := 0 to FColors.Count - 1 do
   if FColors.Strings[i] <> '' then
    begin
     c := StringToColor(FColors.Strings[i]);
     p.Add(IntToStr(GetRValue(c)) + ' ' + IntToStr(GetGValue(c)) + ' ' + IntToStr(GetBValue(c)));
    end;
  p.SaveToFile(FileName);
 finally
  p.Free;
 end;
end;

procedure TmbTBXColorPalette.SetAutoHeight(auto: boolean);
begin
 FAutoHeight := auto;
 FColCount := Width div FCellSize;
 if FAutoHeight and (FColCount <> 0) then
  begin
   if FColors.Count mod FColCount > 0 then
    Height := (FColors.Count div FColCount + 1) * FCellSize
   else
    Height := (FColors.Count div FColCount) * FCellSize;
  end;
 if Height = 0 then Height := FCellSize;
 FRowCount := Height div FCellSize;
 Invalidate;
end;

procedure TmbTBXColorPalette.SetMinColors(m: integer);
var
 i: integer;
begin
 if (FMaxColors > 0) and (m > FMaxColors) then
  m := FMaxColors;
 FMinColors := m;
 if FColors.Count < m then
  for i := 0 to m - FColors.Count - 1 do
   FColors.Add('clNone');
 Invalidate;
end;

procedure TmbTBXColorPalette.SetMaxColors(m: integer);
var
 i: integer;
begin
 if m < 0 then m := 0;
 FMaxColors := m;
 if (m < FMinColors) and (m > 0) then
  SetMinColors(m);
 if (FColors.Count > FMaxColors) and (FMaxColors > 0) then
  for i := FColors.Count - 1 downto FMaxColors do
   FColors.Delete(i);
 Invalidate;
end;

procedure TmbTBXColorPalette.SetSortMode(s: TSortMode);
begin
 FSort := s;
 SortColors;
 Invalidate;
end;

procedure TmbTBXColorPalette.SetSortOrder(s: TSortOrder);
begin
 FOrder := s;
 SortColors;
 Invalidate;
end;

procedure TmbTBXColorPalette.SortColors;

 function MaxPos(s: TStrings; sm: TSortMode): integer;
 var
  i: integer;
  first: TColor;
 begin
  Result := 0;
  first := clBlack;
  for i := 0 to s.Count - 1 do
   case sm of
    smRed:
     if GetRValue(first) < GetRValue(StringToColor(s.Strings[i])) then
      begin
       first := StringToColor(s.Strings[i]);
       Result := i;
      end;
    smGreen:
     if GetGValue(first) < GetGValue(StringToColor(s.Strings[i])) then
      begin
       first := StringToColor(s.Strings[i]);
       Result := i;
      end;
    smBlue:
     if GetBValue(first) < GetBValue(StringToColor(s.Strings[i])) then
      begin
       first := StringToColor(s.Strings[i]);
       Result := i;
      end;
    smHue:
     if GetHValue(first) < GetHValue(StringToColor(s.Strings[i])) then
      begin
       first := StringToColor(s.Strings[i]);
       Result := i;
      end;
    smSaturation:
     if GetSValue(first) < GetSValue(StringToColor(s.Strings[i])) then
      begin
       first := StringToColor(s.Strings[i]);
       Result := i;
      end;
    smLuminance:
     if GetLValue(first) < GetLValue(StringToColor(s.Strings[i])) then
      begin
       first := StringToColor(s.Strings[i]);
       Result := i;
      end;
   end;
 end;

 function MinPos(s: TStrings; sm: TSortMode): integer;
 var
  i: integer;
  first: TColor;
 begin
  Result := 0;
  first := clWhite;
  for i := 0 to s.Count - 1 do
   case sm of
    smRed:
     if GetRValue(first) > GetRValue(StringToColor(s.Strings[i])) then
      begin
       first := StringToColor(s.Strings[i]);
       Result := i;
      end;
    smGreen:
     if GetGValue(first) > GetGValue(StringToColor(s.Strings[i])) then
      begin
       first := StringToColor(s.Strings[i]);
       Result := i;
      end;
    smBlue:
     if GetBValue(first) > GetBValue(StringToColor(s.Strings[i])) then
      begin
       first := StringToColor(s.Strings[i]);
       Result := i;
      end;
    smHue:
     if GetHValue(first) > GetHValue(StringToColor(s.Strings[i])) then
      begin
       first := StringToColor(s.Strings[i]);
       Result := i;
      end;
    smSaturation:
     if GetSValue(first) > GetSValue(StringToColor(s.Strings[i])) then
      begin
       first := StringToColor(s.Strings[i]);
       Result := i;
      end;
    smLuminance:
     if GetLValue(first) > GetLValue(StringToColor(s.Strings[i])) then
      begin
       first := StringToColor(s.Strings[i]);
       Result := i;
      end;
   end;
 end;

var
 i, m: integer;
 s: TStrings;
 old: TColor;
begin
 if FSort <> smNone then
  begin
   if FColors.Count = 0 then Exit;
   old := GetSelColor;
   m := 0;
   s := TStringList.Create;
   s.AddStrings(FColors);
   FColors.Clear;
   for i := s.Count - 1 downto 0 do
    begin
     case FOrder of
      soAscending: m := MinPos(s, FSort);
      soDescending: m := MaxPos(s, FSort);
     end;
     FColors.Add(s.Strings[m]);
     s.Delete(m);
    end;
   SetSelColor(old);
   Invalidate;
   s.Free;
  end;
end;

procedure TmbTBXColorPalette.GeneratePalette(BaseColor: TColor);
var
 i: integer;
begin
 FColors.Clear;
 case FOrder of
  soAscending:
   for i := 239 downto 0 do
    FColors.Add(ColorToString(HSLRangeToRGB(GetHValue(BaseColor), GetSValue(BaseColor), 240 - i)));
  soDescending:
   for i := 0 to 239 do
    FColors.Add(ColorToString(HSLRangeToRGB(GetHValue(BaseColor), GetSValue(BaseColor), 240 - i)));
 end;
 Invalidate;
 if Assigned(FOnChange) then FOnChange(Self);
end;

procedure TmbTBXColorPalette.ColorsChange(Sender: TObject);
begin
 if Assigned(FOnColorsChange) then FOnColorsChange(Self);
 Invalidate;
end;

procedure TmbTBXColorPalette.SetCellSize(s: integer);
begin
 FCellSize := s;
 FColCount := Width div FCellSize;
 if FAutoHeight and (FColCount <> 0) then
  begin
   if FColors.Count mod FColCount > 0 then
    Height := (FColors.Count div FColCount + 1) * FCellSize
   else
    Height := (FColors.Count div FColCount) * FCellSize;
  end;
 if Height = 0 then Height := FCellSize;
 FRowCount := Height div FCellSize;
 Width := FColCount * FCellSize;
 PrepareBmps;
 Invalidate;
end;

function TmbTBXColorPalette.GetSelectedCellRect: TRect;
var
 row, fbottom, fleft: integer;
begin
 if FCheckedIndex > -1 then
  begin
   if FCheckedIndex mod FColCount = 0 then
    begin
     row := FCheckedIndex div FColCount;
     fleft := Width - FCellSize;
    end
   else
    begin
     row := FCheckedIndex div FColCount + 1;
     fleft := (FCheckedIndex mod FColCount - 1) * FCellSize;
    end;
   fbottom := row * FCellSize;
   Result := Rect(fleft, fbottom - FCellSize, fleft + FCellSize, fbottom);
  end
 else
  Result := Rect(0, 0, 0, 0);
end;

function IsMember(sl: TStringList; s: string): boolean;
var
 i: integer;
begin
 Result := false;
 for i := 0 to sl.count -1 do
  if sl.Strings[i] = s then
   Result := true;
end;

procedure TmbTBXColorPalette.GenerateGradientPalette(Colors: array of TColor);
type
  RGBArray = array[0..2] of Byte;
var
  i, j, k, Span: Integer;
  s: TStringList;
  Scolor: string;
  Faktor: double;
  a: RGBArray;
  b: array of RGBArray;
begin
  Span := 300;
  s := TStringList.Create;
  SetLength(b, High(Colors) + 1);
  for i := 0 to High(Colors) do
   begin
    Colors[i] := ColorToRGB(Colors[i]);
    b[i, 0] := GetRValue(Colors[i]);
    b[i, 1] := GetGValue(Colors[i]);
    b[i, 2] := GetBValue(Colors[i]);
   end;
  for i := 0 to High(Colors) - 1 do
   for j := 0 to Span do
    begin
     Faktor := j / Span;
     for k := 0 to 3 do
      a[k] := Trunc(b[i, k] + ((b[i + 1, k] - b[i, k]) * Faktor));
     Scolor := ColorToString(RGB(a[0], a[1], a[2]));
     if not IsMember(s, Scolor) then
      s.add(Scolor);
    end;
  SetStrings(s as TStrings);
  s.Free;
end;

end.
