unit mbTBXColorProvider;

{$I mxs.inc}

interface

uses
  SysUtils, Classes, TBX, TBXThemes, Graphics, Messages;

type
  TmbTBXColorProvider = class(TComponent)
  private
   ii: TTBXItemInfo;
   FToolbarBackColor: TColor;
   FPanelBackColor: TColor;
   FNormalItemColor: TColor;
   FHotItemColor: TColor;
   FDownItemColor: TColor;
   FNormalTextColor: TColor;
   FHotTextColor: TColor;
   FDownTextColor: TColor;
   FNormalImageBackColor: TColor;
   FHotImageBackColor: TColor;
   FDownImageBackColor: TColor;
   FNone: TColor; // null
   FOnThemeChange: TNotifyEvent;
   FCheckedItemColor: TColor;
   FHotCheckedItemColor: TColor;
   FCheckedImageBackColor: TColor;
   FHotCheckedImageBackColor: TColor;
   FCheckedTextColor: TColor;
   FHotCheckedTextColor: TColor;
   FDisabledColors: boolean;
   FDownCheckedItemColor: TColor;
   FDownCheckedTextColor: TColor;
   FDownCheckedImageBackColor: TColor;

   procedure SetDisabled(d: boolean);
   procedure SetItemInfo(state: string; var IInfo: TTBXItemInfo);
   procedure TBMThemeChange(var Message: TMessage); message TBM_THEMECHANGE;
   procedure UpdateColors;
  public
   constructor Create(AOwner: TComponent); override;
   destructor Destroy; override;
  published
   property DisabledColors: boolean read FDisabledColors write SetDisabled stored false default false;
   property CheckedItemColor: TColor read FCheckedItemColor write FNone stored false default clNone;
   property HotCheckedItemColor: TColor read FHotCheckedItemColor write FNone stored false default clNone;
   property CheckedImageBackColor: TColor read FCheckedImageBackColor write FNone stored false default clNone;
   property HotCheckedImageBackColor: TColor read FHotCheckedImageBackColor write FNone stored false default clNone;
   property CheckedTextColor: TColor read FCheckedTextColor write FNone stored false default clNone;
   property HotCheckedTextColor: TColor read FHotCheckedTextColor write FNone stored false default clNone;
   property ToolbarBackColor: TColor read FToolbarBackColor write FNone stored false default clNone;
   property PanelBackColor: TColor read FPanelBackColor write FNone stored false default clNone;
   property NormalItemColor: TColor read FNormalItemColor write FNone stored false default clNone;
   property HotItemColor: TColor read FHotItemColor write FNone stored false default clNone;
   property DownItemColor: TColor read FDownItemColor write FNone stored false default clNone;
   property NormalTextColor: TColor read FNormalTextColor write FNone stored false default clNone;
   property HotTextColor: TColor read FHotTextColor write FNone stored false default clNone;
   property DownTextColor: TColor read FDownTextColor write FNone stored false default clNone;
   property NormalImageBackColor: TColor read FNormalImageBackColor write FNone stored false default clNone;
   property HotImageBackColor: TColor read FHotImageBackColor write FNone stored false default clNone;
   property DownImageBackColor: TColor read FDownImageBackColor write FNone stored false default clNone;
   property DownCheckedItemColor: TColor read FDownCheckedItemColor write FNone stored false default clNone;
   property DownCheckedTextColor: TColor read FDownCheckedTextColor write FNone stored false default clNone;
   property DownCheckedImageBackColor: TColor read FDownCheckedImageBackColor write FNone stored false default clNone;

   property OnThemeChange: TNotifyEvent read FOnThemeChange write FOnThemeChange;
  end;

implementation

constructor TmbTBXColorProvider.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
 AddThemeNotification(Self);
 FNone := clNone; //null - used so properties show up in object inspector
 FToolbarBackColor:= clNone;
 FPanelBackColor:= clNone;
 FNormalItemColor:= clNone;
 FHotItemColor:= clNone;
 FDownItemColor:= clNone;
 FNormalTextColor:= clNone;
 FHotTextColor:= clNone;
 FDownTextColor:= clNone;
 FNormalImageBackColor:= clNone;
 FHotImageBackColor:= clNone;
 FDownImageBackColor:= clNone;
 FCheckedItemColor:= clNone;
 FHotCheckedItemColor:= clNone;
 FCheckedImageBackColor:= clNone;
 FHotCheckedImageBackColor:= clNone;
 FCheckedTextColor:= clNone;
 FHotCheckedTextColor:= clNone;
 FDownCheckedItemColor:= clNone;
 FDownCheckedTextColor:= clNone;
 FDownCheckedImageBackColor:= clNone;
 FDisabledColors := false;
 SetItemInfo('normal', ii);
 UpdateColors;
end;

destructor TmbTBXColorProvider.Destroy;
begin
 RemoveThemeNotification(Self);
 inherited Destroy;
end;

procedure TmbTBXColorProvider.TBMThemeChange(var Message: TMessage);
begin
if Message.WParam = TSC_VIEWCHANGE then
 begin
  UpdateColors;
  if Assigned(FOnThemeChange) then
   FOnThemeChange(Self);
 end;
end;

procedure TmbTBXColorProvider.SetDisabled(d: boolean);
begin
 FDisabledColors := d;
 UpdateColors;
end;

procedure TmbTBXColorProvider.UpdateColors;
begin
ii.Enabled := not FDisabledColors;

FToolbarBackColor:= CurrentTheme.GetViewColor(TVT_NORMALTOOLBAR);
FPanelBackColor:= CurrentTheme.GetViewColor(DPVT_NORMAL);

SetItemInfo('normal', ii);
FNormalItemColor:= CurrentTheme.GetItemColor(ii);
FHotItemColor:= CurrentTheme.GetItemTextColor(ii);
FDownItemColor:= CurrentTheme.GetItemImageBackground(ii);

SetItemInfo('over', ii);
FNormalTextColor:= CurrentTheme.GetItemColor(ii);
FHotTextColor:= CurrentTheme.GetItemTextColor(ii);
FDownTextColor:= CurrentTheme.GetItemImageBackground(ii);

SetItemInfo('down', ii);
FNormalImageBackColor:= CurrentTheme.GetItemColor(ii);
FHotImageBackColor:= CurrentTheme.GetItemTextColor(ii);
FDownImageBackColor:= CurrentTheme.GetItemImageBackground(ii);

SetItemInfo('checked', ii);
FCheckedItemColor:= CurrentTheme.GetItemColor(ii);
FCheckedImageBackColor:= CurrentTheme.GetItemImageBackground(ii);
FCheckedTextColor:= CurrentTheme.GetItemTextColor(ii);

SetItemInfo('hotchecked', ii);
FHotCheckedItemColor:= CurrentTheme.GetItemColor(ii);
FHotCheckedImageBackColor:= CurrentTheme.GetItemImageBackground(ii);
FHotCheckedTextColor:= CurrentTheme.GetItemTextColor(ii);

SetItemInfo('downchecked', ii);
FDownCheckedItemColor:= CurrentTheme.GetItemColor(ii);
FDownCheckedTextColor:= CurrentTheme.GetItemTextColor(ii);
FDownCheckedImageBackColor:= CurrentTheme.GetItemImageBackground(ii);
end;

procedure TmbTBXColorProvider.SetItemInfo(state: string; var IInfo: TTBXItemInfo);
begin
   FillChar(IInfo, SizeOf(TTBXItemInfo), 0);
   IInfo.ViewType := TVT_NORMALTOOLBAR;
   IInfo.ItemOptions := IO_APPACTIVE or IO_TOOLBARSTYLE;

   if state = 'normal' then
   begin
     IInfo.Pushed := false;
     IInfo.Selected := false;
     IInfo.HoverKind := hkNone;
   end;
   if state = 'over' then
   begin
     IInfo.Pushed := false;
     IInfo.Selected := false;
     IInfo.HoverKind := hkMouseHover;
   end;
   if state = 'down' then
   begin
     IInfo.Pushed := true;
     IInfo.Selected := false;
     IInfo.HoverKind := hkMouseHover;
   end;
   if state = 'checked' then
   begin
     IInfo.Pushed := false;
     IInfo.Selected := true;
     IInfo.HoverKind := hkNone;
   end;
   if state = 'hotchecked' then
   begin
     IInfo.Pushed := false;
     IInfo.Selected := true;
     IInfo.HoverKind := hkMouseHover;
   end;
   if state = 'downchecked' then
   begin
     IInfo.Pushed := true;
     IInfo.Selected := true;
     IInfo.HoverKind := hkMouseHover;
   end;
end;

end.

