unit mbTBXComboBox;

{$I mxs.inc}

interface

uses
  Messages, SysUtils, Classes, Controls, StdCtrls, Windows, TBX, TBXUtils, TBXThemes,
  Graphics;

type
  TMouseLoc = (mlNone, mlOver, mlDown);
  TDrawItemLabelEvent = procedure (Sender: TObject; Index: integer; Font: TFont; State: TOwnerDrawState) of object;

  TmbTBXComboBox = class(TComboBox)
  private
   FMouseLoc: TMouseLoc;
   FMouseOver: boolean;
   FThemed: boolean;
   FHotTrack: boolean;

   procedure SetHotTrack(Value: boolean);
   procedure SetThemed(Value: boolean);
   procedure SetCoverBorder(c: boolean);
   procedure SetThemedSelect(t: boolean);
   procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
   procedure TBMThemeChange(var Message: TMessage); message TBM_THEMECHANGE;
  protected
   FThemedSelect: boolean;
   FCoverBorder: boolean;
   procedure SetItemInfo(state: string; var IInfo: TTBXItemInfo; var EInfo: TTBXEditInfo);
   procedure CMGotFocus(var Message: TCMGotFocus);
   procedure CMLostFocus(var Message: TCMLostFocus);
   procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
   procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
   procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
   procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
   procedure DropDown; override;
   {$IFDEF DELPHI_7_UP}
   procedure CloseUp; override;
   {$ENDIF}
   procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
  public
   constructor Create(AOwner: TComponent); override;
   destructor Destroy; override;
  published
   property ThemedSelect: boolean read FThemedSelect write SetThemedSelect default false;
   property CoverBorder: boolean read FCoverBorder write SetCoverBorder default false;
   property Themed: boolean read FThemed write SetThemed default true;
   property HotTracking: boolean read FHotTrack write SetHotTrack default true;
  end;

implementation

constructor TmbTBXComboBox.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
 ParentColor := false;
 AddThemeNotification(Self);
 FMouseLoc := mlNone;
 FMouseOver := false;
 DoubleBuffered := true;
 FThemedSelect := false;
 FCoverBorder := false;
 FThemed := true;
 FHotTrack := true;
end;

destructor TmbTBXComboBox.Destroy;
begin
 inherited Destroy;
 RemoveThemeNotification(Self);
end;

procedure TmbTBXComboBox.WMPaint(var Message: TWMPaint);
var
  R: TRect;
  ii: TTBXItemInfo;
  ed: TTBXEditInfo;
begin
  inherited;
  if not FThemed then Exit;
  GetWindowRect(Handle, R);
  OffsetRect(R, -R.Left, -R.Top);
  Canvas.Brush.Color := Color;
  // set info
  case FMouseLoc of
   mlNone: SetItemInfo('normal', ii, ed);
   mlOver:
    if FHotTrack or Focused then
    SetItemInfo('over', ii, ed)
   else
    SetItemInfo('normal', ii, ed);
   mlDown: SetItemInfo('down', ii, ed);
  end;
  if Focused and (FMouseLoc <> mlDown) then SetItemInfo('over', ii, ed);
  if (csDesigning in ComponentState) or (Enabled = false) then SetItemInfo('normal', ii, ed);
  ii.Enabled := Enabled;
  //paint
  if style = csSimple then
   begin
    ExcludeClipRect(Canvas.Handle, 2, 2, R.Right - 1, R.Bottom - 1);
    // paint the border
    CurrentTheme.PaintFrame(Canvas, R, ii);
    CurrentTheme.PaintBackgnd(Canvas, R, R, R, CurrentTheme.GetViewColor(TVT_NORMALTOOLBAR), false, TVT_NORMALTOOLBAR);
    CurrentTheme.PaintDock(Canvas, R, R, DP_TOP);
    if CurrentTheme.PaintDockBackground then
     CurrentTheme.PaintBackgnd(Canvas, R, R, R, CurrentTheme.GetViewColor(TVT_NORMALTOOLBAR), false, TVT_NORMALTOOLBAR)
    else
     CurrentTheme.PaintDock(Canvas, R, R, DP_TOP);
    CurrentTheme.PaintButton(Canvas, R, ii);
    //hide double border
    Canvas.Brush.Color := Color;
    InflateRect(R, -1, -1);
    Canvas.FillRect(R);
   end
  else
   begin
    if not FCoverBorder then
     ExcludeClipRect(Canvas.Handle, 2, 2, R.Right - (2 + GetSystemMetrics(SM_CXHSCROLL)), R.Bottom - 2)
    else
     ExcludeClipRect(Canvas.Handle, 3, 3, R.Right - (3 + GetSystemMetrics(SM_CXHSCROLL)), R.Bottom - 3);
    // paint the border and button
    CurrentTheme.PaintFrame(Canvas, R, ii);
    CurrentTheme.PaintBackgnd(Canvas, R, R, R, CurrentTheme.GetViewColor(TVT_NORMALTOOLBAR), false, TVT_NORMALTOOLBAR);
    CurrentTheme.PaintDock(Canvas, R, R, DP_TOP);
    if CurrentTheme.PaintDockBackground then
     CurrentTheme.PaintBackgnd(Canvas, R, R, R, CurrentTheme.GetViewColor(TVT_NORMALTOOLBAR), false, TVT_NORMALTOOLBAR)
    else
     CurrentTheme.PaintDock(Canvas, R, R, DP_TOP);
    CurrentTheme.PaintEditFrame(Canvas, R, ii, ed);
    //hide double border
    Canvas.Brush.Color := Color;
    Dec(R.Right, GetSystemMetrics(SM_CXHSCROLL));
    if FCoverBorder then
     InflateRect(R, -1, -1)
    else
     InflateRect(R, -2, -2);
    Canvas.FillRect(R);
   end;
end;

procedure TmbTBXComboBox.SetItemInfo(state: string; var IInfo: TTBXItemInfo; var EInfo: TTBXEditInfo);
begin
   FillChar(IInfo, SizeOf(TTBXItemInfo), 0);
   IInfo.ViewType := TVT_NORMALTOOLBAR;
   IInfo.ItemOptions := IO_APPACTIVE or IO_TOOLBARSTYLE;
   IInfo.Enabled := True;

   FillChar(EInfo, SizeOf(TTBXEditInfo), 0);
   EInfo.RightBtnWidth := GetSystemMetrics(SM_CXHSCROLL);
   EInfo.RightBtnInfo.ButtonType := EBT_DROPDOWN;

   if state = 'normal' then
   begin
     IInfo.Pushed := false;
     IInfo.Selected := false;
     IInfo.HoverKind := hkNone;
     EInfo.RightBtnInfo.ButtonState := 0;
   end;
   if state = 'over' then
   begin
     IInfo.Pushed := false;
     IInfo.Selected := false;
     IInfo.HoverKind := hkMouseHover;
     EInfo.RightBtnInfo.ButtonState := EBDS_HOT;
   end;
   if state = 'down' then
   begin
     IInfo.Pushed := true;
     IInfo.Selected := false;
     IInfo.HoverKind := hkMouseHover;
     EInfo.RightBtnInfo.ButtonState := EBDS_PRESSED;
   end;
end;

procedure TmbTBXComboBox.TBMThemeChange(var Message: TMessage);
begin
if Message.WParam = TSC_VIEWCHANGE then
 begin
  FMouseLoc := mlNone;
  Invalidate;
 end;
end;

procedure TmbTBXComboBox.CMMouseEnter(var Message: TMessage);
begin
FMouseOver := true;
 if (not DroppedDown) and (not Focused) then
  begin
   FMouseLoc := mlOver;
   if FHotTrack then
   Invalidate;
  end;
end;

procedure TmbTBXComboBox.CMMouseLeave(var Message: TMessage);
begin
FMouseOver := false;
 if (not DroppedDown) and (not Focused) then
  begin
   FMouseLoc := mlNone;
   if FHotTrack then
   Invalidate;
  end;
end;

procedure TmbTBXComboBox.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 FMouseLoc := mlDown;
 Invalidate;
end;

procedure TmbTBXComboBox.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 if (not DroppedDown) and (not Focused) then
  begin
   if FMouseOver then
    FMouseLoc := mlOver
   else
    FMouseLoc := mlNone;
   Invalidate;
  end;
end;

procedure TmbTBXComboBox.DropDown;
begin
 inherited;
 if FMouseLoc <> mlDown then
  begin
   FMouseLoc := mlDown;
   Invalidate;
  end;
 if Assigned(OnDropDown) then
  OnDropDown(Self);
end;

{$IFDEF DELPHI_7_UP}
procedure TmbTBXComboBox.CloseUp;
begin
 inherited;
 if FMouseOver then
  FMouseLoc := mlOver
 else
  FMouseLoc := mlNone;
 Invalidate;
end;
{$ENDIF}

procedure TmbTBXComboBox.CMGotFocus(var Message: TCMGotFocus);
begin
 inherited;
 FMouseLoc := mlOver;
 Invalidate;
end;

procedure TmbTBXComboBox.CMLostFocus(var Message: TCMLostFocus);
begin
 inherited;
 FMouseLoc := mlNone;
 Invalidate;
end;

procedure TmbTBXComboBox.CMEnabledChanged(var Message: TMessage);
begin
 inherited;
 Invalidate;
end;

procedure TmbTBXComboBox.SetThemedSelect(t: boolean);
begin
 FThemedSelect := t;
 Invalidate;
end;

procedure TmbTBXComboBox.SetCoverBorder(c: boolean);
begin
 FCoverBorder := c;
 Invalidate;
end;

procedure TmbTBXComboBox.SetThemed(Value: boolean);
begin
 FThemed := Value;
 Invalidate;
end;

procedure TmbTBXComboBox.SetHotTrack(Value: boolean);
begin
 if FHotTrack <> Value then
  begin
   FHotTrack := Value;
   Invalidate;
  end;
end;

end.
