unit mbTBXComboEdit;

{$I mxs.inc}

interface

uses
  SysUtils, Classes, mbTBXEdit, StdCtrls, Windows, Messages, Graphics, TBX,
  TBXUtils, TBXThemes, Controls, TBXDkPanels, Menus, ImgList;

type
  TmbTBXComboEdit = class(TmbTBXEdit)
  private
   FButton: TTBXButton;
   FButtonStyle: TButtonStyle;
   FButtonCaption: TCaption;
   FButtonCursor: TCursor;
   FButtonDropDownCombo: boolean;
   FButtonDropDownMenu: TPopupMenu;
   FButtonEnabled: boolean;
   FButtonImageIndex: TImageIndex;
   FButtonImages: TCustomImageList;
   FButtonPopupMenu: TPopupMenu;
   FButtonHint: string;
   FButtonShowHint: boolean;
   FButtonVisible: boolean;
   FButtonWidth: integer;
   FOnButtonClick: TNotifyEvent;

   function GetMinHeight: Integer;
   procedure SetButtonStyle(s: TButtonStyle);
   procedure SetButtonCaption(c: TCaption);
   procedure SetButtonCursor(c: TCursor);
   procedure SetButtonDropDownCombo(b: boolean);
   procedure SetButtonDropDownMenu(p: TPopupMenu);
   procedure SetButtonEnabled(b: boolean);
   procedure SetButtonImageIndex(i: TImageIndex);
   procedure SetButtonImages(c: TCustomImageList);
   procedure SetButtonPopupMenu(p: TPopupMenu);
   procedure SetButtonHint(s: string);
   procedure SetButtonShowHint(b: boolean);
   procedure SetButtonVisible(b: boolean);
   procedure SetButtonWidth(i: integer);
   procedure ButtonClick(Sender: TObject);
  protected
   procedure SetEditRect;
   procedure WMSize(var Message: TWMSize); message WM_SIZE;
   procedure GetChildren(Proc: TGetChildProc; Root: TComponent); override;
   procedure CreateParams(var Params: TCreateParams); override;
   procedure CreateWnd; override;
   procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
  public
   constructor Create(AOwner: TComponent); override;
   destructor Destroy; override;
   property Button: TTBXButton read FButton;
  published
   property ButtonStyle: TButtonStyle read FButtonStyle write SetButtonStyle default bsNormal;
   property ButtonCaption: TCaption read FButtonCaption write SetButtonCaption;
   property ButtonCursor: TCursor read FButtonCursor write SetButtonCursor default crDefault;
   property ButtonDropDownCombo: boolean read FButtonDropDownCombo write SetButtonDropDownCombo default false;
   property ButtonDropDownMenu: TPopupMenu read FButtonDropDownMenu write SetButtonDropDownMenu;
   property ButtonEnabled: boolean read FButtonEnabled write SetButtonEnabled default true;
   property ButtonImageIndex: TImageIndex read FButtonImageIndex write SetButtonImageIndex default -1;
   property ButtonImageList: TCustomImageList read FButtonImages write SetButtonImages;
   property ButtonPopupMenu: TPopupMenu read FButtonPopupMenu write SetButtonPopupMenu;
   property ButtonHint: string read FButtonHint write SetButtonHint;
   property ButtonShowHint: boolean read FButtonShowHint write SetButtonShowHint default false;
   property ButtonVisible: boolean read FButtonVisible write SetButtonVisible default true;
   property ButtonWidth: integer read FButtonWidth write SetButtonWidth default 18;

   property OnButtonClick: TNotifyEvent read FOnButtonClick write FOnButtonClick;
  end;

implementation

constructor TmbTBXComboEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  ParentColor := false;
  FButton := TTBXButton.Create(Self);
  FButton.Width := 18;
  FButtonWidth := 18;
  FButton.Height := Height - 3;
  FButton.Left := Width - FButton.Width - 5;
  FButton.Top := 0;
  FButton.Visible := True;
  FButtonVisible := true;
  FButton.Parent := Self;
  FButton.AutoSize := false;
  FButton.Cursor := crDefault;
  FButtonCursor := crDefault;
  FButton.Enabled := true;
  FButtonEnabled := true;
  FButton.ImageIndex := -1;
  FButtonImageIndex := -1;
  FButton.DropDownCombo := false;
  FButtonDropDownCombo := false;
  FButton.ShowHint := false;
  FButtonShowHint := false;
  FButton.ButtonStyle := bsNormal;
  FButtonStyle := bsNormal;
  FButton.OnClick := ButtonClick;
  FButton.Name := Name + 'Button';
  FButton.Caption := '';
end;

destructor TmbTBXComboEdit.Destroy;
begin
  FButton := nil;
  inherited Destroy;
end;

procedure TmbTBXComboEdit.CMEnabledChanged(var Message: TMessage);
begin
 inherited;
 if FButton <> nil then
  FButton.Enabled := Enabled;
end;

procedure TmbTBXComboEdit.GetChildren(Proc: TGetChildProc; Root: TComponent);
begin
end;

procedure TmbTBXComboEdit.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  Params.Style := Params.Style or ES_MULTILINE or WS_CLIPCHILDREN;
end;

procedure TmbTBXComboEdit.CreateWnd;
begin
  inherited CreateWnd;
  SetEditRect;
end;

procedure TmbTBXComboEdit.SetEditRect;
var
  Loc: TRect;
begin
  SendMessage(Handle, EM_GETRECT, 0, LongInt(@Loc));
  Loc := Rect(0, 0, ClientWidth - FButton.Width, ClientHeight + 1);
  SendMessage(Handle, EM_SETRECTNP, 0, LongInt(@Loc));
  SendMessage(Handle, EM_GETRECT, 0, LongInt(@Loc));
end;

procedure TmbTBXComboEdit.WMSize(var Message: TWMSize);
var
  MinHeight: Integer;
begin
  inherited;
  MinHeight := GetMinHeight;
  if Height < MinHeight then
    Height := MinHeight
  else if FButton <> nil then
   begin
    FButton.SetBounds(Width - FButton.Width - 5, 0, FButton.Width, Height - 5); 
    SetEditRect;
   end;
end;

function TmbTBXComboEdit.GetMinHeight: Integer;
var
  DC: HDC;
  SaveFont: HFont;
  I: Integer;
  SysMetrics, Metrics: TTextMetric;
begin
  DC := GetDC(0);
  GetTextMetrics(DC, SysMetrics);
  SaveFont := SelectObject(DC, Font.Handle);
  GetTextMetrics(DC, Metrics);
  SelectObject(DC, SaveFont);
  ReleaseDC(0, DC);
  I := SysMetrics.tmHeight;
  if I > Metrics.tmHeight then I := Metrics.tmHeight;
  Result := Metrics.tmHeight + I div 4 + GetSystemMetrics(SM_CYBORDER) * 4 + 2;
end;

procedure TmbTBXComboEdit.SetButtonStyle(s: TButtonStyle);
begin
 FButtonStyle := s;
 FButton.ButtonStyle := s;
end;

procedure TmbTBXComboEdit.SetButtonCaption(c: TCaption);
begin
 FButtonCaption := c;
 FButton.Caption := c;
end;

procedure TmbTBXComboEdit.SetButtonCursor(c: TCursor);
begin
 FButtonCursor := c;
 FButton.Cursor := c;
end;

procedure TmbTBXComboEdit.SetButtonDropDownCombo(b: boolean);
begin
 FButtonDropDownCombo := b;
 FButton.DropDownCombo := b;
end;

procedure TmbTBXComboEdit.SetButtonDropDownMenu(p: TPopupMenu);
begin
 FButtonDropDownMenu := p;
 FButton.DropDownMenu := p;
end;

procedure TmbTBXComboEdit.SetButtonEnabled(b: boolean);
begin
 FButtonEnabled := b;
 FButton.Enabled := b;
end;

procedure TmbTBXComboEdit.SetButtonImageIndex(i: TImageIndex);
begin
 FButtonImageIndex := i;
 FButton.ImageIndex := i;
end;

procedure TmbTBXComboEdit.SetButtonImages(c: TCustomImageList);
begin
 FButtonImages := c;
 FButton.Images := c;
end;

procedure TmbTBXComboEdit.SetButtonPopupMenu(p: TPopupMenu);
begin
 FButtonPopupMenu := p;
 FButton.PopupMenu := p;
end;

procedure TmbTBXComboEdit.SetButtonHint(s: string);
begin
 FButtonHint := s;
 FButton.Hint := s;
end;

procedure TmbTBXComboEdit.SetButtonShowHint(b: boolean);
begin
 FButtonShowHint := b;
 FButton.ShowHint := b;
end;

procedure TmbTBXComboEdit.SetButtonVisible(b: boolean);
var
 w, t, l: integer;
begin
 FButtonVisible := b;
 FButton.Visible := b;
 w := FButtonWidth;
 t := FButton.Top;
 l := FButton.Left;
 if b = true then
  FButton.SetBounds(Width - FButton.Width - 5, 0, FButton.Width, Height - 5)
 else
  FButton.SetBounds(0, 0, 0, 0);
 SetEditRect;
 RPaint;
 FButtonWidth := w;
 FButton.Width := w;
 FButton.Top := t;
 FButton.Left := l;
 FButton.Height := Height - 3;
end;

procedure TmbTBXComboEdit.SetButtonWidth(i: integer);
begin
 FButtonWidth := i;
 FButton.Width := i;
 FButton.SetBounds(Width - FButton.Width - 5, 0, FButton.Width, Height - 5);
 SetEditRect;
 RPaint;
end;

procedure TmbTBXComboEdit.ButtonClick(Sender: TObject);
begin
 if Assigned(FOnButtonClick) then
  FOnButtonClick(Self);
end;

end.
