unit mbTBXDrawGrid;

{$I mxs.inc}

interface

uses
  SysUtils, Classes, Controls, StdCtrls, Windows, Messages, Graphics, TBX,
  TBXUtils, TBXThemes, Grids, Math, Forms;

type
  PIntArray = ^TIntArray;
  TIntArray = array[0..MaxCustomExtents] of Integer;
  TMouseLoc = (mlNone, mlOver, mlDown);
  TFixedStyle = (fsNormal, fsHot, fsDown);
  TmbTBXDrawGrid = class(TDrawGrid)
  private
   FMouseOver: boolean;
   d, o: TTBXItemInfo;
   FStyle: TFixedStyle;
   LineColor: TColor;
   ThemedSelCl: TColor;
   ThemedTextCl: TColor;

   procedure SetThemedSelect(t: boolean);
   procedure SetStyle(S: TFixedStyle);
  protected
   FMouseLoc: TMouseLoc;
   FAfterPaint: TNotifyEvent;
   FThemedSelect: boolean;

   procedure SetItemInfo(state: string; var IInfo: TTBXItemInfo);
   procedure GridRectToScreenRect(GridRect: TGridRect; var ScreenRect: TRect; IncludeLine: Boolean);
   function IsActiveControl: Boolean;
   procedure Paint; override;
   procedure WMNCPaint(var Msg: TWMNCPaint); message WM_NCPAINT;
   procedure TBMThemeChange(var Message: TMessage); message TBM_THEMECHANGE;
   procedure Resize; override;
   procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
   procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
   procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
   procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
   procedure CMGotFocus(var Message: TCMGotFocus); message CM_ENTER;
   procedure CMLostFocus(var Message: TCMLostFocus); message CM_EXIT;
   procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
   procedure RPaint;
   property OnAfterPaint: TNotifyEvent read FAfterPaint write FAfterPaint;
  public
   constructor Create(AOwner: TComponent); override;
   destructor Destroy; override;
  published
   property FixedCellStyle: TFixedStyle read FStyle write SetStyle default fsNormal;
   property ThemedSelect: boolean read FThemedSelect write SetThemedSelect default false;
   property Align;
   property Anchors;
   property BiDiMode;
   property Color;
   property ColCount;
   property Constraints;
   property DefaultColWidth;
   property DefaultRowHeight;
   property DefaultDrawing;
   property DragCursor;
   property DragKind;
   property DragMode;
   property Enabled;
   property FixedCols;
   property RowCount;
   property FixedRows;
   property Font;
   property GridLineWidth;
   property Options;
   property ParentBiDiMode;
   property ParentColor;
   property ParentFont;
   property ParentShowHint;
   property PopupMenu;
   property ScrollBars;
   property ShowHint;
   property TabOrder;
   property Visible;
   property VisibleColCount;
   property VisibleRowCount;

   property OnClick;
   property OnColumnMoved;
   property OnContextPopup;
   property OnDblClick;
   property OnDragDrop;
   property OnDragOver;
   property OnDrawCell;
   property OnEndDock;
   property OnEndDrag;
   property OnEnter;
   property OnExit;
   property OnGetEditMask;
   property OnGetEditText;
   property OnKeyDown;
   property OnKeyPress;
   property OnKeyUp;
   property OnMouseDown;
   property OnMouseMove;
   property OnMouseUp;
   property OnMouseWheelDown;
   property OnMouseWheelUp;
   property OnRowMoved;
   property OnSelectCell;
   property OnSetEditText;
   property OnStartDock;
   property OnStartDrag;
   property OnTopLeftChanged;
  end;

implementation

constructor TmbTBXDrawGrid.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
 AddThemeNotification(Self);
 FMouseLoc := mlNone;
 FMouseOver := false;
 DoubleBuffered := true;
 FStyle := fsNormal;
 SetItemInfo('normal', d);
 SetItemInfo('over', o);
 ThemedSelCl := Blend(CurrentTheme.GetItemColor(o), clWhite, 50);
 ThemedTextCl := Blend(CurrentTheme.GetItemTextColor(o), clBlack, 70);
 FThemedSelect := false;
end;

destructor TmbTBXDrawGrid.Destroy;
begin
 RemoveThemeNotification(Self);
 inherited Destroy;
end;

procedure TmbTBXDrawGrid.TBMThemeChange(var Message: TMessage);
begin
if Message.WParam = TSC_VIEWCHANGE then
 begin
  ThemedSelCl := Blend(CurrentTheme.GetItemColor(o), clWhite, 50);
  ThemedTextCl := Blend(CurrentTheme.GetItemTextColor(o), clBlack, 70);
  RPaint;
 end;
end;

procedure TmbTBXDrawGrid.RPaint;
var
 r: TRect;
begin
 r := ClientRect;
 RedrawWindow(Handle, @R, 0, RDW_INVALIDATE or RDW_UPDATENOW or RDW_FRAME);
end;

procedure TmbTBXDrawGrid.SetItemInfo(state: string; var IInfo: TTBXItemInfo);
begin
   FillChar(IInfo, SizeOf(TTBXItemInfo), 0);
   IInfo.ViewType := TVT_NORMALTOOLBAR;
   IInfo.ItemOptions := IO_APPACTIVE or IO_TOOLBARSTYLE;
   IInfo.Enabled := True;

   if state = 'normal' then
    begin
     IInfo.Pushed := false;
     IInfo.Selected := false;
     IInfo.HoverKind := hkNone;
    end;
   if state = 'over' then
    begin
     IInfo.Pushed := false;
     IInfo.Selected := false;
     IInfo.HoverKind := hkMouseHover;
    end;
   if state = 'down' then
    begin
     IInfo.Pushed := true;
     IInfo.Selected := false;
     IInfo.HoverKind := hkMouseHover;
    end;
end;

procedure TmbTBXDrawGrid.WMNCPaint(var Msg: TWMNCPaint);
var
 DC: HDC;
 ii: TTBXItemInfo;
 R: TRect;
 C: TCanvas;
begin
 inherited;
 C := TCanvas.Create;
 DC := GetWindowDC(Handle);
 GetWindowRect(Handle, R);
 OffsetRect(R, -R.Left, -R.Top);
 C.Handle := DC;
 ExcludeClipRect(DC, 2, 2, R.Right - 1, R.Bottom - 1);
 // set info
 if Focused then FMouseLoc := mlOver;
 if (csDesigning in ComponentState) or (Enabled = false) then FMouseLoc := mlNone;
 case FMouseLoc of
  mlNone: SetItemInfo('normal', ii);
  mlOver: SetItemInfo('over', ii);
  mlDown: SetItemInfo('down', ii);
 end;
 ii.Enabled := Enabled;
 // paint the border
 CurrentTheme.PaintFrame(c, R, ii);
 CurrentTheme.PaintBackgnd(c, R, R, R, CurrentTheme.GetViewColor(TVT_NORMALTOOLBAR), false, TVT_NORMALTOOLBAR);
 CurrentTheme.PaintDock(c, R, R, DP_TOP);
 if CurrentTheme.PaintDockBackground then
  CurrentTheme.PaintBackgnd(c, R, R, R, CurrentTheme.GetViewColor(TVT_NORMALTOOLBAR), false, TVT_NORMALTOOLBAR)
 else
  CurrentTheme.PaintDock(c, R, R, DP_TOP);
 CurrentTheme.PaintButton(c, R, ii);
 //hide double border
 C.Brush.Color := Color;
 InflateRect(R, -1, -1);
 C.FillRect(R);
 // clean up
 C.Handle := 0;
 ReleaseDC(Handle, DC);
 C.Free;
 // do after paint
 if Assigned(FAfterPaint) then
  FAfterPaint(self);
end;

procedure TmbTBXDrawGrid.Resize;
begin
 RPaint;
 if Assigned(OnResize) then
  OnResize(Self);
end;

procedure TmbTBXDrawGrid.CMMouseEnter(var Message: TMessage);
begin
  if not Focused then
   begin
    FMouseOver := true;
    FMouseLoc := mlOver;
    RPaint;
   end;
end;

procedure TmbTBXDrawGrid.CMMouseLeave(var Message: TMessage);
begin
 if not Focused then
  begin
   FMouseOver := false;
   FMouseLoc := mlNone;
   RPaint;
  end;
end;

procedure TmbTBXDrawGrid.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 FMouseLoc := mlDown;
 RPaint;
 inherited;
end;

procedure TmbTBXDrawGrid.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 if not Focused then
  begin
   if FMouseOver then
    FMouseLoc := mlOver
   else
    FMouseLoc := mlNone;
   RPaint;
  end;
 inherited;
end;

procedure TmbTBXDrawGrid.CMGotFocus(var Message: TCMGotFocus);
begin
 inherited;
 FMouseLoc := mlOver;
 RPaint;
end;

procedure TmbTBXDrawGrid.CMLostFocus(var Message: TCMLostFocus);
begin
 inherited;
 FMouseLoc := mlNone;
 RPaint;
end;

procedure TmbTBXDrawGrid.CMEnabledChanged(var Message: TMessage);
begin
 inherited;
 RPaint;
end;

function StackAlloc(Size: Integer): Pointer; register;
asm
  POP   ECX
  MOV   EDX, ESP
  ADD   EAX, 3
  AND   EAX, not 3
  CMP   EAX, 4092
  JLE   @@2
@@1:
  SUB   ESP, 4092
  PUSH  EAX
  SUB   EAX, 4096
  JNS   @@1
  ADD   EAX, 4096
@@2:
  SUB   ESP, EAX
  MOV   EAX, ESP
  PUSH  EDX
  MOV   EDX, ESP
  SUB   EDX, 4
  PUSH  EDX
  PUSH  ECX
end;


procedure StackFree(P: Pointer); register;
asm
  POP   ECX
  MOV   EDX, DWORD PTR [ESP]
  SUB   EAX, 8
  CMP   EDX, ESP
  JNE   @@1
  CMP   EDX, EAX
  JNE   @@1
  MOV   ESP, DWORD PTR [ESP+4]
@@1:
  PUSH  ECX
end;

procedure FillDWord(var Dest; Count, Value: Integer); register;
asm
  XCHG  EDX, ECX
  PUSH  EDI
  MOV   EDI, EAX
  MOV   EAX, EDX
  REP   STOSD
  POP   EDI
end;

function PointInGridRect(Col, Row: Longint; const Rect: TGridRect): Boolean;
begin
  Result := (Col >= Rect.Left) and (Col <= Rect.Right) and (Row >= Rect.Top) and (Row <= Rect.Bottom);
end;

procedure TmbTBXDrawGrid.GridRectToScreenRect(GridRect: TGridRect; var ScreenRect: TRect; IncludeLine: Boolean);

  function LinePos(const AxisInfo: TGridAxisDrawInfo; Line: Integer): Integer;
  var
    Start, I: Longint;
  begin
   Result := 0;
   if Line < AxisInfo.FixedCellCount then
    Start := 0
   else
    begin
     if Line >= AxisInfo.FirstGridCell then
      Result := AxisInfo.FixedBoundary;
     Start := AxisInfo.FirstGridCell;
    end;
   for I := Start to Line - 1 do
    begin
     Inc(Result, AxisInfo.GetExtent(I) + AxisInfo.EffectiveLineWidth);
     if Result > AxisInfo.GridExtent then
      begin
       Result := 0;
       Exit;
      end;
    end;
  end;

  function CalcAxis(const AxisInfo: TGridAxisDrawInfo; GRMin, GRMax: Integer; var SRMin, SRMax: Integer): Boolean;
  begin
   Result := False;
   if (GRMin >= AxisInfo.FixedCellCount) and (GRMin < AxisInfo.FirstGridCell) then
    begin
     if GRMax < AxisInfo.FirstGridCell then
      begin
       FillChar(ScreenRect, SizeOf(ScreenRect), 0);
       Exit;
      end
     else
      GRMin := AxisInfo.FirstGridCell;
    end;
   if GRMax > AxisInfo.LastFullVisibleCell then
    begin
     GRMax := AxisInfo.LastFullVisibleCell;
     if GRMax < AxisInfo.GridCellCount - 1 then
      Inc(GRMax);
     if LinePos(AxisInfo, GRMax) = 0 then
      Dec(GRMax);
    end;
    SRMin := LinePos(AxisInfo, GRMin);
    SRMax := LinePos(AxisInfo, GRMax);
    if SRMax = 0 then
     SRMax := SRMin + AxisInfo.GetExtent(GRMin)
    else
     Inc(SRMax, AxisInfo.GetExtent(GRMax));
    if SRMax > AxisInfo.GridExtent then
     SRMax := AxisInfo.GridExtent;
    if IncludeLine then
     Inc(SRMax, AxisInfo.EffectiveLineWidth);
    Result := True;
  end;

var
  DrawInfo: TGridDrawInfo;
  Hold: Integer;
begin
 FillChar(ScreenRect, SizeOf(ScreenRect), 0);
 if (GridRect.Left > GridRect.Right) or (GridRect.Top > GridRect.Bottom) then Exit;
 CalcDrawInfo(DrawInfo);
 if GridRect.Left > DrawInfo.Horz.LastFullVisibleCell + 1 then Exit;
 if GridRect.Top > DrawInfo.Vert.LastFullVisibleCell + 1 then Exit;
 if CalcAxis(DrawInfo.Horz, GridRect.Left, GridRect.Right, ScreenRect.Left, ScreenRect.Right) then
  CalcAxis(DrawInfo.Vert, GridRect.Top, GridRect.Bottom, ScreenRect.Top, ScreenRect.Bottom);
 if UseRightToLeftAlignment and (Canvas.CanvasOrientation = coLeftToRight) then
  begin
   Hold := ScreenRect.Left;
   ScreenRect.Left := ClientWidth - ScreenRect.Right;
   ScreenRect.Right := ClientWidth - Hold;
  end;
end;

procedure TmbTBXDrawGrid.Paint;
var
 LC: TColor;
 DrawInfo: TGridDrawInfo;
 Sel: TGridRect;
 UpdateRect: TRect;
 AFocusRect, FRect: TRect;
 PointsList: PIntArray;
 StrokeList: PIntArray;
 MaxStroke: Integer;

 procedure DrawLines(DoHorz, DoVert: Boolean; Col, Row: Longint; const CellBounds: array of Integer;
                       OnColor, OffColor: TColor);
  const
    FlatPenStyle = PS_Geometric or PS_Solid or PS_EndCap_Flat or PS_Join_Miter;

    procedure DrawAxisLines(const AxisInfo: TGridAxisDrawInfo; Cell, MajorIndex: Integer; UseOnColor: Boolean);
    var
     Line: Integer;
     LogBrush: TLogBrush;
     Index: Integer;
     Points: PIntArray;
     StopMajor, StartMinor, StopMinor, StopIndex: Integer;
     LineIncr: Integer;
    begin
      if AxisInfo.EffectiveLineWidth <> 0 then
       begin
        Canvas.Pen.Width := GridLineWidth;
        if UseOnColor then
         Canvas.Pen.Color := OnColor
        else
         Canvas.Pen.Color := OffColor;
        if Canvas.Pen.Width > 1 then
         begin
          LogBrush.lbStyle := bs_Solid;
          LogBrush.lbColor := Canvas.Pen.Color;
          LogBrush.lbHatch := 0;
          Canvas.Pen.Handle := ExtCreatePen(FlatPenStyle, Canvas.Pen.Width, LogBrush, 0, nil);
         end;
        Points := PointsList;
        Line := CellBounds[MajorIndex] + AxisInfo.EffectiveLineWidth shr 1 + AxisInfo.GetExtent(Cell);
        if UseRightToLeftAlignment and (MajorIndex = 0) then
         Inc(Line);
        StartMinor := CellBounds[MajorIndex xor 1];
        StopMinor := CellBounds[2 + (MajorIndex xor 1)];
        StopMajor := CellBounds[2 + MajorIndex] + AxisInfo.EffectiveLineWidth;
        StopIndex := MaxStroke * 4;
        Index := 0;
        repeat
         Points^[Index + MajorIndex] := Line;
         Points^[Index + (MajorIndex xor 1)] := StartMinor;
         Inc(Index, 2);
         Points^[Index + MajorIndex] := Line;
         Points^[Index + (MajorIndex xor 1)] := StopMinor;
         Inc(Index, 2);
         repeat
          Inc(Cell);
          LineIncr := AxisInfo.GetExtent(Cell) + AxisInfo.EffectiveLineWidth;
        until (LineIncr > 0) or (Cell > AxisInfo.LastFullVisibleCell);
         Inc(Line, LineIncr);
       until (Line > StopMajor) or (Cell > AxisInfo.LastFullVisibleCell) or (Index > StopIndex);
        PolyPolyLine(Canvas.Handle, Points^, StrokeList^, Index shr 2);
      end;
    end;
  begin
   if (CellBounds[0] = CellBounds[2]) or (CellBounds[1] = CellBounds[3]) then Exit;
   if not DoHorz then
    begin
     DrawAxisLines(DrawInfo.Vert, Row, 1, DoHorz);
     DrawAxisLines(DrawInfo.Horz, Col, 0, DoVert);
    end
   else
    begin
     DrawAxisLines(DrawInfo.Horz, Col, 0, DoVert);
     DrawAxisLines(DrawInfo.Vert, Row, 1, DoHorz);
    end;
  end;

  procedure DrawCells(ACol, ARow: Longint; StartX, StartY, StopX, StopY: Integer; Color: TColor;
                       IncludeDrawState: TGridDrawState);
  var
   CurCol, CurRow: Longint;
   AWhere, Where: TRect;
   DrawState: TGridDrawState;
   Focused: Boolean;
  begin
   CurRow := ARow;
   Where.Top := StartY;
   while (Where.Top < StopY) and (CurRow < RowCount) do
    begin
     CurCol := ACol;
     Where.Left := StartX;
     Where.Bottom := Where.Top + RowHeights[CurRow];
     while (Where.Left < StopX) and (CurCol < ColCount) do
      begin
       Where.Right := Where.Left + ColWidths[CurCol];
       if (Where.Right > Where.Left) and RectVisible(Canvas.Handle, Where) then
        begin
         DrawState := IncludeDrawState;
         Focused := IsActiveControl;
         if Focused and (CurRow = Row) and (CurCol = Col) then
          Include(DrawState, gdFocused);
         if PointInGridRect(CurCol, CurRow, Sel) then
          Include(DrawState, gdSelected);
         if not (gdFocused in DrawState) or not (goEditing in Options) or not EditorMode or (csDesigning in ComponentState) then
          begin
           if DefaultDrawing or (csDesigning in ComponentState) then
            begin
             Canvas.Font := Self.Font;
             if (gdSelected in DrawState) and (not (gdFocused in DrawState) or ([goDrawFocusSelected, goRowSelect] * Options <> [])) then
              begin
               if FThemedSelect then
                begin
                 Canvas.Brush.Color := ThemedSelCl;
                 Canvas.Font.Color := ThemedTextCl;
                end
               else
                begin
                 Canvas.Brush.Color := clHighlight;
                 Canvas.Font.Color := clHighlightText;
                end;
              end
             else
              Canvas.Brush.Color := Color;
             Canvas.FillRect(Where);
           end;
           DrawCell(CurCol, CurRow, Where, DrawState);
           if (gdFixed in DrawState) then
            begin
             Canvas.Brush.Style := bsClear;
             Canvas.Pen.Color := CurrentTheme.GetItemColor(d);
             Canvas.Rectangle(Where);
             Canvas.Brush.Style := bsSolid;
            end;
           if DefaultDrawing and not (csDesigning in ComponentState) and (gdFocused in DrawState) and
           ([goEditing, goAlwaysShowEditor] * Options <> [goEditing, goAlwaysShowEditor])
           and not (goRowSelect in Options) then
            begin
             if not UseRightToLeftAlignment then
              DrawFocusRect(Canvas.Handle, Where)
             else
              begin
               AWhere := Where;
               AWhere.Left := Where.Right;
               AWhere.Right := Where.Left;
               DrawFocusRect(Canvas.Handle, AWhere);
              end;
            end;
          end;
        end;
       Where.Left := Where.Right + DrawInfo.Horz.EffectiveLineWidth;
       Inc(CurCol);
     end;
     Where.Top := Where.Bottom + DrawInfo.Vert.EffectiveLineWidth;
     Inc(CurRow);
    end;
  end;

begin
 LineColor := Blend(CurrentTheme.GetItemColor(d), clBlack, 80);
 {$IFDEF DELPHI_7_UP}
  if UseRightToLeftAlignment then
    ChangeGridOrientation(True);
 {$ENDIF}
  UpdateRect := Canvas.ClipRect;
  CalcDrawInfo(DrawInfo);
  if (DrawInfo.Horz.EffectiveLineWidth > 0) or (DrawInfo.Vert.EffectiveLineWidth > 0) then
   begin
    LC := LineColor;
    MaxStroke := Max(DrawInfo.Horz.LastFullVisibleCell - LeftCol + FixedCols, DrawInfo.Vert.LastFullVisibleCell - TopRow + FixedRows) + 3;
    PointsList := StackAlloc(MaxStroke * SizeOf(TPoint) * 2);
    StrokeList := StackAlloc(MaxStroke * SizeOf(Integer));
    FillDWord(StrokeList^, MaxStroke, 2);
    if ColorToRGB(LineColor) = ColorToRGB(Color) then
     LC := CurrentTheme.GetItemTextColor(d);
    DrawLines(goFixedHorzLine in Options, goFixedVertLine in Options, 0, 0, [0, 0, DrawInfo.Horz.FixedBoundary,
              DrawInfo.Vert.FixedBoundary], LineColor, CurrentTheme.GetItemColor(d));
    DrawLines(goFixedHorzLine in Options, goFixedVertLine in Options, LeftCol, 0, [DrawInfo.Horz.FixedBoundary,
              0, DrawInfo.Horz.GridBoundary, DrawInfo.Vert.FixedBoundary], LineColor, CurrentTheme.GetItemColor(d));
    DrawLines(goFixedHorzLine in Options, goFixedVertLine in Options, 0, TopRow, [0, DrawInfo.Vert.FixedBoundary,
              DrawInfo.Horz.FixedBoundary, DrawInfo.Vert.GridBoundary], LineColor, CurrentTheme.GetItemColor(d));
    DrawLines(goHorzLine in Options, goVertLine in Options, LeftCol, TopRow, [DrawInfo.Horz.FixedBoundary,
              DrawInfo.Vert.FixedBoundary, DrawInfo.Horz.GridBoundary, DrawInfo.Vert.GridBoundary], LC, Color);
    StackFree(StrokeList);
    StackFree(PointsList);
   end;
  Sel := Selection;
  DrawCells(0, 0, 0, 0, DrawInfo.Horz.FixedBoundary, DrawInfo.Vert.FixedBoundary, CurrentTheme.GetItemColor(d), [gdFixed]);
  DrawCells(LeftCol, 0, DrawInfo.Horz.FixedBoundary , 0, DrawInfo.Horz.GridBoundary,
             DrawInfo.Vert.FixedBoundary, CurrentTheme.GetItemColor(d), [gdFixed]);
  DrawCells(0, TopRow, 0, DrawInfo.Vert.FixedBoundary, DrawInfo.Horz.FixedBoundary, DrawInfo.Vert.GridBoundary,
             CurrentTheme.GetItemColor(d), [gdFixed]);
  DrawCells(LeftCol, TopRow, DrawInfo.Horz.FixedBoundary,
             DrawInfo.Vert.FixedBoundary, DrawInfo.Horz.GridBoundary, DrawInfo.Vert.GridBoundary, Color, []);

  if not (csDesigning in ComponentState) and (goRowSelect in Options) and DefaultDrawing and Focused then
   begin
    GridRectToScreenRect(Selection, FRect, False);
    if not UseRightToLeftAlignment then
      Canvas.DrawFocusRect(FRect)
    else
     begin
      AFocusRect := FRect;
      AFocusRect.Left := FRect.Right;
      AFocusRect.Right := FRect.Left;
      DrawFocusRect(Canvas.Handle, AFocusRect);
     end;
   end;
  if DrawInfo.Horz.GridBoundary < DrawInfo.Horz.GridExtent then
   begin
    Canvas.Brush.Color := Color;
    Canvas.FillRect(Rect(DrawInfo.Horz.GridBoundary, 0, DrawInfo.Horz.GridExtent, DrawInfo.Vert.GridBoundary));
   end;
  if DrawInfo.Vert.GridBoundary < DrawInfo.Vert.GridExtent then
   begin
    Canvas.Brush.Color := Color;
    Canvas.FillRect(Rect(0, DrawInfo.Vert.GridBoundary, DrawInfo.Horz.GridExtent, DrawInfo.Vert.GridExtent));
   end;
  {$IFDEF DELPHI_7_UP}
  if UseRightToLeftAlignment then
    ChangeGridOrientation(False);
  {$ENDIF}    
end;

function TmbTBXDrawGrid.IsActiveControl: Boolean;
var
  H: Hwnd;
  ParentForm: TCustomForm;
begin
  Result := False;
  ParentForm := GetParentForm(Self);
  if Assigned(ParentForm) then
  begin
    if ParentForm.ActiveControl = Self then
      Result := True
  end
  else
  begin
    H := GetFocus;
    while IsWindow(H) and (Result = False) do
    begin
      if H = WindowHandle then
        Result := True
      else
        H := GetParent(H);
    end;
  end;
end;

procedure TmbTBXDrawGrid.SetStyle(s: TFixedStyle);
begin
 FStyle := s;
 case s of
  fsNormal: SetItemInfo('normal', d);
  fsHot: SetItemInfo('over', d);
  fsDown: SetItemInfo('down', d);
 end;
 RPaint;
end;

procedure TmbTBXDrawGrid.SetThemedSelect(t: boolean);
begin
 FThemedSelect := t;
 RPaint;
end;

end.
