unit mbTBXExpandablePanel;

{$I mxs.inc}

interface

uses
  SysUtils, Messages, Classes, Controls, Windows, Graphics, TBX, TBXUtils, TBXThemes, TBXDkPanels;

type
  THeaderPaintStyle = (hpsNormal, hpsHot, hpsDown, hpsChecked, hpsCheckedHot, hpsCheckedDown);
  TPanelArrowPosition = (apTop, apMiddle, apBottom, apWithCaption);
  TPanelArrowAlignment = (aaLeft, aaRight);

  TmbTBXExpandablePanel = class(TTBXPanelObject)
  private
   FOHeight: integer;
   FCaption: string;
   FCaptionHeight: integer;
   MouseOverButton: boolean;
   MouseIsDown: boolean;
   captiontop: integer;
   FOnCaptionClick: TNotifyEvent;
   FOnBtnClick: TNotifyEvent;
   IsExpanded: boolean;
   FArrowPos: TPanelArrowPosition;
   FArrowAlign: TPanelArrowAlignment;
   BtnRect: TRect;
   FShowArrow: boolean;
   FOnExpand: TNotifyEvent;
   FOnCollapse: TNotifyEvent;
   FHeaderPaintStyle, FPanelPaintStyle: THeaderPaintStyle;

   procedure SetHeaderPaintStyle(h: THeaderPaintStyle);
   procedure SetPanelPaintStyle(h: THeaderPaintStyle);
   procedure SetOriginalHeight(h: integer);
   procedure SetCaption(c: string);
   procedure SetExpanded(isexp: boolean);
   procedure SetArrowPos(pos: TPanelArrowPosition);
   procedure SetArrowAlign(al: TPanelArrowAlignment);
   procedure SetShowArrow(show: boolean);
   procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
   procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
   function PointInRect(aPoint: TPoint; aRect: TRect): boolean;
   function GetBtnRect: TRect;
   procedure SetItemInfo(state: string; var IInfo: TTBXItemInfo);
   procedure EraseTBXBackground(DC: HDC; ARect: TRect);
  protected
   procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
   procedure WMEraseBkgnd(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
   procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
   procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
   procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
   procedure AdjustClientRect(var Rect: TRect); override;
  public
   constructor Create(AOWner: TComponent); override;
   destructor Destroy; override;
   procedure Paint; override;
   procedure Resize;override;
   procedure BtnClick;
   procedure CaptionClick;
   procedure Expand;
   procedure Collapse;
  published
   property HeaderPaintStyle: THeaderPaintStyle read FHeaderPaintStyle write SetHeaderPaintStyle default hpsNormal;
   property PanelPaintStyle: THeaderPaintStyle read FPanelPaintStyle write SetPanelPaintStyle default hpsNormal;
   property OriginalHeight: integer read FOHeight write SetOriginalHeight;
   property Caption: string read FCaption write SetCaption;
   property Expanded: boolean read IsExpanded write SetExpanded default true;
   property ArrowPosition: TPanelArrowPosition read FArrowPos write SetArrowPos default apWithCaption;
   property ArrowAlign: TPanelArrowAlignment read FArrowAlign write SetArrowAlign default aaLeft;
   property ShowArrow: boolean read FShowArrow write SetShowArrow default true;
   property Enabled;
   property PopupMenu;
   property Visible;
   property ShowHint;
   property ParentShowHint;
   property Align;
   property Anchors;

   property OnBtnClick: TNotifyEvent read FOnBtnClick write FOnBtnClick;
   property OnCaptionClick: TNotifyEvent read FOnCaptionClick write FOnCaptionClick;
   property OnExpand: TNotifyEvent read FOnExpand write FOnExpand;
   property OnCollapse: TNotifyEvent read FOnCollapse write FOnCollapse;
   property OnMouseDown;
   property OnMouseUp;
   property OnMouseMove;
   property OnResize;
   property OnEnter;
   property OnExit;
   property OnKeyDown;
   property OnKeyPress;
   property OnKeyUp;
  end;

implementation

var
 bb: TBitmap;
 tn, cn: string;

constructor TmbTBXExpandablePanel.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
 ControlStyle := ControlStyle + [csOpaque];
 DoubleBuffered := true;
 Color := clBtnFace;
 ParentColor := true;
 Height := 150;
 Width := 150;
 FOHeight := 150;
 FCaptionHeight := 18;
 FArrowPos := apWithCaption;
 FArrowAlign := aaLeft;
 FShowArrow := true;
 IsExpanded := true;
 MouseOverButton := false;
 MouseIsDown := false;
 FHeaderPaintStyle := hpsNormal;
 FPanelPaintStyle := hpsNormal;
end;

destructor TmbTBXExpandablePanel.Destroy;
begin
 inherited Destroy;
end;

procedure TmbTBXExpandablePanel.SetOriginalHeight(h: integer);
begin
 FOHeight := h;
 Height := h;
end;

procedure TmbTBXExpandablePanel.SetArrowPos(pos: TPanelArrowPosition);
begin
 FArrowPos := pos;
 Invalidate;
end;

procedure TmbTBXExpandablePanel.SetArrowAlign(al: TPanelArrowAlignment);
begin
 FArrowAlign := al;
 Invalidate;
end;

function TmbTBXExpandablePanel.GetBtnRect: TRect;
begin
case FArrowPos of
  apTop:
   case FArrowAlign of
    aaLeft: BtnRect := Rect(7, 5+1, 17, 15+1);
    aaRight: BtnRect := Rect(Width - 17, 5+1, Width - 7, 15+1);
   end;
  apMiddle:
   case FArrowAlign of
    aaLeft: BtnRect := Rect(7, (FCaptionHeight - 10) div 2 + 3+1, 17, (FCaptionHeight - 10) div 2 + 13+1);
    aaRight: BtnRect := Rect(Width - 17, (FCaptionHeight - 10) div 2 + 3+1, Width - 7, (FCaptionHeight - 10) div 2 + 13+1);
   end;
  apBottom:
   case FArrowAlign of
    aaLeft: BtnRect := Rect(7, (FCaptionHeight + 3+1) - 13, 17, (FCaptionHeight + 3) - 3+1);
    aaRight: BtnRect := Rect(Width - 17, (FCaptionHeight + 3) - 13+1, Width - 7, (FCaptionHeight + 3) - 3+1);
   end;
  apWithCaption:
   case FArrowAlign of
    aaLeft: BtnRect := Rect(7, captiontop + 5+1, 17, captiontop + 15+1);
    aaRight: BtnRect := Rect(Width - 17, captiontop + 5+1, Width - 7, captiontop + 15+1);
   end;
 end;
 Result := BtnRect;
end;

procedure TmbTBXExpandablePanel.SetItemInfo(state: string; var IInfo: TTBXItemInfo);
begin
   FillChar(IInfo, SizeOf(TTBXItemInfo), 0);
   IInfo.ViewType := TVT_NORMALTOOLBAR;
   IInfo.ItemOptions := IO_APPACTIVE or IO_TOOLBARSTYLE;
   IInfo.Enabled := Enabled;
   IInfo.IsVertical := false;

   if state = 'normal' then
    begin
     IInfo.Pushed := false;
     IInfo.Selected := false;
     IInfo.HoverKind := hkNone;
    end;
   if state = 'over' then
    begin
     IInfo.Pushed := false;
     IInfo.Selected := false;
     IInfo.HoverKind := hkMouseHover;
    end;
   if state = 'down' then
    begin
     IInfo.Pushed := true;
     IInfo.Selected := false;
     IInfo.HoverKind := hkMouseHover;
    end;
   if state = 'checked' then
    begin
     IInfo.Pushed := false;
     IInfo.Selected := true;
     IInfo.HoverKind := hkNone;
    end;
   if state = 'checked-hot' then
    begin
     IInfo.Pushed := false;
     IInfo.Selected := true;
     IInfo.HoverKind := hkMouseHover;
    end;
   if state = 'checked-down' then
    begin
     IInfo.Pushed := true;
     IInfo.Selected := true;
     IInfo.HoverKind := hkNone;
    end;
end;

function PaintPanelBack(Canvas: TCanvas; var R: TRect; cl: TmbTBXExpandablePanel; var FCaptionHeight: integer): integer;
var
 ii: TTBXItemInfo;
 rgn: HRGN;
 CaptionStr: TStringList;
begin
 Result := 3;

 if (bb = nil) or (bb.Width <> cl.Width) or (bb.Height <> cl.Height) or (tn <> CurrentTheme.Name) or (cn <> cl.Caption) then
  begin
   tn := CurrentTheme.Name;
   if bb = nil then
    begin
     bb := TBitmap.Create;
     bb.PixelFormat := pf32bit;
    end;
   bb.Height := cl.Height;
   bb.Width := cl.Width;
   bb.Canvas.CopyRect(bb.Canvas.ClipRect, canvas, canvas.ClipRect);
  end
 else
  begin
   Canvas.Draw(0, 0, bb);
   Exit;
  end;

 // paint back & frame
 DrawParentBackground(cl, bb.Canvas.Handle, R);

 if cl.PanelPaintStyle = hpsNormal then
  begin
   cl.SetItemInfo('over', ii);
   CurrentTheme.PaintFrame(bb.Canvas, R, ii);
   InflateRect(R, -1, -1);
  end;

 case cl.PanelPaintStyle of
  hpsNormal: cl.SetItemInfo('normal', ii);
  hpsHot: cl.SetItemInfo('over', ii);
  hpsDown: cl.SetItemInfo('down', ii);
  hpsChecked: cl.SetItemInfo('checked', ii);
  hpsCheckedHot: cl.SetItemInfo('checked-hot', ii);
  hpsCheckedDown: cl.SetItemInfo('checked-down', ii);
 end;

 rgn := CreateRectRgnIndirect(R);
 SelectClipRgn(bb.Canvas.Handle, rgn);

 InflateRect(R, ABS(cl.Height-cl.Width)+10, 10);
 CurrentTheme.PaintBackgnd(bb.canvas, R, R, R, CurrentTheme.GetViewColor(TVT_NORMALTOOLBAR), false, TVT_NORMALTOOLBAR);
 CurrentTheme.PaintDock(bb.canvas, R, R, DP_TOP);
 if CurrentTheme.PaintDockBackground then
  CurrentTheme.PaintBackgnd(bb.canvas, R, R, R, CurrentTheme.GetViewColor(TVT_NORMALTOOLBAR), false, TVT_NORMALTOOLBAR)
 else
  CurrentTheme.PaintDock(bb.canvas, R, R, DP_TOP);
 InflateRect(R, -ABS(cl.Height-cl.Width)-10, -10);

 DeleteObject(rgn);

 if cl.PanelPaintStyle <> hpsNormal then
  CurrentTheme.PaintButton(bb.Canvas, R, ii);

 CaptionStr := TStringList.Create;
 try
  //calc caption space
  if cl.Caption <> '' then
   begin
    CaptionStr.Text := cl.Caption;
    if FCaptionHeight <> CaptionStr.Count * bb.Canvas.TextHeight(cl.Caption) then
     FCaptionHeight := CaptionStr.Count * bb.Canvas.TextHeight(cl.Caption) + 4;
    Result := (FCaptionHeight - (CaptionStr.Count * bb.Canvas.TextHeight(cl.Caption))) div 2;
   end
  else
   begin
    FCaptionHeight := 18;
    Result := (FCaptionHeight - bb.Canvas.TextHeight('l')) div 2;
   end;
  //paint header
  R := Rect(3, 3, cl.Width - 3, 6 + FCaptionHeight);
 finally
  CaptionStr.Free;
 end;

 if cl.HeaderPaintStyle = hpsNormal then
  begin
   cl.SetItemInfo('over', ii);
   CurrentTheme.PaintFrame(bb.Canvas, R, ii);
   InflateRect(R, -1, -1);
  end;

 case cl.HeaderPaintStyle of
  hpsNormal: cl.SetItemInfo('normal', ii);
  hpsHot: cl.SetItemInfo('over', ii);
  hpsDown: cl.SetItemInfo('down', ii);
  hpsChecked: cl.SetItemInfo('checked', ii);
  hpsCheckedHot: cl.SetItemInfo('checked-hot', ii);
  hpsCheckedDown: cl.SetItemInfo('checked-down', ii);
 end;

 rgn := CreateRectRgnIndirect(R);
 SelectClipRgn(bb.Canvas.Handle, rgn);

 InflateRect(R, ABS(cl.Height-cl.Width)+10, 10);
 CurrentTheme.PaintBackgnd(bb.canvas, R, R, R, CurrentTheme.GetViewColor(TVT_NORMALTOOLBAR), false, TVT_NORMALTOOLBAR);
 CurrentTheme.PaintDock(bb.canvas, R, R, DP_TOP);
 if CurrentTheme.PaintDockBackground then
  CurrentTheme.PaintBackgnd(bb.canvas, R, R, R, CurrentTheme.GetViewColor(TVT_NORMALTOOLBAR), false, TVT_NORMALTOOLBAR)
 else
  CurrentTheme.PaintDock(bb.canvas, R, R, DP_TOP);
 InflateRect(R, -ABS(cl.Height-cl.Width)-10, -10);

 DeleteObject(rgn);

 if cl.HeaderPaintStyle <> hpsNormal then
  CurrentTheme.PaintButton(bb.Canvas, R, ii);

 Canvas.Draw(0, 0, bb);
end;

procedure TmbTBXExpandablePanel.Paint;
var
 ii, ci: TTBXItemInfo;
 C: TRect;
begin
 if IsExpanded then
  Height := FOHeight
 else
  Height := FCaptionHeight + 6+3;
 // paint back & frame
 C := ClientRect;
 captiontop := PaintPanelBack(Canvas, C, Self, FCaptionHeight);

 //paint caption
 SetItemInfo('normal', ci);
 if FCaption <> '' then
  CurrentTheme.PaintCaption(Canvas, Rect(3, 3 + captiontop, Width - 3, FCaptionHeight + captiontop + 6), ci, FCaption, 1, false);
 //paint button
 if FShowArrow then
  begin
   if MouseOverButton then
    begin
     if MouseIsDown then
      SetItemInfo('down', ii)
     else
      SetItemInfo('over', ii);
    end
   else
    SetItemInfo('normal', ii);
   CurrentTheme.PaintButton(Canvas, GetBtnRect, ii);
   //paint arrow
   CurrentTheme.PaintDropDownArrow(Canvas, GetBtnRect, ii);
  end;
end;

procedure TmbTBXExpandablePanel.SetHeaderPaintStyle(h: THeaderPaintStyle);
begin
 FHeaderPaintStyle := h;
 Invalidate;
end;

procedure TmbTBXExpandablePanel.SetPanelPaintStyle(h: THeaderPaintStyle);
begin
 FPanelPaintStyle := h;
 Invalidate;
end;

procedure TmbTBXExpandablePanel.AdjustClientRect(var Rect: TRect);
begin
  inherited AdjustClientRect(Rect);
  Rect.Top := Rect.Top + 6 + FCaptionHeight;
end;

procedure TmbTBXExpandablePanel.SetCaption(c: string);
begin
 FCaption := c;
 Invalidate;
end;

procedure TmbTBXExpandablePanel.Resize;
begin
 if (Height <> FCaptionHeight + 6+3) and (IsExpanded = true) then
  FOHeight := Height;
 Invalidate;
 if Assigned(OnResize) then
  OnResize(self);
end;

procedure TmbTBXExpandablePanel.CMMouseLeave(var Message: TMessage);
begin
 MouseOverButton := false;
 MouseIsDown := false;
 Invalidate;
end;

procedure TmbTBXExpandablePanel.CMMouseEnter(var Message: TMessage);
begin
 Invalidate;
end;

procedure TmbTBXExpandablePanel.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
 MouseOverButton := PointInRect(Point(x, y), GetBtnRect);
 Invalidate;
 if Assigned(OnMouseMove) then
  OnMouseMove(self, shift, x, y);
end;

procedure TmbTBXExpandablePanel.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 MouseIsDown := PointInRect(Point(x, y), GetBtnRect);
 Invalidate;
 if Assigned(OnMouseDown) then
  OnMouseDown(self, button, shift, x, y);
end;

procedure TmbTBXExpandablePanel.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 MouseIsDown := false;
 if PointInRect(Point(x, y), GetBtnRect) then
  begin
   MouseOverButton := true;
   BtnClick;
  end
 else
  MouseOverButton := false;
 if (PointInRect(Point(x, y), Rect(3, 3, Width - 3, 3 + FCaptionHeight)) = true) and (PointInRect(Point(x, y), GetBtnRect) = false) then
  begin
   MouseOverButton := false;
   CaptionClick;
  end;
 Invalidate;
 if Assigned(OnMouseUp) then
  OnMouseUp(self, button, shift, x, y);
end;

function TmbTBXExpandablePanel.PointInRect(aPoint: TPoint; aRect: TRect): boolean;
begin
 result := (aPoint.X > aRect.Left) and (aPoint.X < aRect.Right) and (aPoint.Y > aRect.Top) and (aPoint.Y < aRect.Bottom);
end;

procedure TmbTBXExpandablePanel.BtnClick;
begin
if FShowArrow then
 begin
  if Height = FCaptionHeight + 6+3 then
   begin
    Height := FOHeight;
    IsExpanded := true;
    Expand;
   end
  else
   begin
    Height := FCaptionHeight + 6+3;
    IsExpanded := false;
    Collapse;
   end;
  if Assigned(FOnBtnClick) then
   FOnBtnClick(self);
 end;
end;

procedure TmbTBXExpandablePanel.CaptionClick;
begin
if Assigned(FOnCaptionClick) then
 FOnCaptionClick(Self);
end;

procedure TmbTBXExpandablePanel.Expand;
begin
if Assigned(FOnExpand) then
 FOnExpand(Self);
end;

procedure TmbTBXExpandablePanel.Collapse;
begin
if Assigned(FOnCollapse) then
 FOnCollapse(Self);
end;

procedure TmbTBXExpandablePanel.SetExpanded(isexp: boolean);
begin
 IsExpanded := isexp;
 Invalidate;
 if isexp then
  Expand
 else
  Collapse;
 Invalidate;
end;

procedure TmbTBXExpandablePanel.SetShowArrow(show: boolean);
begin
 FShowArrow := show;
 Invalidate;
end;

procedure TmbTBXExpandablePanel.WMEraseBkgnd(var Message: TWMEraseBkgnd);
begin
 Message.Result := 1;
 EraseTBXBackground(Message.DC, ClientRect);
end;

procedure TmbTBXExpandablePanel.EraseTBXBackground(DC: HDC; ARect: TRect);
var
  ACanvas: TCanvas;
  B: TBitmap;
  R: TRect;
begin
  if (csDestroying in ComponentState) then Exit;

  R := Rect(0, 0, ARect.Right - ARect.Left, ARect.Bottom - ARect.Top);
  B := TBitmap.Create;
  ACanvas := TCanvas.Create;
  try
    ACanvas.Handle := DC;
    B.PixelFormat := pf32bit;
    B.Width := R.Right;
    B.Height := R.Bottom;

    PaintPanelBack(B.Canvas, R, Self, FCaptionHeight);

    ACanvas.Draw(ARect.Left, ARect.Top, B);
  finally
    B.Free;
    ACanvas.Handle := 0;
    ACanvas.Free;
  end;
end;

procedure TmbTBXExpandablePanel.CMEnabledChanged(var Message: TMessage);
begin
 inherited;
 Invalidate;
end;

end.
