unit mbTBXGroupBox;

{$I mxs.inc}

interface

uses
  SysUtils, Classes, Controls, Windows, Messages, Graphics, TBX, StdCtrls,
  TBXUtils,
 {$IFDEF DELPHI_7_UP}
  Themes,
 {$ENDIF}
  TBXThemes;

type
  TmbTBXGroupBox = class(TGroupBox)
  private
   FThemeBack: boolean;
   FThemed: boolean;
   FGrad: boolean;

   procedure SetGrad(Value: boolean);
   procedure SetThemed(Value: boolean);
   procedure SetThemeBack(t: boolean);
   procedure EraseTBXBackground(DC: HDC; ARect: TRect);
  protected
   procedure Paint; override;
   procedure TBMThemeChange(var Message: TMessage); message TBM_THEMECHANGE;
   procedure Resize; override;
   procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
   procedure WMEraseBkgnd(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
  public
   constructor Create(AOwner: TComponent); override;
   destructor Destroy; override;
  published
   property Themed: boolean read FThemed write SetThemed default true;
   property ThemedBackground: boolean read FThemeBack write SetThemeBack default false;
   property GradientBorder: boolean read FGrad write SetGrad default true;
  end;

implementation

var
 bb: TBitmap;
 tn, cn: string;

constructor TmbTBXGroupBox.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
 AddThemeNotification(Self);
 DoubleBuffered := true;
 FThemeBack := false;
 FThemed := true;
 FGrad := true;
end;

destructor TmbTBXGroupBox.Destroy;
begin
 RemoveThemeNotification(Self);
 inherited Destroy;
end;

procedure TmbTBXGroupBox.TBMThemeChange(var Message: TMessage);
begin
inherited;
if Message.WParam = TSC_VIEWCHANGE then
 Invalidate;
end;

procedure SetItemInfo(var IInfo: TTBXItemInfo);
begin
   FillChar(IInfo, SizeOf(TTBXItemInfo), 0);
   IInfo.ViewType := TVT_NORMALTOOLBAR;
   IInfo.ItemOptions := IO_APPACTIVE or IO_TOOLBARSTYLE;
   IInfo.Enabled := True;
   IInfo.Pushed := false;
   IInfo.Selected := false;
   IInfo.HoverKind := hkMouseHover;
end;

procedure PaintGroup(Canvas: TCanvas; R: TRect; cl: TmbTBXGroupBox);
var
 h, w: integer;
 c: string;
 T: TRect;
 ii: TTBXItemInfo;
begin

 if (bb = nil) or (bb.Width <> cl.Width) or (bb.Height <> cl.Height) or (tn <> CurrentTheme.Name) or (cn <> cl.Caption) then
  begin
   tn := CurrentTheme.Name;
   cn := cl.Caption;
   if bb = nil then
    begin
     bb := TBitmap.Create;
     bb.PixelFormat := pf32bit;
    end;
   bb.Height := cl.Height;
   bb.Width := cl.Width;
   bb.Canvas.CopyRect(bb.Canvas.ClipRect, canvas, canvas.ClipRect);
  end
 else
  begin
   Canvas.Draw(0, 0, bb);
   Exit;
  end;

 h := Canvas.TextHeight('Ip');
 if cl.Caption <> '' then
  c := cl.Caption
 else
  c := ' ';
 w := Canvas.TextWidth(c);
 T := Rect(6, 0, w + 6, 1 + h);
 if cl.ThemedBackground then
  begin
   CurrentTheme.PaintBackgnd(bb.canvas, R, R, R, CurrentTheme.GetViewColor(TVT_NORMALTOOLBAR), false, TVT_NORMALTOOLBAR);
   CurrentTheme.PaintDock(bb.canvas, R, R, DP_TOP);
   if CurrentTheme.PaintDockBackground then
    CurrentTheme.PaintBackgnd(bb.canvas, R, R, R, CurrentTheme.GetViewColor(TVT_NORMALTOOLBAR), false, TVT_NORMALTOOLBAR)
   else
    CurrentTheme.PaintDock(bb.canvas, R, R, DP_TOP);
  end;
 // set info
 SetItemInfo(ii);
 ii.Enabled := cl.Enabled;
 ExcludeClipRect(bb.Canvas.Handle, T.Left - 2, T.TOp, T.Right, T.Bottom);
 ExcludeClipRect(bb.Canvas.Handle, 1, (h div 2) + 1, R.Right - 1, R.Bottom - 1);
 Inc(R.Top, h div 2);
 // paint the border
 CurrentTheme.PaintBackgnd(bb.canvas, R, R, R, CurrentTheme.GetViewColor(TVT_NORMALTOOLBAR), false, TVT_NORMALTOOLBAR);
 CurrentTheme.PaintDock(bb.canvas, R, R, DP_TOP);
 if CurrentTheme.PaintDockBackground then
  CurrentTheme.PaintBackgnd(bb.canvas, R, R, R, clBtnShadow{CurrentTheme.GetViewColor(TVT_NORMALTOOLBAR)}, false, TVT_NORMALTOOLBAR)
 else
  CurrentTheme.PaintDock(bb.canvas, R, R, DP_TOP);
 if not cl.GradientBorder then
  CurrentTheme.PaintFrame(bb.canvas, R, ii);
 Canvas.Draw(0, 0, bb);
end;

procedure TmbTBXGroupBox.Paint;
var
 R, T: TRect;
 h, w: integer;
 c: string;
 ii: TTBXItemInfo;
 rgn: hrgn;
begin
 if not FThemed then
  begin
   inherited;
   Exit;
  end;
 R := ClientRect;
 Canvas.Brush.Style := bsClear;
 Canvas.Font := Font;
 PaintGroup(Canvas, R, Self);

 h := Canvas.TextHeight('Ip');
 if Caption <> '' then
  c := Caption
 else
  c := ' ';
 w := Canvas.TextWidth(c);
 T := Rect(6, 0, w + 6, 1 + h);
 SetItemInfo(ii);

 //hide double border
 Inc(R.Top, h div 2);
 with R do
  begin
   Top := 0;
   Left := 0;
   Right := Width;
   Bottom := h div 2
  end;
 rgn := CreateRectRgnIndirect(R);
 SelectObject(Canvas.Handle, rgn);

 DrawParentBackground(Self, Canvas.Handle, R);

 DeleteObject(rgn);

 rgn := CreateRectRgnIndirect(clientRect);
 SelectObject(Canvas.Handle, rgn);

 Canvas.Font.Color := CurrentTheme.GetItemTextColor(ii);
 Canvas.Font := Font;
 DrawText(Canvas.Handle, PChar(c), Length(c), T, DT_SINGLELINE);

 DeleteObject(rgn);
end;

procedure TmbTBXGroupBox.Resize;
begin
 Invalidate;
 if Assigned(OnResize) then
  OnResize(Self);
end;

procedure TmbTBXGroupBox.CMEnabledChanged(var Message: TMessage);
begin
 inherited;
 Invalidate;
end;

procedure TmbTBXGroupBox.SetThemeBack(t: boolean);
begin
 FThemeBack := t;
 invalidate;
end;

procedure TmbTBXGroupBox.SetThemed(Value: boolean);
begin
 FThemed := Value;
 Invalidate;
end;

procedure TmbTBXGroupBox.SetGrad(Value: boolean);
begin
 FGrad := Value;
 Invalidate;
end;

procedure TmbTBXGroupBox.WMEraseBkgnd(var Message: TWMEraseBkgnd);
begin
 Message.Result := 1;
 EraseTBXBackground(Message.DC, ClientRect);
end;

procedure TmbTBXGroupBox.EraseTBXBackground(DC: HDC; ARect: TRect);
var
  ACanvas: TCanvas;
  B: TBitmap;
  R: TRect;
begin
  if (csDestroying in ComponentState) then Exit;

  R := Rect(0, 0, ARect.Right - ARect.Left, ARect.Bottom - ARect.Top);
  B := TBitmap.Create;
  ACanvas := TCanvas.Create;
  try
    ACanvas.Handle := DC;
    B.PixelFormat := pf32bit;
    B.Width := R.Right;
    B.Height := R.Bottom;

    DrawParentBackground(Self, B.Canvas.Handle, R);
     if FThemeBack then
    PaintGroup(B.Canvas, R, Self);

    ACanvas.Draw(ARect.Left, ARect.Top, B);
  finally
    B.Free;
    ACanvas.Handle := 0;
    ACanvas.Free;
  end;
end;

end.
