unit mbTBXHeaderControl;

{$I mxs.inc}

interface

uses
  SysUtils, Classes, Controls, ComCtrls, Windows, Messages, Graphics, TBX,
  TBXUtils, TBXThemes, CommCtrl;

type
  TMouseLoc = (mlNone, mlOver, mlDown);
  {$IFDEF DELPHI_7_UP}
  TmbTBXCustomHeaderControl = class(TCustomHeaderControl)
  {$Else}
  TmbTBXCustomHeaderControl = class(THeaderControl)
  {$ENDIF}

  private
   c: TCanvas;
   FThemed: boolean;
   FHotTrack: boolean;

   procedure SetHotTrack(Value: boolean);
   procedure SetThemed(Value: boolean);
  protected
   FMouseLoc: TMouseLoc;
   mx, my: integer;
   FMouseOver: boolean;

   procedure WMEraseBkgnd(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
   procedure WMPaint (var Msg: TWMPaint); message WM_PAINT;
   procedure TBMThemeChange(var Message: TMessage); message TBM_THEMECHANGE;
   procedure Resize; override;
   procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
   procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
   procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
   procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
   procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
   procedure CMGotFocus(var Message: TCMGotFocus); message CM_ENTER;
   procedure CMLostFocus(var Message: TCMLostFocus); message CM_EXIT;
   procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
  public
   constructor Create(AOwner: TComponent); override;
   destructor Destroy; override;

   property Themed: boolean read FThemed write SetThemed default true;
   property HotTracking: boolean read FHotTrack write SetHotTrack default true;
  end;

 TDrawSectionEvent = procedure(HeaderControl: THeaderControl; Section: THeaderSection; const Rect: TRect; Pressed: Boolean) of object;
 TSectionNotifyEvent = procedure(HeaderControl: THeaderControl; Section: THeaderSection) of object;
 TSectionTrackEvent = procedure(HeaderControl: THeaderControl; Section: THeaderSection; Width: Integer; State: TSectionTrackState) of object;
 {$IFDEF DELPHI_7_UP}
 THCCreateSectionClassEvent = procedure(Sender: THeaderControl; var SectionClass: THeaderSectionClass) of object;
 {$ENDIF}
 TmbTBXHeaderControl = class(TmbTBXCustomHeaderControl)
  private
   function GetOnDrawSection: TDrawSectionEvent;
   function GetOnSectionClick: TSectionNotifyEvent;
   function GetOnSectionResize: TSectionNotifyEvent;
   function GetOnSectionTrack: TSectionTrackEvent;
   procedure SetOnDrawSection(const Value: TDrawSectionEvent);
   procedure SetOnSectionClick(const Value: TSectionNotifyEvent);
   procedure SetOnSectionResize(const Value: TSectionNotifyEvent);
   procedure SetOnSectionTrack(const Value: TSectionTrackEvent);
  protected
   procedure CNDrawItem(var Message: TWMDrawItem); message CN_DRAWITEM;
   procedure DrawSection(Section: THeaderSection; const Rect: TRect; Pressed: Boolean); override;
  published
   property Align default alTop;
   property Anchors;
   property BiDiMode;
   property DragCursor;
   property DragKind;
   property DragMode;
   property DragReorder;
   property Enabled;
   property Font;
   property FullDrag default True;
   property HotTrack default False;
   property Images;
   property Constraints;
   property Sections;
   property Style default hsButtons;
   property ShowHint;
   property ParentBiDiMode;
   property ParentFont;
   property ParentShowHint;
   property PopupMenu;
   property Visible;
   property Themed;
   property HotTracking;

   property OnContextPopup;
   {$IFDEF DELPHI_7_UP}
   property OnCreateSectionClass;
   {$ENDIF}
   property OnDragDrop;
   property OnDragOver;
   property OnEndDock;
   property OnEndDrag;
   property OnMouseDown;
   property OnMouseMove;
   property OnMouseUp;
   property OnResize;
   property OnDrawSection: TDrawSectionEvent read GetOnDrawSection write SetOnDrawSection;
   property OnSectionClick: TSectionNotifyEvent read GetOnSectionClick write SetOnSectionClick;
   property OnSectionResize: TSectionNotifyEvent read GetOnSectionResize write SetOnSectionResize;
   property OnSectionTrack: TSectionTrackEvent read GetOnSectionTrack write SetOnSectionTrack;
   property OnSectionDrag;
   property OnSectionEndDrag;
   property OnStartDock;
   property OnStartDrag;
  end;

implementation
{$IFDEF DELPHI_7_UP}
uses Types;
{$ENDIF}

constructor TmbTBXCustomHeaderControl.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
 AddThemeNotification(Self);
 DoubleBuffered := true;
 ControlStyle := ControlStyle + [csOpaque];
 c := TCanvas.Create;
 FMouseLoc := mlNone;
 FMouseOver := false;
 Style := hsButtons;
 FThemed := true;
 FHotTrack := true;
end;

destructor TmbTBXCustomHeaderControl.Destroy;
begin
 c.free;
 RemoveThemeNotification(Self);
 inherited Destroy;
end;

procedure TmbTBXCustomHeaderControl.TBMThemeChange(var Message: TMessage);
begin
if Message.WParam = TSC_VIEWCHANGE then
 Invalidate;
end;

procedure SetItemInfo(state: string; var IInfo: TTBXItemInfo; var EInfo: TTBXEditInfo);
begin
   FillChar(IInfo, SizeOf(TTBXItemInfo), 0);
   IInfo.ViewType := TVT_NORMALTOOLBAR;
   IInfo.ItemOptions := IO_APPACTIVE or IO_TOOLBARSTYLE;
   IInfo.Enabled := True;
   IInfo.IsVertical := false;

   FillChar(EInfo, SizeOf(TTBXEditInfo), 0);
   EInfo.RightBtnWidth := 0;
   EInfo.RightBtnInfo.ButtonType := EBT_DROPDOWN;

   if state = 'normal' then
   begin
     IInfo.Pushed := false;
     IInfo.Selected := false;
     IInfo.HoverKind := hkNone;
     EInfo.RightBtnInfo.ButtonState := 0;
   end;
   if state = 'over' then
   begin
     IInfo.Pushed := false;
     IInfo.Selected := false;
     IInfo.HoverKind := hkMouseHover;
     EInfo.RightBtnInfo.ButtonState := EBDS_HOT;
   end;
   if state = 'down' then
   begin
     IInfo.Pushed := true;
     IInfo.Selected := false;
     IInfo.HoverKind := hkMouseHover;
     EInfo.RightBtnInfo.ButtonState := EBDS_PRESSED;
   end;
end;

procedure TmbTBXCustomHeaderControl.WMPaint(var Msg: TWMPaint);
var
 DC: HDC;
 ii, savedinfo: TTBXItemInfo;
 ed: TTBXEditInfo;
 R: TRect;
 i, k: integer;
begin
 inherited;
 if not FThemed then Exit;
 DC := GetWindowDC(Handle);
 GetWindowRect(Handle, R);
 OffsetRect(R, -R.Left, -R.Top);
 C.Handle := DC;
 // calculate section space
 k := 0;
 for i := 0 to Sections.Count - 1 do
  k := k + Sections.Items[i].Width;
 // exclude section space
 ExcludeClipRect(DC, 2, 2, (R.Right - 2) - (R.Right - k), R.Bottom - 2);
 // set info
 case FMouseLoc of
  mlNone: SetItemInfo('normal', ii, ed);
  mlOver:
   if FHotTrack then
    SetItemInfo('over', ii, ed)
   else
    SetItemInfo('normal', ii, ed);
  mlDown: SetItemInfo('down', ii, ed);
 end;
 if (csDesigning in ComponentState) or (Enabled = false) then SetItemInfo('normal', ii, ed);
 ii.Enabled := Enabled;
 // paint the border
 CurrentTheme.PaintBackgnd(c, R, R, R, CurrentTheme.GetViewColor(TVT_NORMALTOOLBAR), false, VT_TOOLBAR);
 CurrentTheme.PaintDock(c, R, R, DP_TOP);
 // paint leftovers
 savedinfo := ii;
 SetItemInfo('normal', ii, ed);
 CurrentTheme.PaintButton(Canvas, R, ii);
 ExcludeClipRect(DC, 2, 2, R.Right - 2, R.Bottom - 2);
 ii := savedinfo;
 CurrentTheme.PaintButton(c, R, ii);
 // clean up
 C.Handle := 0;
 ReleaseDC(Handle, DC);
 // set section paint mode
 for i := 0 to Sections.Count - 1 do
  if Sections.Items[i].Style <> hsOwnerDraw then
   Sections.Items[i].Style := hsOwnerDraw;
end;

procedure TmbTBXCustomHeaderControl.Resize;
begin
 Invalidate;
 if Assigned(OnResize) then
  OnResize(Self);
end;

procedure TmbTBXCustomHeaderControl.CMMouseEnter(var Message: TMessage);
begin
  if not Focused then
   begin
    FMouseOver := true;
    FMouseLoc := mlOver;
    if FHotTrack then
    Invalidate;
   end;
end;

procedure TmbTBXCustomHeaderControl.CMMouseLeave(var Message: TMessage);
begin
 if not Focused then
  begin
   FMouseOver := false;
   FMouseLoc := mlNone;
   if FHotTrack then
   Invalidate;
  end;
end;

procedure TmbTBXCustomHeaderControl.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
 if not Focused then
  begin
   FMouseOver := true;
   FMouseLoc := mlOver;
   if FHotTrack then
   Invalidate;
  end;
 mx := x;
 my := y;
 if Assigned(OnMouseMove) then
  OnMouseMove(Self, Shift, x, y);
end;

procedure TmbTBXCustomHeaderControl.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 begin
  FMouseLoc := mlDown;
  Invalidate;
 end;
 if Assigned(OnMouseDown) then
  OnMouseDown(Self, Button, Shift, x, y);
end;

procedure TmbTBXCustomHeaderControl.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 if not Focused then
  begin
   if FMouseOver then
    FMouseLoc := mlOver
   else
    FMouseLoc := mlNone;
   Invalidate;
  end;
 if Assigned(OnMouseUp) then
  OnMouseUp(Self, Button, Shift, x, y);
end;

procedure TmbTBXCustomHeaderControl.CMGotFocus(var Message: TCMGotFocus);
begin
 FMouseLoc := mlOver;
 Invalidate;
end;

procedure TmbTBXCustomHeaderControl.CMLostFocus(var Message: TCMLostFocus);
begin
 FMouseLoc := mlNone;
 Invalidate;
end;

procedure TmbTBXCustomHeaderControl.CMEnabledChanged(var Message: TMessage);
begin
 inherited;
 Invalidate;
end;

procedure TmbTBXCustomHeaderControl.SetThemed(Value: boolean);
var
 i: integer;
begin
 FThemed := Value;
 if not Value then
  for i := 0 to Sections.Count - 1 do
   if Sections.Items[i].Style <> hsText then
    Sections.Items[i].Style := hsText;
 Invalidate;
end;

{TmbTBXHeaderControl}

function TmbTBXHeaderControl.GetOnDrawSection: TDrawSectionEvent;
begin
  Result := TDrawSectionEvent(inherited OnDrawSection);
end;

function TmbTBXHeaderControl.GetOnSectionClick: TSectionNotifyEvent;
begin
  Result := TSectionNotifyEvent(inherited OnSectionClick);
end;

function TmbTBXHeaderControl.GetOnSectionResize: TSectionNotifyEvent;
begin
  Result := TSectionNotifyEvent(inherited OnSectionResize);
end;

function TmbTBXHeaderControl.GetOnSectionTrack: TSectionTrackEvent;
begin
  Result := TSectionTrackEvent(inherited OnSectionTrack);
end;

procedure TmbTBXHeaderControl.SetOnDrawSection(const Value: TDrawSectionEvent);
begin
  {$IFDEF DELPHI_7_UP}
  inherited OnDrawSection := TCustomDrawSectionEvent(Value);
  {$ELSE}
  inherited OnDrawSection := TDrawSectionEvent(Value);
  {$ENDIF}
end;

procedure TmbTBXHeaderControl.SetOnSectionClick(const Value: TSectionNotifyEvent);
begin
  {$IFDEF DELPHI_7_UP}
  inherited OnSectionClick := TCustomSectionNotifyEvent(Value);
  {$ELSE}
  inherited OnSectionClick := TSectionNotifyEvent(Value);
  {$ENDIF}
end;

procedure TmbTBXHeaderControl.SetOnSectionResize(const Value: TSectionNotifyEvent);
begin
  {$IFDEF DELPHI_7_UP}
  inherited OnSectionResize := TCustomSectionNotifyEvent(Value);
  {$ELSE}
  inherited OnSectionResize := TSectionNotifyEvent(Value);
  {$ENDIF}
end;

procedure TmbTBXHeaderControl.SetOnSectionTrack(const Value: TSectionTrackEvent);
begin
  {$IFDEF DELPHI_7_UP}
  inherited OnSectionTrack := TCustomSectionTrackEvent(Value);
  {$ELSE}
  inherited OnSectionTrack := TSectionTrackEvent(Value);
  {$ENDIF}
end;

procedure TmbTBXHeaderControl.DrawSection(Section: THeaderSection; const Rect: TRect; Pressed: Boolean);
const
 AlignFlag: array[TAlignment] of Cardinal = (DT_LEFT, DT_RIGHT, DT_CENTER);
var
 ii: TTBXItemInfo;
 ed: TTBXEditInfo;
 IR: TRect;
 FImageWidth: integer;
begin
if not Themed then
 begin
  inherited;
  Exit;
 end;
FImageWidth := 0;
// draw
if not FMouseOver then
 SetItemInfo('normal', ii, ed)
else
 begin
  if not Pressed then
   begin
    if PtInRect(Rect, Point(mx, my)) and FHotTrack then
     SetItemInfo('over', ii, ed)
    else
     SetItemInfo('normal', ii, ed);
   end
  else
   SetItemInfo('down', ii, ed);
 end;
if (csDesigning in ComponentState) or (Enabled = false) then SetItemInfo('normal', ii, ed);
ii.Enabled := Enabled;
// paint the section
CurrentTheme.PaintFrame(Canvas, Rect, ii);
CurrentTheme.PaintBackgnd(Canvas, Rect, Rect, Rect, CurrentTheme.GetViewColor(TVT_NORMALTOOLBAR), false, TVT_NORMALTOOLBAR);
CurrentTheme.PaintDock(Canvas, Rect, Rect, DP_TOP);
if CurrentTheme.PaintDockBackground then
 CurrentTheme.PaintBackgnd(Canvas, Rect, Rect, Rect, CurrentTheme.GetViewColor(TVT_NORMALTOOLBAR), false, TVT_NORMALTOOLBAR)
else
 CurrentTheme.PaintDock(Canvas, Rect, Rect, DP_TOP);
CurrentTheme.PaintButton(Canvas, Rect, ii);
// paint image
if (Images <> nil) and (Section.ImageIndex > -1) then
 begin
  IR := Rect;
  IR.Top := ((IR.Bottom - IR.Top) - Images.Height) div 2;
  IR.Bottom := IR.Top + Images.Height;
  Inc(IR.Left, 2);
  FImageWidth := Images.Width;
  CurrentTheme.PaintImage(Canvas, IR, ii, Images, Section.ImageIndex);
 end
else
 IR := Rect;
// paint text
if Section.Text <> '' then
 begin
  Canvas.Brush.Style := bsClear;
  Canvas.Font := Font;
  C.Font.Color := CurrentTheme.GetItemTextColor(ii);
  Inc(IR.Left, FImageWidth + 4);
  DrawText(Canvas.Handle, PChar(Section.Text), Length(Section.Text), IR, DT_SINGLELINE or DT_VCENTER or DT_END_ELLIPSIS or AlignFlag[Section.Alignment]);
 end;
inherited;
end;

procedure TmbTBXHeaderControl.CNDrawItem(var Message: TWMDrawItem);
var
  SaveIndex: Integer;
begin
 if not Themed then
  begin
   inherited;
   Exit;
  end;
  with Message.DrawItemStruct^ do
  begin
    SaveIndex := SaveDC(hDC);
    Canvas.Lock;
    try
      Canvas.Handle := hDC;
      DrawSection(Sections[itemID], rcItem, itemState and ODS_SELECTED <> 0);
    finally
      Canvas.Handle := 0;
      Canvas.Unlock;
      RestoreDC(hDC, SaveIndex);
    end;
  end;
  Message.Result := 1;
end;

procedure TmbTBXCustomHeaderControl.WMEraseBkgnd(var Message: TWMEraseBkgnd);
begin
 if not Themed then
  begin
   inherited;
   Exit;
  end;
 Message.Result := 1;
end;

procedure TmbTBXCustomHeaderControl.SetHotTrack(Value: boolean);
begin
 if FHotTrack <> Value then
  begin
   FHotTrack := Value;
   Invalidate;
  end;
end;

end.
