{  This component is based on Maik Porkerts TFlatHint  }
{ component from the great FlatStyle package, I've only }
{       themed it, the original can be found at:        }
{                    www.flatstyle.de                   }
{ ----------------------------------------------------- }
{ 09-20-04 Dierk Schmid: added shadow *                 }
{ * Marko Binic - added shadow properties               }

unit mbTBXHint;

{$I mxs.inc}

interface

uses
  Classes, Windows, Graphics, Messages, Controls, Forms, SysUtils, TBX, TBXThemes,
  TBXUtils;

type
  THintPaintMode = (pmNormal, pmHot, pmDown, pmChecked, pmCheckedHot, pmCheckedDown);
  TArrowPos = (NE, NW, SE, SW);

  TmbTBXHint = class(TComponent)
  private
   FHintFont: TFont;
   FHintWidth: Integer;
   FShadowOpacity: integer;
   FShadowSize: integer;
   FOnShowHint: TShowHintEvent;
   FBidiMode: TBidiMode;
   FPaintMode: THintPaintMode;
   FArrowColor: TColor;
   FShadow: boolean;
   FBar: boolean;

   procedure SetHintFont (Value: TFont);
   procedure GetHintInfo (var HintStr: string; var CanShow: Boolean; var HintInfo: THintInfo);
   procedure SetBidiMode(const Value: TBidiMode);
  public
   constructor Create (AOwner: TComponent); override;
   destructor Destroy; override;
  published
   property PaintMode: THintPaintMode read FPaintMode write FPaintMode default pmHot;
   property MaxHintWidth: Integer read FHintWidth write FHintWidth default 200;
   property Font: TFont read FHintFont write SetHintFont;
   property OnShowHint: TShowHintEvent read FOnShowHint write FOnShowHint;
   property BidiMode: TBidiMode read FBidiMode write SetBidiMode default bdLeftToRight;
   property OverrideArrowColor: TColor read FArrowColor write FArrowColor default clNone;
   property ShadowSize: integer read FShadowSize write FShadowSize default 3;
   property ShadowOpacity: integer read FShadowOpacity write FShadowOpacity default 75;
   property UseShadow: boolean read FShadow write FShadow default true;
   property MenuBarAppearence: boolean read FBar write FBar default false;
  end;

  TmbTBXHintWindow = class(THintWindow)
  private
   FShadows: TShadows;
   FArrowPos: TArrowPos;
   FArrowPoint: TPoint;

   procedure DestroyShadow;
   procedure WMShowWindow(var Message: TWMShowWindow); message WM_SHOWWINDOW;
  protected
   procedure WMEraseBkgnd(var Msg: TWMEraseBkgnd); message WM_ERASEBKGND;
   procedure Paint; override;
   procedure CreateParams (var Params: TCreateParams); override;
   procedure SetItemInfo(state: string; var IInfo: TTBXItemInfo);
  public
   constructor Create(AOwner: TComponent); override;
   destructor Destroy; override;
   procedure ActivateHint (HintRect: TRect; const AHint: string); Override;
  end;

implementation

var
 FHint: TmbTBXHint;
 HintControl: TControl; // control the tooltip belongs to

constructor TmbTBXHint.Create (AOwner: TComponent);
begin
  inherited Create(AOwner);
  FHint := Self;
  if not (csDesigning in ComponentState) then
   begin
    HintWindowClass := TmbTBXHintWindow;
    with Application do
     begin
      ShowHint := not ShowHint;
      ShowHint := not ShowHint;
      OnShowHint := GetHintInfo;
      HintShortPause := 25;
      HintPause := 500;
      HintHidePause := 5000;
     end;
   end;
   FHintWidth := 200;
   FHintFont := TFont.Create;
   FPaintMode := pmHot;
   FArrowColor := clNone;
   FBidiMode := bdLeftToRight;
   FShadowOpacity := 75;
   FShadowSize := 3;
   FShadow := true;
   FBar := false;
end;

destructor TmbTBXHint.Destroy;
begin
  FHintFont.Free;
  inherited Destroy;
end;

procedure TmbTBXHint.SetHintFont (Value: TFont);
begin
  FHintFont.Assign(Value);
end;

procedure TmbTBXHint.GetHintInfo (var HintStr: string; var CanShow: Boolean; var HintInfo: THintInfo);
begin
  if Assigned(FOnShowHint) then
    FOnShowHint(HintStr, CanShow, HintInfo);
  HintControl := HintInfo.HintControl;
end;

procedure TmbTBXHint.SetBidiMode(const Value: TBidiMode);
begin
  if FBiDiMode <> Value then
    FBiDiMode := Value;
end;

{ TmbTBXHintWindow }

procedure TmbTBXHintWindow.CreateParams (var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  Params.Style := Params.Style - WS_BORDER;
end;

procedure TmbTBXHintWindow.SetItemInfo(state: string; var IInfo: TTBXItemInfo);
begin
   FillChar(IInfo, SizeOf(TTBXItemInfo), 0);
   if not FHint.FBar then
    IInfo.ViewType := TVT_NORMALTOOLBAR
   else
    IInfo.ViewType := TVT_MENUBAR;
   IInfo.ItemOptions := IO_APPACTIVE or IO_TOOLBARSTYLE;
   IInfo.Enabled := True;
   IInfo.IsVertical := false;

   if state = 'normal' then
   begin
     IInfo.Pushed := false;
     IInfo.Selected := false;
     IInfo.HoverKind := hkNone;
   end;
   if state = 'over' then
   begin
     IInfo.Pushed := false;
     IInfo.Selected := false;
     IInfo.HoverKind := hkMouseHover;
   end;
   if state = 'down' then
   begin
     IInfo.Pushed := true;
     IInfo.Selected := false;
     IInfo.HoverKind := hkMouseHover;
   end;
   if State = 'checked' then
   begin
     IInfo.Pushed := False;
     IInfo.Selected := True;
     IInfo.HoverKind := hkNone;
   end;
   if State = 'checked-hot' then
   begin
     IInfo.Pushed := False;
     IInfo.Selected := True;
     IInfo.HoverKind := hkMouseHover;
   end;
   if State = 'checked-down' then
   begin
     IInfo.Pushed := True;
     IInfo.Selected := True;
     IInfo.HoverKind := hkMouseHover;
   end;
end;

procedure TmbTBXHintWindow.Paint;
var
 ArrowRect, TextRect, R, TempRect: TRect;
 ii: TTBXItemInfo;
begin
  DestroyShadow;
  // Set the Rect's
  case FArrowPos of
    NW, SW:
      begin
        ArrowRect := Rect(ClientRect.Left + 1, ClientRect.Top + 1, ClientRect.Left + 15, ClientRect.Bottom - 1);
        TextRect  := Rect(ClientRect.Left + 15, ClientRect.Top + 1, ClientRect.Right - 1, ClientRect.Bottom - 1);
      end;
    NE, SE:
      begin
        ArrowRect := Rect(ClientRect.Right - 15, ClientRect.Top + 1, ClientRect.Right - 1, ClientRect.Bottom - 1);
        TextRect  := Rect(ClientRect.Left + 1, ClientRect.Top + 1, ClientRect.Right - 15, ClientRect.Bottom - 1);
      end;
  end;
  //back
  R := ClientRect;
  case FHint.FPaintMode of
   pmNormal: SetItemInfo('normal', ii);
   pmHot: SetItemInfo('over', ii);
   pmDown: SetItemInfo('down', ii);
   pmChecked: SetItemInfo('checked', ii);
   pmCheckedHot: SetItemInfo('checked-hot', ii);
   pmCheckedDown: SetItemInfo('checked-down', ii);
  end;
  CurrentTheme.PaintFrame(Canvas, R, ii);
  CurrentTheme.PaintBackgnd(Canvas, R, R, R, CurrentTheme.GetViewColor(TVT_NORMALTOOLBAR), false, TVT_NORMALTOOLBAR);
  CurrentTheme.PaintDock(Canvas, R, R, DP_TOP);
  if CurrentTheme.PaintDockBackground then
   CurrentTheme.PaintBackgnd(Canvas, R, R, R, CurrentTheme.GetViewColor(TVT_NORMALTOOLBAR), false, TVT_NORMALTOOLBAR)
  else
   CurrentTheme.PaintDock(Canvas, R, R, DP_TOP);
  CurrentTheme.PaintButton(Canvas, R, ii);
  // text back
  Canvas.Brush.Color := CurrentTheme.GetViewColor(VT_DOCKPANEL);
  Canvas.FillRect(TextRect);
  // DrawArrow
  case FArrowPos of
    NW: FArrowPoint := Point(ArrowRect.Left + 2, ArrowRect.Top + 2);
    NE: FArrowPoint := Point(ArrowRect.Right - 3, ArrowRect.Top + 2);
    SW: FArrowPoint := Point(ArrowRect.Left + 2, ArrowRect.Bottom - 3);
    SE: FArrowPoint := Point(ArrowRect.Right - 3, ArrowRect.Bottom - 3);
  end;
  if FHint.FArrowColor <> clNone then
   Canvas.Pen.Color := FHint.FArrowColor
  else
   Canvas.Pen.Color := CurrentTheme.GetItemTextColor(ii);
  case FArrowPos of
    NW: Canvas.Polyline([Point(FArrowPoint.x,     FArrowPoint.y),     Point(FArrowPoint.x, FArrowPoint.y + 6),
                         Point(FArrowPoint.x + 1, FArrowPoint.y + 6), Point(FArrowPoint.x + 1, FArrowPoint.y),
                         Point(FArrowPoint.x + 6, FArrowPoint.y),     Point(FArrowPoint.x + 6, FArrowPoint.y + 1),
                         Point(FArrowPoint.x + 2, FArrowPoint.y + 1), Point(FArrowPoint.x + 2, FArrowPoint.y + 4),
                         Point(FArrowPoint.x + 5, FArrowPoint.y + 7), Point(FArrowPoint.x + 6, FArrowPoint.y + 7),
                         Point(FArrowPoint.x + 3, FArrowPoint.y + 4), Point(FArrowPoint.x + 3, FArrowPoint.y + 3),
                         Point(FArrowPoint.x + 6, FArrowPoint.y + 6), Point(FArrowPoint.x + 7, FArrowPoint.y + 6),
                         Point(FArrowPoint.x + 3, FArrowPoint.y + 2), Point(FArrowPoint.x + 4, FArrowPoint.y + 2),
                         Point(FArrowPoint.x + 7, FArrowPoint.y + 5), Point(FArrowPoint.x + 7, FArrowPoint.y + 6)]);
    NE: Canvas.Polyline([Point(FArrowPoint.x,     FArrowPoint.y),     Point(FArrowPoint.x, FArrowPoint.y + 6),
                         Point(FArrowPoint.x - 1, FArrowPoint.y + 6), Point(FArrowPoint.x - 1, FArrowPoint.y),
                         Point(FArrowPoint.x - 6, FArrowPoint.y),     Point(FArrowPoint.x - 6, FArrowPoint.y + 1),
                         Point(FArrowPoint.x - 2, FArrowPoint.y + 1), Point(FArrowPoint.x - 2, FArrowPoint.y + 4),
                         Point(FArrowPoint.x - 5, FArrowPoint.y + 7), Point(FArrowPoint.x - 6, FArrowPoint.y + 7),
                         Point(FArrowPoint.x - 3, FArrowPoint.y + 4), Point(FArrowPoint.x - 3, FArrowPoint.y + 3),
                         Point(FArrowPoint.x - 6, FArrowPoint.y + 6), Point(FArrowPoint.x - 7, FArrowPoint.y + 6),
                         Point(FArrowPoint.x - 3, FArrowPoint.y + 2), Point(FArrowPoint.x - 4, FArrowPoint.y + 2),
                         Point(FArrowPoint.x - 7, FArrowPoint.y + 5), Point(FArrowPoint.x - 7, FArrowPoint.y + 6)]);
    SW: Canvas.Polyline([Point(FArrowPoint.x,     FArrowPoint.y),     Point(FArrowPoint.x, FArrowPoint.y - 6),
                         Point(FArrowPoint.x + 1, FArrowPoint.y - 6), Point(FArrowPoint.x + 1, FArrowPoint.y),
                         Point(FArrowPoint.x + 6, FArrowPoint.y),     Point(FArrowPoint.x + 6, FArrowPoint.y - 1),
                         Point(FArrowPoint.x + 2, FArrowPoint.y - 1), Point(FArrowPoint.x + 2, FArrowPoint.y - 4),
                         Point(FArrowPoint.x + 5, FArrowPoint.y - 7), Point(FArrowPoint.x + 6, FArrowPoint.y - 7),
                         Point(FArrowPoint.x + 3, FArrowPoint.y - 4), Point(FArrowPoint.x + 3, FArrowPoint.y - 3),
                         Point(FArrowPoint.x + 6, FArrowPoint.y - 6), Point(FArrowPoint.x + 7, FArrowPoint.y - 6),
                         Point(FArrowPoint.x + 3, FArrowPoint.y - 2), Point(FArrowPoint.x + 4, FArrowPoint.y - 2),
                         Point(FArrowPoint.x + 7, FArrowPoint.y - 5), Point(FArrowPoint.x + 7, FArrowPoint.y - 6)]);
    SE: Canvas.Polyline([Point(FArrowPoint.x,     FArrowPoint.y),     Point(FArrowPoint.x, FArrowPoint.y - 6),
                         Point(FArrowPoint.x - 1, FArrowPoint.y - 6), Point(FArrowPoint.x - 1, FArrowPoint.y),
                         Point(FArrowPoint.x - 6, FArrowPoint.y),     Point(FArrowPoint.x - 6, FArrowPoint.y - 1),
                         Point(FArrowPoint.x - 2, FArrowPoint.y - 1), Point(FArrowPoint.x - 2, FArrowPoint.y - 4),
                         Point(FArrowPoint.x - 5, FArrowPoint.y - 7), Point(FArrowPoint.x - 6, FArrowPoint.y - 7),
                         Point(FArrowPoint.x - 3, FArrowPoint.y - 4), Point(FArrowPoint.x - 3, FArrowPoint.y - 3),
                         Point(FArrowPoint.x - 6, FArrowPoint.y - 6), Point(FArrowPoint.x - 7, FArrowPoint.y - 6),
                         Point(FArrowPoint.x - 3, FArrowPoint.y - 2), Point(FArrowPoint.x - 4, FArrowPoint.y - 2),
                         Point(FArrowPoint.x - 7, FArrowPoint.y - 5), Point(FArrowPoint.x - 7, FArrowPoint.y - 6)]);
  end;
  // DrawHintText
  Canvas.brush.Style := bsClear;
  InflateRect(TextRect, -3, -1);
  if BidiMode = bdRightToLeft then
    DrawText(Canvas.handle, PChar(Caption), Length(Caption), TextRect, DT_RIGHT or DT_WORDBREAK or DT_NOPREFIX)
  else
    DrawText(Canvas.handle, PChar(Caption), Length(Caption), TextRect, DT_WORDBREAK or DT_NOPREFIX);
  //shadows - added by Dierk
  if FHint.FShadow then
   begin
    temprect := Rect(0, 0, 0, 0);
    FShadows := TShadows.Create(TempRect, BoundsRect, FHint.FShadowSize, FHint.FShadowOpacity, false);
    FShadows.Show(Handle);
   end;
end;

procedure TmbTBXHintWindow.WMEraseBkgnd(var Msg: TWMEraseBkgnd);
begin
 Msg.Result := 1;
end;

procedure TmbTBXHintWindow.ActivateHint (HintRect: TRect; const AHint: string);
var
  curWidth: Byte;
  Pnt: TPoint;
  HintHeight, HintWidth: Integer;
  NordWest, NordEast, SouthWest, SouthEast: TRect;
begin
  DestroyShadow;
  DoubleBuffered := true;
  Caption := AHint;

  if FHint <> nil then
  begin                 //little correction DS
    Canvas.Font.Assign(FHint.FHintFont);

    // Calculate width and height
    HintRect.Right := HintRect.Left + FHint.FHintWidth - 22;

    if BidiMode = bdRightToLeft then
      DrawText(Canvas.Handle, @AHint[1], Length(AHint), HintRect, DT_RIGHT or DT_CALCRECT or DT_WORDBREAK or DT_NOPREFIX)
    else
      DrawText(Canvas.Handle, @AHint[1], Length(AHint), HintRect, DT_CALCRECT or DT_WORDBREAK or DT_NOPREFIX);

    DrawText(Canvas.Handle, @AHint[1], Length(AHint), HintRect, DT_CALCRECT or DT_WORDBREAK or DT_NOPREFIX);
    Inc(HintRect.Right, 22);
    Inc(HintRect.Bottom, 6);

    // Divide the screen in 4 pices
    NordWest :=  Rect(0, 0, Screen.Width div 2, Screen.Height div 2);
    NordEast :=  Rect(Screen.Width div 2, 0, Screen.Width, Screen.Height div 2);
    SouthWest := Rect(0, Screen.Height div 2, Screen.Width div 2, Screen.Height);
    SouthEast := Rect(Screen.Width div 2, Screen.Height div 2, Screen.Width, Screen.Height);

    GetCursorPos(Pnt);

    if PtInRect(NordWest, Pnt) then
      FArrowPos := NW
    else
      if PtInRect(NordEast, Pnt) then
        FArrowPos := NE
      else
        if PtInRect(SouthWest, Pnt) then
          FArrowPos := SW
        else
          FArrowPos := SE;

    // Calculate the position of the hint
    if FArrowPos = NW then
      curWidth := 12
    else
      curWidth := 5;

    HintHeight := HintRect.Bottom - HintRect.Top;
    HintWidth  := HintRect.Right - HintRect.Left;

    case FArrowPos of
      NW: HintRect := Rect(Pnt.x + curWidth, Pnt.y + curWidth, Pnt.x + HintWidth + curWidth, Pnt.y + HintHeight + curWidth);
      NE: HintRect := Rect(Pnt.x - HintWidth - curWidth, Pnt.y + curWidth, Pnt.x - curWidth, Pnt.y + HintHeight + curWidth);
      SW: HintRect := Rect(Pnt.x + curWidth, Pnt.y - HintHeight - curWidth, Pnt.x + HintWidth + curWidth, Pnt.y - curWidth);
      SE: HintRect := Rect(Pnt.x - HintWidth - curWidth, Pnt.y - HintHeight - curWidth, Pnt.x - curWidth, Pnt.y - curWidth);
    end;

    BoundsRect := HintRect;

    Pnt := ClientToScreen(Point(0, 0));

    SetWindowPos(Handle, HWND_TOPMOST, Pnt.X, Pnt.Y, 0, 0, SWP_SHOWWINDOW or SWP_NOACTIVATE or SWP_NOSIZE);
  end;
end;

procedure TmbTBXHintWindow.DestroyShadow;
begin
  FShadows.Free;
  FShadows := nil;
end;

procedure TmbTBXHintWindow.WMShowWindow(var Message: TWMShowWindow);
begin
  with Message do
   if not Show then (DestroyShadow);
  inherited;
end;

constructor TmbTBXHintWindow.Create(AOwner: TComponent);
begin
 inherited;
 ControlStyle := ControlStyle + [csOpaque];
end;

destructor TmbTBXHintWindow.Destroy;
begin
  DestroyShadow;
  inherited;
end;

end.
