unit mbTBXHotKeyEdit;

{$I mxs.inc}

interface

uses
  SysUtils, Classes, Controls, ComCtrls, Windows, Messages, Graphics, TBX,
  TBXUtils, TBXThemes;

type
  TMouseLoc = (mlNone, mlOver, mlDown);

  TmbTBXHotKeyEdit = class(THotKey)
  private
   FMouseOver: boolean;
   FThemed: boolean;
   FHotTrack: boolean;

   procedure SetHotTrack(Value: boolean);
   procedure SetThemed(Value: boolean);
  protected
   FMouseLoc: TMouseLoc;

   procedure WMNCPaint (var Msg: TWMNCPaint); message WM_NCPAINT;
   procedure TBMThemeChange(var Message: TMessage); message TBM_THEMECHANGE;
   procedure Resize; override;
   procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
   procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
   procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
   procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
   procedure CMGotFocus(var Message: TCMGotFocus); message CM_ENTER;
   procedure CMLostFocus(var Message: TCMLostFocus); message CM_EXIT;
   procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
   procedure RPaint;
  public
   constructor Create(AOwner: TComponent); override;
   destructor Destroy; override;
  published
   property Themed: boolean read FThemed write SetThemed default true;
   property HotTracking: boolean read FHotTrack write SetHotTrack default true;
  end;

implementation

constructor TmbTBXHotKeyEdit.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
 ParentColor := false;
 AddThemeNotification(Self);
 FMouseLoc := mlNone;
 FMouseOver := false;
 FThemed := true;
 FHotTrack := true;
end;

destructor TmbTBXHotKeyEdit.Destroy;
begin
 RemoveThemeNotification(Self);
 inherited Destroy;
end;

procedure TmbTBXHotKeyEdit.TBMThemeChange(var Message: TMessage);
begin
if Message.WParam = TSC_VIEWCHANGE then
 RPaint;
end;

procedure TmbTBXHotKeyEdit.RPaint;
var
 r: TRect;
begin
 r := ClientRect;
 RedrawWindow(Handle, @R, 0, RDW_INVALIDATE or RDW_UPDATENOW or RDW_FRAME{ or RDW_INTERNALPAINT});
end;

procedure SetItemInfo(state: string; var IInfo: TTBXItemInfo; var EInfo: TTBXEditInfo);
begin
   FillChar(IInfo, SizeOf(TTBXItemInfo), 0);
   IInfo.ViewType := TVT_NORMALTOOLBAR;
   IInfo.ItemOptions := IO_APPACTIVE or IO_TOOLBARSTYLE;
   IInfo.Enabled := True;

   FillChar(EInfo, SizeOf(TTBXEditInfo), 0);
   EInfo.RightBtnWidth := CurrentTheme.GetIntegerMetrics(TMI_EDIT_BTNWIDTH);
   EInfo.RightBtnInfo.ButtonType := EBT_DROPDOWN;

   if state = 'normal' then
   begin
     IInfo.Pushed := false;
     IInfo.Selected := false;
     IInfo.HoverKind := hkNone;
     EInfo.RightBtnInfo.ButtonState := 0;
   end;
   if state = 'over' then
   begin
     IInfo.Pushed := false;
     IInfo.Selected := false;
     IInfo.HoverKind := hkMouseHover;
     EInfo.RightBtnInfo.ButtonState := EBDS_HOT;
   end;
   if state = 'down' then
   begin
     IInfo.Pushed := true;
     IInfo.Selected := false;
     IInfo.HoverKind := hkMouseHover;
     EInfo.RightBtnInfo.ButtonState := EBDS_PRESSED;
   end;
end;

procedure TmbTBXHotKeyEdit.WMNCPaint(var Msg: TWMNCPaint);
var
 DC: HDC;
 ii: TTBXItemInfo;
 ed: TTBXEditInfo;
 R: TRect;
 C: TCanvas;
begin
 inherited;
 if not FThemed then Exit;
 C := TCanvas.Create;
 DC := GetWindowDC(Handle);
 GetWindowRect(Handle, R);
 OffsetRect(R, -R.Left, -R.Top);
 C.Handle := DC;
 ExcludeClipRect(DC, 2, 2, R.Right - 2, R.Bottom - 2); // Important
 C.Brush.Color := Color;
 C.FillRect(R);
 // set info
 case FMouseLoc of
  mlNone: SetItemInfo('normal', ii, ed);
  mlOver:
   if FHotTrack then
    SetItemInfo('over', ii, ed)
   else
    SetItemInfo('normal', ii, ed);
  mlDown: SetItemInfo('down', ii, ed);
 end;
 if Focused then SetItemInfo('over', ii, ed);
 if (csDesigning in ComponentState) or (Enabled = false) then SetItemInfo('normal', ii, ed);
 ii.Enabled := Enabled;
 // paint the border
 CurrentTheme.PaintFrame(c, R, ii);
 CurrentTheme.PaintBackgnd(c, R, R, R, CurrentTheme.GetViewColor(TVT_NORMALTOOLBAR), false, TVT_NORMALTOOLBAR);
 CurrentTheme.PaintDock(c, R, R, DP_TOP);
 if CurrentTheme.PaintDockBackground then
  CurrentTheme.PaintBackgnd(c, R, R, R, CurrentTheme.GetViewColor(TVT_NORMALTOOLBAR), false, TVT_NORMALTOOLBAR)
 else
  CurrentTheme.PaintDock(c, R, R, DP_TOP);
 CurrentTheme.PaintButton(c, R, ii);
 //hide double border
 C.Brush.Color := Color;
 InflateRect(R, -1, -1);
 C.FillRect(R);
 // clean up
 C.Handle := 0;
 ReleaseDC(Handle, DC);
 c.free;
end;

procedure TmbTBXHotKeyEdit.Resize;
begin
 RPaint;
 if Assigned(OnResize) then
  OnResize(Self);
end;

procedure TmbTBXHotKeyEdit.CMMouseEnter(var Message: TMessage);
begin
  if not Focused then
   begin
    FMouseOver := true;
    FMouseLoc := mlOver;
    if FHotTrack then
    RPaint;
   end;
end;

procedure TmbTBXHotKeyEdit.CMMouseLeave(var Message: TMessage);
begin
 if not Focused then
  begin
   FMouseOver := false;
   FMouseLoc := mlNone;
   if FHotTrack then
   RPaint;
  end;
end;

procedure TmbTBXHotKeyEdit.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 FMouseLoc := mlDown;
 RPaint;
 inherited;
end;

procedure TmbTBXHotKeyEdit.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 if not Focused then
  begin
   if FMouseOver then
    FMouseLoc := mlOver
   else
    FMouseLoc := mlNone;
   RPaint;
  end;
 inherited;
end;

procedure TmbTBXHotKeyEdit.CMGotFocus(var Message: TCMGotFocus);
begin
 inherited;
 FMouseLoc := mlOver;
 RPaint;
end;

procedure TmbTBXHotKeyEdit.CMLostFocus(var Message: TCMLostFocus);
begin
 inherited;
 FMouseLoc := mlNone;
 RPaint;
end;

procedure TmbTBXHotKeyEdit.CMEnabledChanged(var Message: TMessage);
begin
 inherited;
 RPaint;
end;

procedure TmbTBXHotKeyEdit.SetThemed(Value: boolean);
begin
 FThemed := Value;
 RPaint;
end;

procedure TmbTBXHotKeyEdit.SetHotTrack(Value: boolean);
begin
 if FHotTrack <> Value then
  begin
   FHotTrack := Value;
   RPaint;
  end;
end;

end.
