unit mbTBXImageCheck;

{$I mxs.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Graphics, TBX, TBXThemes, ActnList,
  ImgList, TBXUtils;

type
  TCustomSize = (csBig, csMedium, csSmall);
  TMouseLoc = (mlNone, mlOver, mlDown);
  TmbTBXImageCheck = class;

  TmbTBXImageCheckActionLink = class(TControlActionLink)
  protected
    FClient: TmbTBXImageCheck;
    procedure AssignClient(AClient: TObject); override;
    function IsCheckedLinked: Boolean; override;
    procedure SetChecked(Value: Boolean); override;
  end;

  TmbTBXImageCheck = class(TCustomControl)
  private
   FChecked: boolean;
   FAlt: boolean;
   FCustomSize: TCustomSize;
   FImages: TCustomImageList;
   FImageChangeLink: TChangeLink;
   FImageIndex: TImageIndex;
   FCenter, FAutoSize: boolean;
   FTransparent: boolean;
   FOnChange: TNotifyEvent;
   FHotTrack: boolean;

   procedure SetHotTrack(Value: boolean);
   procedure SetTransparent(t: boolean);
   procedure SetCenter(c: boolean);
   procedure SetIRAutoSize(a: boolean);
   procedure SetImageIndex(i: TImageIndex);
   procedure ImageListChange(Sender: TObject);
   Procedure SetImages(i: TCustomImageList);
   procedure SetCustomSize(s: TCustomSize);
   procedure SetChecked(c: boolean);
  protected
   FMouseLoc: TMouseLoc;
   FMouseOver: boolean;

   procedure Paint; override;
   procedure DrawImage(R: TRect); dynamic;
   function GetActionLinkClass: TControlActionLinkClass; override;
   procedure TBMThemeChange(var Message: TMessage); message TBM_THEMECHANGE;
   procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
   procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
   procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
   procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
   procedure CMGotFocus(var Message: TCMGotFocus); message CM_ENTER;
   procedure CMLostFocus(var Message: TCMLostFocus); message CM_EXIT;
   procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
   procedure Notification(AComponent: TComponent; Operation: TOperation); override;
   procedure KeyDown(var Key: Word; Shift: TShiftState); override;
  public
   constructor Create(AOwner: TComponent); override;
   destructor Destroy; override;
  published
   property Action;
   property AutoSize: boolean read FAutoSize write SetIRAutoSize default false;
   property Center: boolean read FCenter write SetCenter default true;
   property TransparentImage: boolean read FTransparent write SetTransparent default true;
   property CustomSize: TCustomSize read FCustomSize write SetCustomSize default csBig;
   property Checked: boolean read FChecked write SetChecked default false;
   property AlternateCheckMode: boolean read FAlt write FAlt default false;
   property Images: TCustomImageList read FImages write SetImages;
   property ImageIndex: TImageIndex read FImageIndex write SetImageIndex default -1;
   property HotTracking: boolean read FHotTrack write SetHotTrack default true;
   property Color default clWindow;
   property ParentColor default false;
   property Enabled;
   property Anchors;
   property Constraints;
   property DragCursor;
   property DragKind;
   property DragMode;
   property ParentShowHint;
   property ShowHint;
   property TabOrder;
   property TabStop default true;
   property Visible;
   property PopupMenu;

   property OnChange: TNotifyEvent read FOnChange write FOnChange;
   property OnContextPopup;
   property OnClick;
   property OnDblClick;
   property OnDragDrop;
   property OnDragOver;
   property OnEndDock;
   property OnEndDrag;
   property OnEnter;
   property OnExit;
   property OnKeyDown;
   property OnKeyPress;
   property OnKeyUp;
   property OnStartDock;
   property OnStartDrag;
   property OnMouseMove;
   property OnMouseDown;
   property OnMouseUp;
  end;

implementation

{ TmbTBXImageCheckActionLink }

procedure TmbTBXImageCheckActionLink.AssignClient(AClient: TObject);
begin
  inherited AssignClient(AClient);
  FClient := AClient as TmbTBXImageCheck;
end;

function TmbTBXImageCheckActionLink.IsCheckedLinked: Boolean;
begin
  Result := inherited IsCheckedLinked and (FClient.Checked = (Action as TCustomAction).Checked);
end;

procedure TmbTBXImageCheckActionLink.SetChecked(Value: Boolean);
begin
  if IsCheckedLinked then TmbTBXImageCheck(FClient).Checked := Value;
end;

{ TmbTBXImageCheck }

constructor TmbTBXImageCheck.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
 AddThemeNotification(Self);
 DoubleBuffered := true;
 ControlStyle := ControlStyle - [csAcceptsControls] + [csOpaque];
 Height := 104;
 Width := 96;
 ParentColor := false;
 Color := clWindow;
 TabStop := true;
 FChecked := false;
 FAlt := false;
 FCustomSize := csBig;
 FImageIndex := -1;
 FImageChangeLink := TChangeLink.Create;
 FImageChangeLink.OnChange := ImageListChange;
 FAutoSize := false;
 FCenter := true;
 FTransparent := true;
 FMouseLoc := mlNone;
 FMouseOver := false;
 FHotTrack := true;
end;

destructor TmbTBXImageCheck.Destroy;
begin
 RemoveThemeNotification(Self);
 FImageChangeLink.Free;
 inherited;
end;

procedure TmbTBXImageCheck.TBMThemeChange(var Message: TMessage);
begin
if Message.WParam = TSC_VIEWCHANGE then
 Invalidate;
end;

procedure SetItemInfo(state: string; var IInfo: TTBXItemInfo; var EInfo: TTBXEditInfo);
begin
   FillChar(IInfo, SizeOf(TTBXItemInfo), 0);
   IInfo.ViewType := TVT_NORMALTOOLBAR;
   IInfo.ItemOptions := IO_APPACTIVE or IO_TOOLBARSTYLE;
   IInfo.Enabled := True;
   IInfo.IsVertical := false;

   FillChar(EInfo, SizeOf(TTBXEditInfo), 0);
   EInfo.RightBtnWidth := CurrentTheme.GetIntegerMetrics(TMI_EDIT_BTNWIDTH);
   EInfo.RightBtnInfo.ButtonType := EBT_DROPDOWN;

   if state = 'normal' then
   begin
     IInfo.Pushed := false;
     IInfo.Selected := false;
     IInfo.HoverKind := hkNone;
     EInfo.RightBtnInfo.ButtonState := 0;
   end;
   if state = 'over' then
   begin
     IInfo.Pushed := false;
     IInfo.Selected := false;
     IInfo.HoverKind := hkMouseHover;
     EInfo.RightBtnInfo.ButtonState := EBDS_HOT;
   end;
   if state = 'down' then
   begin
     IInfo.Pushed := true;
     IInfo.Selected := false;
     IInfo.HoverKind := hkMouseHover;
     EInfo.RightBtnInfo.ButtonState := EBDS_PRESSED;
   end;
end;

procedure TmbTBXImageCheck.Paint;
var
 R: TRect;
 ii: TTBXItemInfo;
 ed: TTBXEditInfo;
begin
 R := ClientRect;
 case FMouseLoc of
  mlNone: SetItemInfo('normal', ii, ed);
  mlOver:
   if FHotTrack then
    SetItemInfo('over', ii, ed)
   else
    SetItemInfo('normal', ii, ed);
  mlDown: SetItemInfo('down', ii, ed);
 end;
 if Focused and (FMouseLoc <> mlDown) then SetItemInfo('over', ii, ed);
 if (csDesigning in ComponentState) or (Enabled = false) then SetItemInfo('normal', ii, ed);
 ii.Enabled := Enabled;
 //paint frame
 CurrentTheme.PaintFrame(canvas, R, ii);
 CurrentTheme.PaintBackgnd(canvas, R, R, R, CurrentTheme.GetViewColor(TVT_NORMALTOOLBAR), false, TVT_NORMALTOOLBAR);
 CurrentTheme.PaintDock(canvas, R, R, DP_TOP);
 if CurrentTheme.PaintDockBackground then
  CurrentTheme.PaintBackgnd(canvas, R, R, R, CurrentTheme.GetViewColor(TVT_NORMALTOOLBAR), false, TVT_NORMALTOOLBAR)
 else
  CurrentTheme.PaintDock(canvas, R, R, DP_TOP);
 CurrentTheme.PaintButton(canvas, R, ii);
 CurrentTheme.PaintEditFrame(canvas, R, ii, ed);
 //paint focus & select rect
 Canvas.Brush.Style := bsSolid;
 Canvas.Brush.Color := Color;
 InflateRect(R, -1, -1);
 Canvas.FillRect(R);
 InflateRect(R, -2, -2);
 Inc(R.Left, 1);
 Inc(R.Top, 1);
 if FChecked then
  begin
   Canvas.Pen.Color := Blend(CurrentTheme.GetItemColor(ii), clHighlight, 80);
   Canvas.Pen.Width := 2;
   Canvas.Brush.Style := bsClear;
   Canvas.Rectangle(R);
  end;
 InflateRect(R, -1, -1);
 DrawImage(R);
end;

procedure TmbTBXImageCheck.DrawImage(R: TRect);
begin
 if FImages <> nil then
  begin
   if FTransparent then
    FImages.DrawingStyle := dsTransparent
   else
    FImages.DrawingStyle := dsNormal;
   if FCenter then
    begin
     R.Left := R.Left + ((R.Right - R.Left) - FImages.Width) div 2;
     R.Top := R.Top + ((R.Bottom - R.Top) - FImages.Height) div 2;
    end;
   // nadji bolje za disabled
   FImages.Draw(Canvas, R.Left, R.Top, FImageIndex, Enabled);
  end;
end;

function TmbTBXImageCheck.GetActionLinkClass: TControlActionLinkClass;
begin
 Result := TmbTBXImageCheckActionLink;
end;

procedure TmbTBXImageCheck.CMMouseEnter(var Message: TMessage);
begin
  if not Focused then
   begin
    FMouseOver := true;
    FMouseLoc := mlOver;
    if FHotTrack then
    Invalidate;
   end;
end;

procedure TmbTBXImageCheck.CMMouseLeave(var Message: TMessage);
begin
 if not Focused then
  begin
   FMouseOver := false;
   FMouseLoc := mlNone;
   if FHotTrack then
   Invalidate;
  end;
end;

procedure TmbTBXImageCheck.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 if Button = mbLeft then
  begin
   FMouseLoc := mlDown;
   SetFocus;
   if not FAlt then
    SetChecked(not FChecked);
   Invalidate;
  end;
 inherited;
end;

procedure TmbTBXImageCheck.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 if Button = mbLeft then
  begin
   if not Focused then
    begin
     if FMouseOver then
      FMouseLoc := mlOver
     else
      FMouseLoc := mlNone;
    end;
   if FAlt and PtInRect(ClientRect, Point(x, y)) then
    SetChecked(not FChecked);
   Invalidate;
  end;
 inherited;
end;

procedure TmbTBXImageCheck.CMGotFocus(var Message: TCMGotFocus);
begin
 inherited;
 FMouseLoc := mlOver;
 Invalidate;
end;

procedure TmbTBXImageCheck.CMLostFocus(var Message: TCMLostFocus);
begin
 inherited;
 FMouseLoc := mlNone;
 Invalidate;
end;

procedure TmbTBXImageCheck.CMEnabledChanged(var Message: TMessage);
begin
 inherited;
 Invalidate;
end;

procedure TmbTBXImageCheck.SetChecked(c: boolean);
begin
 if FChecked <> c then
  begin
   FChecked := c;
   if (Action is TCustomAction) then TCustomAction(Action).Checked := c;
   invalidate;
   if Assigned(FOnChange) then FOnChange(Self);
  end;
end;

procedure TmbTBXImageCheck.SetCustomSize(s: TCustomSize);
begin
 FCustomSize := s;
 case FCustomSize of
  csBig:
   begin
    Width := 96;
    Height := 104;
   end;
  csMedium:
   begin
    Width := 54;
    Height := 55;
   end;
  csSmall:
   begin
    Width := 36;
    Height := 40;
   end;
 end;
end;

procedure TmbTBXImageCheck.Notification(AComponent: TComponent; Operation: TOperation);
begin
 inherited Notification(AComponent, Operation);
 if Operation = opRemove then
  if AComponent = Images then
   SetImages(nil);
end;

procedure TmbTBXImageCheck.SetImages(i: TCustomImageList);
begin
 if FImages <> nil then
  FImages.UnRegisterChanges(FImageChangeLink);
 FImages := i;
 if FImages <> nil then
  begin
   FImages.RegisterChanges(FImageChangeLink);
   FImages.FreeNotification(Self);
  end;
 if FAutoSize and (FImages <> nil) then
  begin
   Width := FImages.Width + 2*6;
   Height := FImages.Height + 2*6;
  end;
 Invalidate;
end;

procedure TmbTBXImageCheck.ImageListChange(Sender: TObject);
begin
 if Sender = Images then
  begin
   if FAutoSize and (FImages <> nil) then
    begin
     Width := FImages.Width + 2*6;
     Height := FImages.Height + 2*6;
    end;
   Invalidate;
  end;
end;

procedure TmbTBXImageCheck.SetImageIndex(i: TImageIndex);
begin
 if FImageIndex <> i then
  begin
   FImageIndex := i;
   Invalidate;
 end;
end;

procedure TmbTBXImageCheck.SetCenter(c: boolean);
begin
 FCenter := c;
 invalidate;
end;

procedure TmbTBXImageCheck.SetIRAutoSize(a: boolean);
begin
 FAutoSize := a;
 if FImages <> nil then
  begin
   Width := FImages.Width + 2*6;
   Height := FImages.Height + 2*6;
  end;
 invalidate;
end;

procedure TmbTBXImageCheck.SetTransparent(t: boolean);
begin
 FTransparent := t;
 invalidate;
end;

procedure TmbTBXImageCheck.KeyDown(var Key: Word; Shift: TShiftState);
begin
 case Key of
  VK_SPACE:
   begin
    SetChecked(not FChecked);
    Invalidate;
   end;
  VK_ESCAPE:
   begin
    SetChecked(false);
    Invalidate;
   end;
 end;
 inherited;
end;

procedure TmbTBXImageCheck.SetHotTrack(Value: boolean);
begin
 if FHotTrack <> Value then
  begin
   FHotTrack := Value;
   Invalidate;
  end;
end;

end.
