unit mbTBXImageComboBox;

{$I mxs.inc}

interface

uses
  Messages, SysUtils, Classes, Controls, mbTBXComboBox, Windows, TBX, TBXUtils, TBXThemes,
  Graphics, ImgList, StdCtrls;

type
  TmbTBXImageComboBox = class(TmbTBXComboBox)
  private
   FImages: TCustomImageList;
   FHotImages: TCustomImageList;
   FDrawLabel: TDrawItemLabelEvent;

   procedure SetImages(i: TCustomImageList);
   procedure SetHotImages(i: TCustomImageList);
  protected
   procedure DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
   procedure EnabledChanged; 
  public
   constructor Create(AOwner: TComponent); override;
   destructor Destroy; override;
  published
   property Images: TCustomImageList read FImages write SetImages;
   property HotImages: TCustomImageList read FHotImages write SetHotImages;

   property OnDrawItemLabel: TDrawItemLabelEvent read FDrawLabel write FDrawLabel;
  end;

implementation

constructor TmbTBXImageComboBox.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
ParentColor := false;
 Style := csOwnerDrawFixed;
 ItemHeight := 17;
end;

destructor TmbTBXImageComboBox.Destroy;
begin
 inherited Destroy;
end;

procedure TmbTBXImageComboBox.DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
 ii: TTBXItemInfo;
 ed: TTBXEditInfo;
begin
 Canvas.Font := Font;
 Canvas.Brush.Style := bsSolid;
 if odSelected in State then
  begin
   SetItemInfo('over', ii, ed);
   if FThemedSelect then
    begin
     Canvas.Pen.color := CurrentTheme.GetItemTextColor(ii);
     Canvas.Brush.Color := CurrentTheme.GetItemColor(ii);
     Canvas.Font.Color := CurrentTheme.GetItemTextColor(ii);
    end
   else
    begin
     Canvas.Brush.Color := clHighlight;
     Canvas.Pen.Color := clHighlightText;
     Canvas.Font.Color := clHighlightText;
    end;
  end
 else
  begin
   SetItemInfo('normal', ii, ed);
   Canvas.Brush.Color := Color;
   if FThemedSelect then
    begin
     Canvas.Pen.color := CurrentTheme.GetItemTextColor(ii);
     Canvas.Font.Color := CurrentTheme.GetItemTextColor(ii);
    end
   else
    begin
     Canvas.Pen.Color := clWindowText;
     Canvas.Font.Color := clWindowText;
    end;
  end;
 Canvas.FillRect(Rect);
 if FImages <> nil then
  begin
   if odSelected in State then
    begin
     if FHotImages <> nil then
      FHotImages.Draw(Canvas, Rect.Left + 4, Rect.Top + ((Rect.Bottom - Rect.Top) - FHotImages.Height) div 2, Index, true)
     else
      FImages.Draw(Canvas, Rect.Left + 4, Rect.Top + ((Rect.Bottom - Rect.Top) - FImages.Height) div 2, Index, true);
    end
   else
    FImages.Draw(Canvas, Rect.Left + 4, Rect.Top + ((Rect.Bottom - Rect.Top) - FImages.Height) div 2, Index, true);
   Inc(Rect.Left, FImages.Width + 8);
  end
 else
  Inc(Rect.Left, 4);
 if Assigned(FDrawLabel) then FDrawLabel(Self, Index, Canvas.Font, State);
 DrawText(Canvas.Handle, PChar(Items[Index]), -1, Rect, DT_SINGLELINE or DT_VCENTER or DT_LEFT);
 if Assigned(OnDrawItem) then
  OnDrawItem(Self, Index, Rect, State);
end;

procedure TmbTBXImageComboBox.SetImages(i: TCustomImageList);
begin
 FImages := i;
 Invalidate;
end;

procedure TmbTBXImageComboBox.SetHotImages(i: TCustomImageList);
begin
 FHotImages := i;
 Invalidate;
end;

procedure TmbTBXImageComboBox.EnabledChanged;
begin
 inherited;
 Invalidate;
end;

end.
