unit mbTBXLineGroup;

{$I mxs.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Graphics, TBX, TBXThemes, TBXUtils;

type
  TmbTBXLineGroup = class(TCustomControl)
  private
   FTopMargin, FLeftMargin, FSpacing: integer;
   FCaption: TCaption;

   procedure SetTopMargin(m: integer);
   procedure SetLeftMargin(m: integer);
   procedure SetSpacing(s: integer);
   procedure SetCaption(c: TCaption);
  protected
   procedure Paint; override;
   procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
   procedure AdjustClientRect(var ARect: TRect); override;
   procedure TBMThemeChange(var Message: TMessage); message TBM_THEMECHANGE;
  public
   constructor Create(AOwner: TComponent); override;
   destructor Destroy; override;
  published
   property Caption: TCaption read FCaption write SetCaption;
   property TopMargin: integer read FTopMargin write SetTopMargin default 0;
   property LeftMargin: integer read FLeftMargin write SetLeftMargin default 0;
   property Spacing: integer read FSpacing write SetSpacing default 1;
   property Align;
   property Anchors;
   property Enabled;
   property Constraints;
   property Color;
   property Font;
   property ParentColor;
   {$IFDEF DELPHI_7_UP}
   property ParentBackground;
   {$ENDIF}
   property Hint;
   property ShowHint;
   property ParentShowHint;
   property TabStop;
   property TabOrder;
   property PopupMenu;
   property Visible;

   property OnClick;
   property OnDblClick;
   property OnEnter;
   property OnExit;
   property OnContextPopup;
   property OnMouseMove;
   property OnMouseDown;
   property OnMouseUp;
  end;

implementation

constructor TmbTBXLineGroup.Create(AOwner: TComponent);
begin
 inherited;
 AddThemeNotification(Self);
 {$IFDEF DELPHI_7_UP}
 ControlStyle := ControlStyle + [csAcceptsControls, csParentBackground, csOpaque];
 {$ELSE}
 ControlStyle := ControlStyle + [csAcceptsControls, csOpaque];
 {$ENDIF}
 DoubleBuffered := true;
 FTopmargin := 0;
 FLeftMargin := 0;
 FSpacing := 1;
 FCaption := 'mbTBXLineGroup';
 Height := 105;
 Width := 185;
end;

destructor TmbTBXLineGroup.Destroy;
begin
 RemoveThemeNotification(Self);
 inherited Destroy;
end;

procedure TmbTBXLineGroup.TBMThemeChange(var Message: TMessage);
begin
if Message.WParam = TSC_VIEWCHANGE then
 Invalidate;
end;

procedure TmbTBXLineGroup.Paint;
var
 r: TRect;
 IInfo: TTBXItemInfo;
begin
 FillChar(IInfo, SizeOf(TTBXItemInfo), 0);
 IInfo.ViewType := TVT_NORMALTOOLBAR;
 IInfo.ItemOptions := IO_APPACTIVE or IO_TOOLBARSTYLE;
 IInfo.Enabled := Enabled;
 IInfo.Pushed := false;
 IInfo.Selected := false;
 IInfo.HoverKind := hkNone;
 Font.Color := CurrentTheme.GetItemTextColor(IInfo);
 Canvas.Font := Font;
 Canvas.Pen.Color := Blend(CurrentTheme.GetItemColor(IInfo), clBtnShadow, 50);
 Canvas.Brush.Style := bsClear;
 if csDesigning in ComponentState then
  begin
   Canvas.Pen.Style := psDot;
   Canvas.Rectangle(ClientRect);
   Canvas.Pen.Style := psSolid;
  end;
 r := Rect(FLeftMargin, FTopMargin, Canvas.TextWidth(FCaption) + FLeftMargin, Canvas.TextHeight(FCaption) + FTopMargin);
 DrawText(Canvas.Handle, PChar(FCaption), Length(FCaption), r, DT_VCENTER or DT_SINGLELINE or DT_LEFT);
 r := Rect(r.Right + FSpacing, FTopMargin, Width, r.Bottom);
 Canvas.MoveTo(r.Left + FSpacing, (r.Bottom - r.Top) div 2 + 1 + FTopMargin);
 Canvas.LineTo(Width, (r.Bottom - r.Top) div 2 + 1 + FTopMargin);
end;

procedure TmbTBXLineGroup.SetTopMargin(m: integer);
begin
 FTopMargin := m;
 Perform(WM_SIZE, Width, Height);
 invalidate;
end;

procedure TmbTBXLineGroup.SetLeftMargin(m: integer);
begin
 FLeftMargin := m;
 Perform(WM_SIZE, Width, Height);
 invalidate;
end;

procedure TmbTBXLineGroup.SetSpacing(s: integer);
begin
 FSpacing := s;
 invalidate;
end;

procedure TmbTBXLineGroup.SetCaption(c: TCaption);
begin
 FCaption := c;
 invalidate;
end;

procedure TmbTBXLineGroup.CMEnabledChanged(var Message: TMessage);
begin
 inherited;
 invalidate;
end;

procedure TmbTBXLineGroup.AdjustClientRect(var ARect: TRect);
begin
 Inc(ARect.Top, Canvas.TextHeight(FCaption) + FTopMargin);
 Inc(ARect.Left, FLeftMargin);
end;

end.
