unit mbTBXNonDockablePanel;

{$I mxs.inc}

interface

uses
  SysUtils, Messages, Classes, Controls, Windows, Graphics, TBX, TBXUtils,
  TBXThemes, TBXDkPanels, Forms;

type
  TMouseLoc = (mlNone, mlOver, mlDown);

  TmbTBXNonDockablePanel = class(TTBXPanelObject)
  private
   FMouseLoc: TMouseLoc;
   mx, my: integer;
   FColor: TColor;
   FCaption: TCaption;
   FVertical: boolean;
   FBorderStyle: TBorderStyle;
   FShowCaption: boolean;
   FCloseButton: boolean;
   FOnCloseBtnClick: TNotifyEvent;

   function OverClose: boolean;
   procedure SetColor(c: TColor);
   procedure SetCaption(c: TCaption);
   procedure SetVertical(v: boolean);
   procedure SetBorderStyle(b: TBorderStyle);
   procedure SetShowCaption(s: boolean);
   procedure SetCloseButton(c: boolean);
  protected
   procedure Resize; override;
   procedure WMEraseBkgnd(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
   procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
   procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
   procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
   procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
   procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
   procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
   procedure Paint; override;
   function GetCloseBtnRect: TRect;
   procedure AdjustClientRect(var Rect: TRect); override;
  public
   constructor Create(AOwner: TComponent); override;
   destructor Destroy; override;
  published
   property Color: TColor read FColor write SetColor default clNone;
   property Caption: TCaption read FCaption write SetCaption;
   property Vertical: boolean read FVertical write SetVertical default true;
   property BorderStyle: TBorderStyle read FBorderStyle write SetBorderStyle default bsSingle;
   property ShowCaption: boolean read FShowCaption write SetShowCaption default true;
   property CloseButton: boolean read FCloseButton write SetCloseButton default true;
   property Align;
   property Anchors;
   property AutoSize;
   property Constraints;
   property DragCursor;
   property DragKind;
   property DragMode;
   property Enabled;
   property ShowHint;
   property TabOrder;
   property TabStop;
   property Visible;
   property Hint;
   property PopupMenu;
   property ParentShowHint;

   property OnCloseBtnClick: TNotifyEvent read FOnCloseBtnClick write FOnCloseBtnClick;
   property OnClick;
   property OnContextPopup;
   property OnDblClick;
   property OnDragDrop;
   property OnDragOver;
   property OnEndDock;
   property OnEndDrag;
   property OnMouseDown;
   property OnMouseEnter;
   property OnMouseLeave;
   property OnMouseMove;
   property OnMouseUp;
   property OnStartDock;
   property OnStartDrag;
  end;

implementation

constructor TmbTBXNonDockablePanel.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
 ControlStyle := ControlStyle + [csOpaque];
 FMouseLoc := mlNone;
 FColor := clNone;
 FVertical := true;
 FBorderStyle := bsSingle;
 FShowCaption := true;
 FCloseButton := true;
 DoubleBuffered := true;
 Width := 50;
 Height := 50;
end;

destructor TmbTBXNonDockablePanel.Destroy;
begin
 inherited Destroy;
end;

procedure TmbTBXNonDockablePanel.Paint;

procedure SetInfo(state: string; var dpi: TTBXDockPanelInfo; ACaption: string; vertical: boolean; BorderStyle: TBorderStyle; ShowCaption: Boolean; CloseBtn: boolean; AColor: TColor);
begin
 if AColor = clNone then
  dpi.EffectiveColor := CurrentTheme.GetViewColor(DPVT_NORMAL)
 else
  dpi.EffectiveColor := AColor;
 dpi.IsVertical := vertical;
 dpi.AllowDrag := true;
 dpi.BorderStyle := BorderStyle;
 dpi.BorderSize := Point(1, 2);
 dpi.ShowCaption := ShowCaption;
 dpi.Caption := PChar(ACaption);
 dpi.ViewType := DPVT_NORMAL;
 if CloseBtn then
  begin
   dpi.CloseButtonState := CDBS_VISIBLE;
   if state = 'normal' then
    dpi.CloseButtonState := CDBS_VISIBLE;
   if state = 'over' then
    dpi.CloseButtonState := dpi.CloseButtonState or CDBS_HOT;
   if state = 'down' then
    dpi.CloseButtonState := dpi.CloseButtonState or CDBS_PRESSED;
  end
 else
  dpi.CloseButtonState := 0;
end;

var
 dpi: TTBXDockPanelInfo;
 R: TRect;
begin
 R := ClientRect;
 if FColor = clNone then
  Canvas.Brush.Color := CurrentTheme.GetViewColor(DPVT_NORMAL)
 else
  Canvas.Brush.Color := FColor;
 Canvas.FillRect(R);
 if (csDesigning in ComponentState) or (not Enabled) then FMouseLoc := mlNone;
 case FMouseLoc of
  mlNone: SetInfo('normal', dpi, FCaption, FVertical, FBorderStyle, FShowCaption, FCloseButton, FColor);
  mlOver: SetInfo('over', dpi, FCaption, FVertical, FBorderStyle, FShowCaption, FCloseButton, FColor);
  mlDown: SetInfo('down', dpi, FCaption, FVertical, FBorderStyle, FShowCaption, FCloseButton, FColor);
 end;
 CurrentTheme.PaintDockPanelNCArea(Canvas, R, dpi);
end;

function TmbTBXNonDockablePanel.OverClose: boolean;
var
 R: TRect;
begin
 R := GetCloseBtnRect;
 Result := PtInRect(R, Point(mx, my));
end;

function TmbTBXNonDockablePanel.GetCloseBtnRect: TRect;
var
  X, Y, Z: Integer;
begin
  Z := GetSystemMetrics(SM_CYSMCAPTION) - 1;
  if FVertical then
   begin
    X := (ClientWidth + 1) - Z;
    Y := 1;
   end
  else
   begin
    X := 1;
    Y := ClientHeight + 1 - Z;
   end;
  Result := Bounds(X, Y, Z, Z);
end;

procedure TmbTBXNonDockablePanel.Resize;
begin
 Invalidate;
 if Assigned(OnResize) then
  OnResize(Self);
end;

procedure TmbTBXNonDockablePanel.CMMouseEnter(var Message: TMessage);
begin
 if OverClose then
  FMouseLoc := mlOver
 else
  FMouseLoc := mlNone;
 Invalidate;
end;

procedure TmbTBXNonDockablePanel.CMMouseLeave(var Message: TMessage);
begin
 FMouseLoc := mlNone;
 Invalidate;
end;

procedure TmbTBXNonDockablePanel.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
 mx := x;
 my := y;
 if OverClose then
  FMouseLoc := mlOver
 else
  FMouseLoc := mlNone;
 Invalidate;
 if Assigned(OnMouseMove) then
  OnMouseMove(Self, Shift, x, y);
end;

procedure TmbTBXNonDockablePanel.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 mx := x;
 my := y;
 if OverClose then
  FMouseLoc := mlDown
 else
  FMouseLoc := mlNone;
 Invalidate;
 if Assigned(OnMouseDown) then
  OnMouseDown(Self, Button, Shift, x, y);
end;

procedure TmbTBXNonDockablePanel.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 mx := x;
 my := y;
 if OverClose then
  FMouseLoc := mlOver
 else
  FMouseLoc := mlNone;
 Invalidate;
 if Assigned(OnMouseUp) then
  OnMouseUp(Self, Button, Shift, x, y);
 if OverClose then
  begin
   Visible := false;
   if Assigned(FOnCloseBtnClick) then
    FOnCloseBtnClick(Self);
  end;
end;

procedure TmbTBXNonDockablePanel.CMEnabledChanged(var Message: TMessage);
begin
 inherited;
 Invalidate;
end;

procedure TmbTBXNonDockablePanel.SetColor(c: TColor);
begin
 FColor := c;
 Invalidate;
end;

procedure TmbTBXNonDockablePanel.SetCaption(c: TCaption);
begin
 FCaption := c;
 Invalidate;
end;

procedure TmbTBXNonDockablePanel.SetVertical(v: boolean);
begin
 FVertical := v;
 Invalidate;
end;

procedure TmbTBXNonDockablePanel.SetBorderStyle(b: TBorderStyle);
begin
 FBorderStyle := b;
 Invalidate;
end;

procedure TmbTBXNonDockablePanel.SetShowCaption(s: boolean);
begin
 FShowCaption := s;
 Invalidate;
end;

procedure TmbTBXNonDockablePanel.SetCloseButton(c: boolean);
begin
 FCloseButton := c;
 Invalidate;
end;

procedure TmbTBXNonDockablePanel.WMEraseBkgnd(var Message: TWMEraseBkgnd);
var
 ACanvas: TCanvas;
begin
 Message.Result := 1;
 ACanvas := TCanvas.Create;
  try
   ACanvas.Handle := Message.DC;
   if Color = clNone then
    ACanvas.Brush.Color := CurrentTheme.GetViewColor(DPVT_NORMAL)
   else
    ACanvas.Brush.Color := Color;
   ACanvas.FillRect(ClientRect);
  finally
   ACanvas.Free;
  end;
end;

procedure TmbTBXNonDockablePanel.AdjustClientRect(var Rect: TRect);
begin
  inherited AdjustClientRect(Rect);
  InflateRect(Rect, -2, -2);
  Rect.Top := Rect.Top + GetSystemMetrics(SM_CYSMCAPTION);
end;

end.
