unit mbTBXPageScroller;

{$I mxs.inc}

interface

uses
  SysUtils, Classes, Controls, Messages, Windows, Graphics, TBXDkPanels,
 {$IFDEF DELPHI_7_UP}
  Themes,
 {$ENDIF}
  TBX, TBXThemes;

type
  TmbTBXPageScroller = class(TTBXPageScroller)
  protected
   procedure TBMThemeChange(var Message: TMessage); message TBM_THEMECHANGE;
   procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
   procedure Resize; override;
   procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
  public
   constructor Create(AOwner: TComponent); override;
   destructor Destroy; override;
  published
  {$IFDEF DELPHI_7_UP}
   property ParentBackground default true;
  {$ENDIF}
   property DoubleBuffered default true;
  end;

implementation

constructor TmbTBXPageScroller.Create(AOwner: TComponent);
begin
 inherited;
 AddThemeNotification(Self);
 DoubleBuffered := true;
 {$IFDEF DELPHI_7_UP}
 ParentBackground := true;
 {$ENDIF}
end;

destructor TmbTBXPageScroller.Destroy;
begin
 inherited;
 RemoveThemeNotification(Self);
end;

procedure TmbTBXPageScroller.TBMThemeChange(var Message: TMessage);
begin
if Message.WParam = TSC_VIEWCHANGE then
 Invalidate;
end;

procedure TmbTBXPageScroller.WMPaint(var Message: TWMPaint);
var
 c: TControlCanvas;
 MemDC: HDC;
 OldBMP: HBITMAP;
 PBack: tbitmap;
begin
 inherited;
 C := TControlCanvas.Create;
 C.Control := Self;
 try
  if csDesigning in ComponentState then
   begin
    C.Pen.Style := psDot;
    C.Brush.Style := bsClear;
    C.Pen.Color := clBtnShadow;
    C.Rectangle(ClientRect);
   end
  else
  {$IFDEF DELPHI_7_UP}
   if ParentBackground then
    begin
     PBack := TBitmap.Create;
     try
      PBack.PixelFormat := pf24bit;
      PBack.Width := width;
      PBack.Height := height;
      PBack.Canvas.Brush.Color := Color;
      PBack.Canvas.FillRect(PBack.Canvas.ClipRect);
      with ThemeServices do
       if ThemesEnabled then
        begin
         MemDC := CreateCompatibleDC(0);
         OldBMP := SelectObject(MemDC, PBack.Handle);
         DrawParentBackground(Handle, MemDC, nil, False);
         if OldBMP <> 0 then SelectObject(MemDC, OldBMP);
         if MemDC <> 0 then DeleteDC(MemDC);
        end;
      C.Draw(0, 0, PBack);
     finally
      PBack.Free;
     end;
    end;
    {$ENDIF}
 finally
  C.Free;
 end;
end;

procedure TmbTBXPageScroller.Resize;
begin
 inherited;
 Invalidate;
end;

procedure TmbTBXPageScroller.CMEnabledChanged(var Message: TMessage);
begin
 inherited;
 Invalidate;
end;

end.
