unit mbTBXProgressBar;

{$I mxs.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, TBX, TBXUtils, TBXThemes;

type
  TmbTBXProgressBar = class(TCustomControl)
  private
   TempBmp, Chunk: TBitmap;
   FShowPosText: boolean;
   FPercentage: boolean;
   FPosition: integer;
   FMax: integer;
   FMin: integer;
   FPosTextPrefix: string;
   FPosTextSuffix: string;
   FChunked: boolean;

   procedure SetChunked(Value: boolean);
   procedure SetMax(const Value: integer);
   procedure SetMin(const Value: integer);
   procedure SetPercentage(const Value: boolean);
   procedure SetPosition(const Value: integer);
   procedure SetPosTextPrefix(const Value: string);
   procedure SetPosTextSuffix(const Value: string);
   procedure SetShowPosText(const Value: boolean);
  protected
   procedure WMEraseBkgnd(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
   procedure CMFontChanged(var Message: TMessage); message CM_FontChanged;
   procedure TBMThemeChange(var Message: TMessage); message TBM_THEMECHANGE;
   procedure Paint; override;
   procedure Resize; override;
   procedure SetItemInfo(state: string; var IInfo: TTBXItemInfo);
   procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
   procedure DrawChunk; dynamic;
   procedure CreateWnd; override;
  public
   constructor Create(AOwner: TComponent); override;
   destructor Destroy; override;
   procedure AddPosition(APos: integer);
  published
   property Max: integer read FMax write SetMax default 100;
   property Min: integer read FMin write SetMin default 0;
   property Percentage: boolean read FPercentage write SetPercentage default true;
   property Position: integer read FPosition write SetPosition default 0;
   property PosTextPrefix: string read FPosTextPrefix write SetPosTextPrefix;
   property PosTextSuffix: string read FPosTextSuffix write SetPosTextSuffix;
   property ShowPosText: boolean read FShowPosText write SetShowPosText default true;
   property Chunked: boolean read FChunked write SetChunked default false;
   property PopupMenu;
   property Align;
   property Font;
   property ParentShowHint;
   property ShowHint;
   property Visible;
   property Hint;
   property Anchors;
   property Enabled;
   property Cursor;
   property DragCursor;
   property DragKind;
   property DragMode;

   property OnContextPopup;
   property OnDragDrop;
   property OnDragOver;
   property OnEndDock;
   property OnEndDrag;
   property OnStartDock;
   property OnStartDrag;
   property OnClick;
   property OnDblClick;
   property OnMouseDown;
   property OnMouseMove;
   property OnMouseUp;
  end;

implementation

constructor TmbTBXProgressBar.Create(AOwner: TComponent);
var
 ii: TTBXItemInfo;
begin
  inherited;
  ControlStyle := ControlStyle + [csOpaque];
  DoubleBuffered := true;
  Width := 169;
  Height := 25;
  AddThemeNotification(Self);
  fPosition := 0;
  fMin := 0;
  fMax := 100;
  fPosTextSuffix := '';
  fPosTextPrefix := '';
  fPercentage := True;
  FShowPosText := true;
  TempBmp := TBitmap.Create;
  TempBmp.PixelFormat:= pf32bit;
  Chunk := TBitmap.Create;
  Chunk.PixelFormat:= pf32bit;
  TabStop := false;
  SetItemInfo('normal', ii);
  Font.Color := CurrentTheme.GetItemTextColor(ii);
  FChunked := false;
end;

procedure TmbTBXProgressBar.CreateWnd;
begin
 inherited;
 DrawChunk;
end;

destructor TmbTBXProgressBar.Destroy;
begin
  TempBmp.Free;
  Chunk.Free;
  RemoveThemeNotification(Self);
  inherited;
end;

procedure TmbTBXProgressBar.Paint;
var
  ii: TTBXItemInfo;
  R: TRect;
  Len: integer;
  Percent: Extended;
  ACap: string;
  i, x: integer;
  rgn: HRGN;
begin
  TempBmp.Width := Width;
  TempBmp.Height := Height;
  with TempBmp do
   begin
    Canvas.Brush.Style := bsSolid;
    //background
    SetItemInfo('over', ii);
    CurrentTheme.PaintButton(Canvas, Canvas.ClipRect, ii);
    R := Canvas.ClipRect;
    InflateRect(R, -1, -1);
    SetItemInfo('normal', ii);
    CurrentTheme.PaintBackgnd(Canvas, R, R, R, CurrentTheme.GetViewColor(TVT_NORMALTOOLBAR), false, TVT_NORMALTOOLBAR);
    CurrentTheme.PaintDock(Canvas, R, R, DP_TOP);
    if CurrentTheme.PaintDockBackground then
     CurrentTheme.PaintBackgnd(Canvas, R, R, R, CurrentTheme.GetViewColor(TVT_NORMALTOOLBAR), false, TVT_NORMALTOOLBAR)
    else
     CurrentTheme.PaintDock(Canvas, R, R, DP_TOP);
    //set position
    Len := FMax - FMin;
    if FMax = FMin then
     Percent := 0
    else
     Percent := (fPosition - FMin) / Len;
    R := Canvas.ClipRect;
    r.Right := Round(Width * Percent);
    //position
    if FPosition > 0 then
     begin
      if not FChunked then
       begin
        SetItemInfo('over', ii);
        CurrentTheme.PaintButton(Canvas, R, ii);
       end
      else
       begin
        rgn := CreateRectRgn(2, 0, Width - 3, Height);
        SelectClipRgn(TempBmp.Canvas.Handle, rgn);
        x := 2;
        for i := 0 to Round(FPosition*Width/100) div 11 - 1 do
         begin
          Canvas.Draw(x, 2, Chunk);
          x := x + 11;
         end;
        DeleteObject(rgn);
       end;
     end;
    //text
    if FPercentage then
     ACap := IntToStr(Round(Percent * 100)) + '%'
    else
     aCap := IntToStr(FPosition);
    if FShowPosText then
     ACap := fPosTextPrefix + ACap + fPosTextSuffix;
    Canvas.Font := Font;
    Canvas.Brush.Style := bsClear;
    R := Canvas.ClipRect;
    SetItemInfo('normal', ii);
    Canvas.Font.Color := CurrentTheme.GetItemTextColor(ii);
    if FShowPosText then
     DrawText(Canvas.Handle, PChar(ACap), Length(ACap), R, DT_VCENTER or DT_CENTER or DT_SINGLELINE);
    Canvas.Brush.Style := bsSolid;
   end;
  Canvas.Draw(0, 0, TempBmp);
end;

procedure TmbTBXProgressBar.SetItemInfo(state: string; var IInfo: TTBXItemInfo);
begin
   FillChar(IInfo, SizeOf(TTBXItemInfo), 0);
   IInfo.ViewType := TVT_NORMALTOOLBAR;
   IInfo.ItemOptions := IO_APPACTIVE or IO_TOOLBARSTYLE;
   IInfo.Enabled := Enabled;
   IInfo.IsVertical := false;

   if state = 'normal' then
    begin
     IInfo.Pushed := false;
     IInfo.Selected := false;
     IInfo.HoverKind := hkNone;
    end;
   if state = 'over' then
    begin
     IInfo.Pushed := false;
     IInfo.Selected := false;
     IInfo.HoverKind := hkMouseHover;
    end;
   if state = 'down' then
    begin
     IInfo.Pushed := true;
     IInfo.Selected := false;
     IInfo.HoverKind := hkMouseHover;
    end;
   if State = 'checked' then
    begin
     IInfo.Pushed := False;
     IInfo.Selected := True;
     IInfo.HoverKind := hkNone;
    end;
   if State = 'checked-hot' then
    begin
     IInfo.Pushed := False;
     IInfo.Selected := True;
     IInfo.HoverKind := hkMouseHover;
    end;
   if State = 'checked-down' then
    begin
     IInfo.Pushed := True;
     IInfo.Selected := True;
     IInfo.HoverKind := hkMouseHover;
    end;
end;

procedure TmbTBXProgressBar.TBMThemeChange(var Message: TMessage);
begin
if Message.WParam = TSC_VIEWCHANGE then
 begin
  DrawChunk;
  Invalidate;
 end;
end;

procedure TmbTBXProgressBar.DrawChunk;
var
 ii: TTBXItemInfo;
begin
 if Chunk = nil then
  begin
   Chunk := TBitmap.Create;
   Chunk.PixelFormat:= pf32bit;
  end;
 Chunk.Width := 10;
 Chunk.Height := Height - 4;
 SetItemInfo('over', ii);
 ii.Enabled := Enabled;
 Chunk.Canvas.Brush.Color := CurrentTheme.GetViewColor(TVT_NORMALTOOLBAR);
 Chunk.Canvas.FillRect(Chunk.Canvas.ClipRect);
 CurrentTheme.PaintButton(Chunk.Canvas, Rect(0, 0, 10, Height - 4), ii);
end;

procedure TmbTBXProgressBar.Resize;
begin
 inherited;
 DrawChunk;
 Invalidate;
end;

procedure TmbTBXProgressBar.AddPosition(APos: integer);
begin
 Inc(FPosition, APos);
 Invalidate;
end;

procedure TmbTBXProgressBar.CMFontChanged(var Message: TMessage);
begin
 TempBmp.Canvas.Font.Assign(Self.Font);
 Invalidate;
end;

procedure TmbTBXProgressBar.SetMax(const Value: integer);
begin
 if FMax <> Value then
  begin
   FMax := Value;
   Invalidate;
  end;
end;

procedure TmbTBXProgressBar.SetMin(const Value: integer);
begin
 if FMin <> Value then
  begin
   FMin := Value;
   Invalidate;
  end;
end;

procedure TmbTBXProgressBar.SetPercentage(const Value: boolean);
begin
 if FPercentage <> Value then
  begin
   FPercentage := Value;
   Invalidate;
  end;
end;

procedure TmbTBXProgressBar.SetPosition(const Value: integer);
begin
if FPosition <> Value then
 begin
  if Value > FMax then
   FPosition := FMax
  else
   if Value < FMin then
    FPosition := FMin
   else
    FPosition := Value;
  Invalidate;
 end;
end;

procedure TmbTBXProgressBar.SetPosTextPrefix(const Value: string);
begin
 if FPosTextPrefix <> Value then
  begin
   FPosTextPrefix := Value;
   Invalidate;
  end;
end;

procedure TmbTBXProgressBar.SetPosTextSuffix(const Value: string);
begin
 if FPosTextSuffix <> Value then
  begin
   FPosTextSuffix := Value;
   Invalidate;
  end;
end;

procedure TmbTBXProgressBar.SetShowPosText(const Value: boolean);
begin
 if FShowPosText <> Value then
  begin
   FShowPosText := Value;
   Invalidate;
  end;
end;

procedure TmbTBXProgressBar.WMEraseBkgnd(var Message: TWMEraseBkgnd);
begin
 Message.Result := 1;
end;

procedure TmbTBXProgressBar.CMEnabledChanged(var Message: TMessage);
begin
 inherited;
 DrawChunk;
 Invalidate;
end;

procedure TmbTBXProgressBar.SetChunked(Value: boolean);
begin
 if FChunked <> Value then
  begin
   FChunked := Value;
   Invalidate;
  end;
end;

end.

