unit mbTBXReg;

interface

{$I mxs.inc}

uses
  Windows, SysUtils, Classes, Controls, ImgList, Graphics,
  {$IFDEF DELPHI_6_UP} DesignIntf, DesignEditors, VCLEditors
  {$ELSE} DsgnIntf {$ENDIF}, mbTBXArtCombo, mbTBXBooleanCombo,
  mbTBXBrushStyleCombo, mbTBXButton, mbTBXCheckGroup, mbTBXCheckListBox,
  {$IFDEF DELPHI_6_UP} mbTBXColorBox, {$ENDIF}
  mbTBXComboBox, mbTBXComboEdit, mbTBXEdit, mbTBXFontCombo, mbTBXGroupBox, mbTBXHeaderControl,
  mbTBXHint, mbTBXHotKeyEdit, mbTBXImageComboBox, mbTBXImageListCombo, mbTBXListBox,
  mbTBXMaskEdit, mbTBXMediaPlayer, mbTBXMemo, mbTBXPanel, mbTBXPenStyleCombo, mbTBXPenWidthCombo,
  mbTBXProgressBar, mbTBXRadioGroup, mbTBXRichEdit, mbTBXScrollBar, mbTBXScrollBox,
  {$IFDEF DELPHI_6_UP} mbTBXShellTreeView, {$ENDIF}
  mbTBXSpinButton, mbTBXSpinEdit, mbTBXTreeView, mbTBXValidateEdit,
  mbTBXExpandablePanel, mbTBXClipartBox, mbTBXClipartScrollBox, mbTBXFloatSpinEdit,
  mbTBXNonDockablePanel, mbTBXStringGrid, mbTBXDrawGrid, mbTBXColorProvider, mbTBXSplitter,
  mbTBXColorPalette, mbTBXArrowButton, mbTBXArrowEdit, mbTBXBevel, mbTBXImageRadio,
  mbTBXImageCheck, mbTBXLineGroup, mbTBXFixedStuff, mbTBXVerticalRuler, mbTBXHorizontalRuler,
  mbTBXPageScroller, mbTBXImageListBox;

type
  TmbTBXImageIndexPropertyEditor = class(TIntegerProperty
    {$IFDEF DELPHI_6_UP} , ICustomPropertyListDrawing {$ENDIF})
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
    function GetImageListAt(Index: Integer): TCustomImageList; virtual;

    procedure ListMeasureHeight(const Value: string; ACanvas: TCanvas;
      var AHeight: Integer); {$IFNDEF DELPHI_6_UP} override; {$ENDIF}
    procedure ListMeasureWidth(const Value: string; ACanvas: TCanvas;
      var AWidth: Integer); {$IFNDEF DELPHI_6_UP} override; {$ENDIF}
    procedure ListDrawValue(const Value: string; ACanvas: TCanvas;
      const ARect: TRect; ASelected: Boolean); {$IFNDEF DELPHI_6_UP} override; {$ENDIF}
  end;

  TmbTBXControlImageIndexPropertyEditor = class(TmbTBXImageIndexPropertyEditor)
  public
    function GetImageListAt (Index: Integer): TCustomImageList; override;
  end;

procedure Register;

implementation

function TmbTBXImageIndexPropertyEditor.GetAttributes: TPropertyAttributes;
begin
  Result := [paMultiSelect, paValueList, paRevertable];
end;

function TmbTBXImageIndexPropertyEditor.GetImageListAt(Index: Integer): TCustomImageList;
begin
  Result := nil;
end;

procedure TmbTBXImageIndexPropertyEditor.GetValues(Proc: TGetStrProc);
var
  ImgList: TCustomImageList;
  I: Integer;
begin
  ImgList := GetImageListAt(0);
  if Assigned(ImgList) then
    for I := 0 to ImgList.Count-1 do
      Proc(IntToStr(I));
end;

procedure TmbTBXImageIndexPropertyEditor.ListDrawValue(const Value: string;
  ACanvas: TCanvas; const ARect: TRect; ASelected: Boolean);
var
  ImgList: TCustomImageList;
  X: Integer;
begin
  ImgList := GetImageListAt(0);
  ACanvas.FillRect(ARect);
  X := ARect.Left + 2;
  if Assigned(ImgList) then
   begin
    ImgList.Draw(ACanvas, X, ARect.Top + 2, StrToInt(Value));
    Inc(X, ImgList.Width);
   end;
  ACanvas.TextOut(X + 3, ARect.Top + 1, Value);
end;

procedure TmbTBXImageIndexPropertyEditor.ListMeasureHeight(const Value: string;
  ACanvas: TCanvas; var AHeight: Integer);
var
  ImgList: TCustomImageList;
begin
  ImgList := GetImageListAt(0);
  AHeight := ACanvas.TextHeight(Value) + 2;
  if Assigned(ImgList) and (ImgList.Height + 4 > AHeight) then
    AHeight := ImgList.Height + 4;
end;

procedure TmbTBXImageIndexPropertyEditor.ListMeasureWidth(const Value: string;
  ACanvas: TCanvas; var AWidth: Integer);
var
  ImgList: TCustomImageList;
begin
  ImgList := GetImageListAt(0);
  AWidth := ACanvas.TextWidth(Value) + 4;
  if Assigned(ImgList) then
    Inc(AWidth, ImgList.Width);
end;

function TmbTBXControlImageIndexPropertyEditor.GetImageListAt(Index: Integer): TCustomImageList;
var
  C: TPersistent;
begin
  Result := nil;
  C := GetComponent(Index);
  if C is TmbTBXArrowButton then
   Result := (C as TmbTBXArrowButton).Images;

  if C is TmbTBXArrowEdit then
   Result := (C as TmbTBXArrowEdit).ButtonImageList;

  if C is TmbTBXButton then
   Result := (C as TmbTBXButton).Images;

  if C is TmbTBXClipartBox then
   Result := (C as TmbTBXClipartBox).Images;

  if C is TmbTBXClipartScrollBox then
   Result := (C as TmbTBXClipartScrollBox).Images;

  if C is TmbTBXComboEdit then
   Result := (C as TmbTBXComboEdit).ButtonImageList;

  if C is TmbTBXImageCheck then
   Result := (C as TmbTBXImageCheck).Images;

  if C is TmbTBXImageRadio then
   Result := (C as TmbTBXImageRadio).Images;
end;

procedure Register;
begin
 RegisterComponents('mbTBX Lib', [TmbTBXArrowButton]);
 RegisterPropertyEditor(TypeInfo(TImageIndex), TmbTBXArrowButton, 'ImageIndex',
    TmbTBXControlImageIndexPropertyEditor);

 RegisterComponents('mbTBX Lib', [TmbTBXArrowEdit]);
 RegisterPropertyEditor(TypeInfo(TImageIndex), TmbTBXArrowEdit, 'ButtonImageIndex',
    TmbTBXControlImageIndexPropertyEditor);

 RegisterComponents('mbTBX Lib', [TmbTBXArtCombo]);
 RegisterComponents('mbTBX Lib', [TmbTBXBevel]);
 RegisterComponents('mbTBX Lib', [TmbTBXBooleanCombo]);
 RegisterComponents('mbTBX Lib', [TmbTBXBrushStyleCombo]);

 RegisterComponents('mbTBX Lib', [TmbTBXButton]);
 RegisterPropertyEditor(TypeInfo(TImageIndex), TmbTBXButton, 'ImageIndex',
    TmbTBXControlImageIndexPropertyEditor);

 RegisterComponents('mbTBX Lib', [TmbTBXCheckGroup]);
 RegisterComponents('mbTBX Lib', [TmbTBXCheckListBox]);

 RegisterComponents('mbTBX Lib', [TmbTBXClipartBox]);
 RegisterPropertyEditor(TypeInfo(TImageIndex), TmbTBXClipartBox, 'ImageIndex',
    TmbTBXControlImageIndexPropertyEditor);

 RegisterComponents('mbTBX Lib', [TmbTBXClipartScrollBox]);
 RegisterPropertyEditor(TypeInfo(TImageIndex), TmbTBXClipartScrollBox, 'ImageIndex',
    TmbTBXControlImageIndexPropertyEditor);

 {$IFDEF DELPHI_6_UP}
 RegisterComponents('mbTBX Lib', [TmbTBXColorBox]);
 {$ENDIF}

 RegisterComponents('mbTBX Lib', [TmbTBXColorPalette]);
 RegisterComponents('mbTBX Lib', [TmbTBXColorProvider]);
 RegisterComponents('mbTBX Lib', [TmbTBXComboBox]);

 RegisterComponents('mbTBX Lib', [TmbTBXComboEdit]);
 RegisterPropertyEditor(TypeInfo(TImageIndex), TmbTBXComboEdit, 'ButtonImageIndex',
    TmbTBXControlImageIndexPropertyEditor);

 RegisterComponents('mbTBX Lib', [TmbTBXDrawGrid]);
 RegisterComponents('mbTBX Lib', [TmbTBXEdit]);
 RegisterComponents('mbTBX Lib', [TmbTBXExpandablePanel]);
 RegisterComponents('mbTBX Lib', [TmbTBXLabel, TmbTBXCheckBox, TmbTBXRadioButton]);
 RegisterComponents('mbTBX Lib', [TmbTBXFloatSpinEdit]);
 RegisterComponents('mbTBX Lib', [TmbTBXFontCombo]);
 RegisterComponents('mbTBX Lib', [TmbTBXGroupBox]);
 RegisterComponents('mbTBX Lib', [TmbTBXHeaderControl]);
 RegisterComponents('mbTBX Lib', [TmbTBXHint]);
 RegisterComponents('mbTBX Lib', [TmbTBXHorizontalRuler]);
 RegisterComponents('mbTBX Lib', [TmbTBXHotKeyEdit]);

 RegisterComponents('mbTBX Lib', [TmbTBXImageCheck]);
 RegisterPropertyEditor(TypeInfo(TImageIndex), TmbTBXImageCheck, 'ImageIndex',
    TmbTBXControlImageIndexPropertyEditor);

 RegisterComponents('mbTBX Lib', [TmbTBXImageComboBox]);
 RegisterComponents('mbTBX Lib', [TmbTBXImageListBox]);
 RegisterComponents('mbTBX Lib', [TmbTBXImageListCombo]);

 RegisterComponents('mbTBX Lib', [TmbTBXImageRadio]);
 RegisterPropertyEditor(TypeInfo(TImageIndex), TmbTBXImageRadio, 'ImageIndex',
    TmbTBXControlImageIndexPropertyEditor);

 RegisterComponents('mbTBX Lib', [TmbTBXLineGroup]);
 RegisterComponents('mbTBX Lib', [TmbTBXListBox]);
 RegisterComponents('mbTBX Lib', [TmbTBXMaskEdit]);
 RegisterComponents('mbTBX Lib', [TmbTBXMediaPlayer]);
 RegisterComponents('mbTBX Lib', [TmbTBXMemo]);
 RegisterComponents('mbTBX Lib', [TmbTBXNonDockablePanel]);
 RegisterComponents('mbTBX Lib', [TmbTBXPageScroller]);
 RegisterComponents('mbTBX Lib', [TmbTBXPanel]);
 RegisterComponents('mbTBX Lib', [TmbTBXPenStyleCombo]);
 RegisterComponents('mbTBX Lib', [TmbTBXPenWidthCombo]);
 RegisterComponents('mbTBX Lib', [TmbTBXProgressBar]);
 RegisterComponents('mbTBX Lib', [TmbTBXRadioGroup]);
 RegisterComponents('mbTBX Lib', [TmbTBXRichEdit]);
 RegisterComponents('mbTBX Lib',[TmbTBXScrollBar]);
 RegisterComponents('mbTBX Lib', [TmbTBXScrollBox]);
 RegisterComponents('mbTBX Lib', [TmbTBXShellTreeView]);
 RegisterComponents('mbTBX Lib', [TmbTBXSpinButton]);
 RegisterComponents('mbTBX Lib', [TmbTBXSpinEdit]);
 RegisterComponents('mbTBX Lib', [TmbTBXSplitter]);
 RegisterComponents('mbTBX Lib', [TmbTBXStringGrid]);
 RegisterComponents('mbTBX Lib', [TmbTBXTreeView]);
 RegisterComponents('mbTBX Lib', [TmbTBXValidateEdit]);
 RegisterComponents('mbTBX Lib', [TmbTBXVerticalRuler]);
end;

end.
