unit mbTBXScrollBar;

{$I mxs.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Dialogs, StdCtrls,
  mbTBXButton, TBX, TBXUtils, TBXThemes, ImgList, ExtCtrls;

{ TmbTBXScrollbarThumb }

type
  TScrollBarKind = (sbHorizontal, sbVertical);
  TScrollDirection = (sdUp, sdDown, sdLeft, sdRight);
  TScrollArrowStyle = (asFlat, asXPStyle);
  TOnScroll = procedure (Sender: TObject; ScrollPos: Integer) of object;
  TTrackScroll = procedure of object;

  TmbTBXScrollbarThumb = class(TmbTBXButton)
  private
    FDown: Boolean;
    FOldX, FOldY: Integer;
    FTopLimit: Integer;
    FBottomLimit: Integer;
  protected
    FKind: TScrollbarKind;
    FImages: TCustomImageList;

    procedure WMPaint(var Msg: TWMPaint); message WM_PAINT;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  public
    constructor Create(AOwner: TComponent); override;
  end;

{ TmbTBXScrollbarTrack }

type
  TmbTBXScrollbarTrack = class (TCustomControl)
  private
    FThumb: TmbTBXScrollbarThumb;
    FKind: TScrollBarKind;
    FSmallChange: Integer;
    FLargeChange: Integer;
    FMin: Integer;
    FMax: Integer;
    FPosition: Integer;
    FPageSize: integer;
    FBtnSize: integer;
    FTimer: TTimer;
    FUp: boolean;
    FLPos: integer;
    FOnScroll: TTrackScroll;

    procedure SetBtnSize(Value: integer);
    procedure SetPageSize(Value: integer);
    procedure SetSmallChange(Value: Integer);
    procedure SetLargeChange(Value: Integer);
    procedure SetMin(Value: Integer);
    procedure SetMax(Value: Integer);
    procedure SetPosition(Value: Integer);
    procedure SetKind(Value: TScrollBarKind);
    procedure WMSize(var Message: TMessage); message WM_SIZE;
    function ThumbFromPosition: Integer;
    function PositionFromThumb: Integer;
    procedure DoPositionChange;
    procedure DoHScroll(var Message: TWMScroll);
    procedure DoVScroll(var Message: TWMScroll);
    procedure DoEnableArrows(var Message: TMessage);
    procedure DoGetPos(var Message: TMessage);
    procedure DoGetRange(var Message: TMessage);
    procedure DoSetPos(var Message: TMessage);
    procedure DoSetRange(var Message: TMessage);
    procedure DoKeyDown(var Message: TWMKeyDown);
  protected
    FImages: TCustomImageList;

    procedure Paint; override;
    procedure UpdatePageSize;
    procedure UpdateThumbSize;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure DoOnTimer(Sender: TObject);
    function DoMouseWheelDown(Shift: TShiftState; MousePos: TPoint): Boolean; override;
    function DoMouseWheelUp(Shift: TShiftState; MousePos: TPoint): Boolean; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Align;
    property Min: Integer read FMin write SetMin;
    property Max: Integer read FMax write SetMax;
    property SmallChange: Integer read FSmallChange write SetSmallChange default 1;
    property LargeChange: Integer read FLargeChange write SetLargeChange default 10;
    property Position: Integer read FPosition write SetPosition;
    property Kind: TScrollBarKind read FKind write SetKind;

    property OnScroll: TTrackScroll read FOnScroll write FOnScroll;
  end;

{ TmbTBXScrollbarButton }

type
  TmbTBXScrollbarButton = class(TmbTBXButton)
  private
    FNewDown: Boolean;
    FTimer: TTimer;
    FOnDown: TNotifyEvent;
    FInterval: integer;

    procedure DoTimer(Sender: TObject);
  protected
    FKind: TScrollBarKind;
    FImages: TCustomImageList;
    FImageIndex: TImageIndex;
    FDirection: TScrollDirection;
    FStyle: TScrollArrowStyle;

    procedure SetImageIndex(i: TImageIndex);
    procedure WMPaint(var Msg: TWMPaint); message WM_PAINT;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Align;
    property OnDown: TNotifyEvent read FOnDown write FOnDown;
  end;

{ TmbTBXScrollBar }

type
  TmbTBXScrollBar = class(TCustomControl)
  private
    FTrack: TmbTBXScrollbarTrack;
    FBtnOne: TmbTBXScrollbarButton;
    FBtnTwo: TmbTBXScrollbarButton;
    FMin: Integer;
    FMax: Integer;
    FSmallChange: Integer;
    FLargeChange: Integer;
    FPosition: Integer;
    FKind: TScrollBarKind;
    FOnScroll: TOnScroll;
    FImages: TCustomImageList;
    FImageChangeLink: TChangeLink;
    FStyle: TScrollArrowStyle;
    FHotTrack: boolean;
    FExtraFrame: boolean;
    FPageSize: integer;
    FBtnSize: integer;
    FSInterval: integer;

    procedure SetSInterval(Value: integer);
    procedure SetBtnSize(Value: integer);
    procedure SetPageSize(Value: integer);
    procedure SetExtraFrame(Value: boolean);
    procedure SetHotTrack(Value: boolean);
    procedure SetStyle(s: TScrollArrowStyle);
    procedure ImageListChange(Sender: TObject);
    Procedure SetImages(i: TCustomImageList);
    procedure SetSmallChange(Value: Integer);
    procedure SetLargeChange(Value: Integer);
    procedure SetMin(Value: Integer);
    procedure SetMax(Value: Integer);
    procedure SetPosition(Value: Integer);
    procedure SetKind(Value: TScrollBarKind);
    procedure BtnOneClick(Sender: TObject);
    procedure BtnTwoClick(Sender: TObject);
    procedure EnableBtnOne(Value: Boolean);
    procedure EnableBtnTwo(Value: Boolean);
    procedure DoScroll;
  protected
    procedure UpdatePageSize;
    procedure UpdateThumbSize;
    procedure UpdateSize;
    procedure TrackScroll;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure WMSize(var Message: TWMSize); message WM_SIZE;
    procedure CNHScroll(var Message: TWMScroll); message WM_HSCROLL;
    procedure CNVScroll(var Message: TWMScroll); message WM_VSCROLL;
    procedure SBMEnableArrows(var Message: TMessage); message SBM_ENABLE_ARROWS;
    procedure SBMGetPos(var Message: TMessage); message SBM_GETPOS;
    procedure SBMGetRange(var Message: TMessage); message SBM_GETRANGE;
    procedure SBMSetPos(var Message: TMessage); message SBM_SETPOS;
    procedure SBMSetRange(var Message: TMessage); message SBM_SETRANGE;
    procedure WMKeyDown(var Message: TWMKeyDown); message WM_KEYDOWN;
    procedure TBMThemeChange(var Message: TMessage); message TBM_THEMECHANGE;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure Center;
  published
    property ScrollArrowStyle: TScrollArrowStyle read FStyle write SetStyle default asXPStyle;
    property Images: TCustomImageList read FImages write SetImages;
    property Min: Integer read FMin write SetMin default 0;
    property Max: Integer read FMax write SetMax default 100;
    property SmallChange: Integer read FSmallChange write SetSmallChange default 1;
    property LargeChange: Integer read FLargeChange write SetLargeChange default 10;
    property Position: Integer read FPosition write SetPosition default 0;
    property Kind: TScrollBarKind read FKind write SetKind default sbVertical;
    property HotTracking: boolean read FHotTrack write SetHotTrack default true;
    property ExtraFrame: boolean read FExtraFrame write SetExtraFrame default false;
    property PageSize: integer read FPageSize write SetPageSize default 0;
    property ButtonSize: integer read FbtnSize write SetBtnSize default 18;
    property ScrollInterval: integer read FSInterval write SetSInterval default 20;
    property Align;
    property Anchors;
    property Constraints;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop default True;
    property Visible;

    property OnContextPopup;
    property OnEndDock;
    property OnKeyPress;
    property OnStartDock;
    property OnStartDrag;
    property OnScroll: TOnScroll read FOnScroll write FOnScroll;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyUp;
  end;

implementation

{ TmbTBXScrollbarTrackThumb }

constructor TmbTBXScrollbarThumb.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
 DoubleBuffered := true;
 ShowFocus := false;
end;

procedure SetItemInfo(state: string; var IInfo: TTBXItemInfo);
begin
   FillChar(IInfo, SizeOf(TTBXItemInfo), 0);
   IInfo.ViewType := TVT_NORMALTOOLBAR;
   IInfo.ItemOptions := IO_APPACTIVE or IO_TOOLBARSTYLE;
   IInfo.Enabled := True;

   if state = 'normal' then
   begin
     IInfo.Pushed := false;
     IInfo.Selected := false;
     IInfo.HoverKind := hkNone;
   end;
   if state = 'over' then
   begin
     IInfo.Pushed := false;
     IInfo.Selected := false;
     IInfo.HoverKind := hkMouseHover;
   end;
   if state = 'down' then
   begin
     IInfo.Pushed := true;
     IInfo.Selected := false;
     IInfo.HoverKind := hkMouseHover;
   end;
   if State = 'checked' then
  begin
    IInfo.Pushed := False;
    IInfo.Selected := True;
    IInfo.HoverKind := hkNone;
  end;
  if State = 'checked-hot' then
  begin
    IInfo.Pushed := False;
    IInfo.Selected := True;
    IInfo.HoverKind := hkMouseHover;
  end;
  if State = 'checked-down' then
  begin
    IInfo.Pushed := True;
    IInfo.Selected := True;
    IInfo.HoverKind := hkMouseHover;
  end;
end;

procedure TmbTBXScrollBarThumb.WMPaint(var Msg: TWMPaint);
var
 DC: HDC;
 ii: TTBXItemInfo;
 R: TRect;
 c: TCanvas;
begin
 inherited;
 C := TCanvas.Create;
 DC := GetWindowDC(Handle);
 GetWindowRect(Handle, R);
 OffsetRect(R, -R.Left, -R.Top);
 C.Handle := DC;
 SetItemInfo('over', ii);
 ii.IsVertical := (FKind = sbVertical);
 if (csDesigning in ComponentState) or (Enabled = false) then FMouseLoc := mlNone;
 case FMouseLoc of
  mlNone: SetItemInfo('normal', ii);
  mlOver: SetItemInfo('over', ii);
  mlDown: SetItemInfo('down', ii);
 end;
 ii.Enabled := Enabled;
 ii.IsVertical := (FKind = sbVertical);
 //paint image
 if FImages <> nil then
  begin
   with R do
    begin
     R.Top := R.Top + ((R.Bottom - R.Top) - FImages.Height) div 2;
     R.Bottom := R.Top + FImages.Height;
     R.Left := R.Left + ((R.Right - R.Left) - FImages.Width) div 2;
     R.Right := R.Left + FImages.Width;
    end;
   CurrentTheme.PaintImage(C, R, ii, FImages, 2);
  end;
 //end paint image
 C.Handle := 0;
 ReleaseDC(Handle, DC);
 C.Free;
end;

procedure TmbTBXScrollbarThumb.MouseMove(Shift: TShiftState; X, Y: Integer);
var
  iTop: Integer;
begin
  if TmbTBXScrollbarTrack(Parent).Kind = sbVertical then
  begin
    FTopLimit := 0;
    FBottomLimit := TmbTBXScrollbarTrack(Parent).Height;
    if FDown = True then
    begin
      iTop := Top + Y - FOldY;
      if iTop < FTopLimit then
      begin
        iTop := FTopLimit;
      end;
      if (iTop > FBottomLimit) or ((iTop + Height) > FBottomLimit) then
      begin
        iTop := FBottomLimit - Height;
      end;
      Top := iTop;
    end;
  end
  else
  begin
    FTopLimit := 0;
    FBottomLimit := TmbTBXScrollbarTrack(Parent).Width;
    if FDown = True then
    begin
      iTop := Left + X - FOldX;
      if iTop < FTopLimit then
      begin
        iTop := FTopLimit;
      end;
      if (iTop > FBottomLimit) or ((iTop + Width) > FBottomLimit) then
      begin
        iTop := FBottomLimit - Width;
      end;
      Left := iTop;
    end;
  end;
  TmbTBXScrollbarTrack(Parent).FPosition := TmbTBXScrollbarTrack(Parent).PositionFromThumb;
  TmbTBXScrollbarTrack(Parent).DoPositionChange;
  inherited MouseMove(Shift,X,Y);
end;

procedure TmbTBXScrollbarThumb.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  FDown := False;
  inherited MouseUp(Button,Shift,X,Y);
end;

procedure TmbTBXScrollbarThumb.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if (Button = mbleft) and not FDown then FDown := True;
  FOldX := X;
  FOldy := Y;
  inherited MouseDown(Button,Shift,X,Y);
end;

{ TmbTBXScrollbarTrack }

constructor TmbTBXScrollbarTrack.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  DoubleBuffered := true;
  FThumb := TmbTBXScrollbarThumb.Create(Self);
  FThumb.FKind := FKind;
  FThumb.FImages := FImages;
  FThumb.Height := 17;
  InsertControl(FThumb);
  FMin := 0;
  FMax := 100;
  FSmallChange := 1;
  FLargeChange := 10;
  FPosition := 0;
  FThumb.Top := ThumbFromPosition;
  FPageSize := 0;
  FTimer := TTimer.Create(Self);
  FTimer.Interval := 20;
  FTimer.OnTimer := DoOnTimer;
  FUp := false;
  FlPos := 0;
end;

destructor TmbTBXScrollbarTrack.Destroy;
begin
  FThumb.Free;
  FTimer.Free;
  inherited Destroy;
end;

procedure TmbTBXScrollbarTrack.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
 inherited;
 if not (ssLeft in Shift) then Exit;
 case FKind of
  sbHorizontal:
   begin
    FlPos := X;
    FUp := X < FThumb.Left;
   end;
  sbVertical:
   begin
    FlPos := Y;
    FUp := Y < FThumb.Top;
   end;
 end;
 FTimer.Enabled := true;
end;

procedure TmbTBXScrollbarTrack.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 inherited;
 if Button <> mbLeft then Exit;
 case FKind of
  sbHorizontal:
   begin
    FlPos := X;
    FUp := X < FThumb.Left;
   end;
  sbVertical:
   begin
    FlPos := Y;
    FUp := Y < FThumb.Top;
   end;
 end;
 FTimer.Enabled := true;
end;

procedure TmbTBXScrollbarTrack.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 if Button = mbLeft then
  FTimer.Enabled := false;
 inherited;
end;

procedure TmbTBXScrollbarTrack.DoOnTimer(Sender: TObject);
var
 m: TWMScroll;
begin
 case FKind of
  sbHorizontal:
   begin
    if FUp then
     begin
      if FThumb.Left > FLPos then
       begin
        if FThumb.Left - FLargeChange >= FLpos then
         m.ScrollCode := SB_PAGELEFT
        else
         m.ScrollCode := SB_LINELEFT;
        DoHScroll(m)
       end
      else
       FTimer.Enabled := false;
     end
    else
     begin
      if FThumb.Left + FThumb.Width < FLPos then
       begin
        if FThumb.Left + FThumb.Width + FLargeChange <= FLpos then
         m.ScrollCode := SB_PAGERIGHT
        else
         m.ScrollCode := SB_LINERIGHT;
        DoHSCroll(m);
       end
      else
       FTimer.Enabled := false;
     end;
   end;
  sbVertical:
   begin
    if FUp then
     begin
      if FThumb.Top > FLPos then
       begin
        if FThumb.Top - FLargeChange >= FLpos then
         m.ScrollCode := SB_PAGEUP
        else
         m.ScrollCode := SB_LINEUP;
        DoVScroll(m)
       end
      else
       FTimer.Enabled := false;
     end
    else
     begin
      if FThumb.Top + FThumb.Height < FLPos then
       begin
        if FThumb.Top + FThumb.Height + FLargeChange <= FLpos then
         m.ScrollCode := SB_PAGEDOWN
        else
         m.ScrollCode := SB_LINEDOWN;
        DoVSCroll(m);
       end
      else
       FTimer.Enabled := false;
     end;
   end;
 end;
 if Assigned(FOnScroll) then FOnScroll;
end;

function TmbTBXScrollbarTrack.DoMouseWheelDown(Shift: TShiftState; MousePos: TPoint): Boolean;
var
 m: TWMScroll;
begin
 Result := inherited DoMouseWheelDown(Shift, MousePos);
 case FKind of
  sbHorizontal:
   begin
    m.ScrollCode := SB_LINERIGHT;
    DoHScroll(m);
   end;
  sbVertical:
   begin
    m.ScrollCode := SB_LINEDOWN;
    DoVScroll(m);
   end;
 end;
 if Assigned(FOnScroll) then FOnScroll;
end;

function TmbTBXScrollbarTrack.DoMouseWheelUp(Shift: TShiftState; MousePos: TPoint): Boolean;
var
 m: TWMScroll;
begin
 Result := inherited DoMouseWheelUp(Shift, MousePos);
 case FKind of
  sbHorizontal:
   begin
    m.ScrollCode := SB_LINELEFT;
    DoHScroll(m);
   end;
  sbVertical:
   begin
    m.ScrollCode := SB_LINEUP;
    DoVScroll(m);
   end;
 end;
 if Assigned(FOnScroll) then FOnScroll;
end;

procedure TmbTBXScrollbarTrack.Paint;
var
 R: TRect;
 ii: TTBXItemInfo;
begin
 R := ClientRect;
 SetItemInfo('checked-down', ii);
 Canvas.Brush.Color := Blend(CurrentTheme.GetItemColor(ii), Color, 30);
 Canvas.FillRect(R);
 //Currenttheme.PaintButton(Canvas, R, ii);
end;

procedure TmbTBXScrollbarTrack.SetSmallChange(Value: Integer);
begin
 if Value <> FSmallChange then
  begin
   if Value < 1 then
    Value := 1;
   FSmallChange := Value;
  end;
end;

procedure TmbTBXScrollbarTrack.SetBtnSize(Value: Integer);
begin
 if Value <> FBtnSize then
  begin
   FBtnSize := Value;
   UpdatePageSize;
   UpdateThumbSize;
  end;
end;

procedure TmbTBXScrollbarTrack.SetLargeChange(Value: Integer);
begin
 if Value <> FLargeChange then
  begin
   if Value < 1 then
    Value := 1;
   FLargeChange := Value;
  end;
end;

procedure TmbTBXScrollbarTrack.SetPageSize(Value: integer);
begin
 if Value < 1 then
  Value := 1;
 if Value > (FMax - FMin) then
  Value := FMax - FMin;
 if FPageSize <> Value then
  begin
   FPageSize := Value;
   UpdatePageSize;
   UpdateThumbSize;
  end;
end;

procedure TmbTBXScrollbarTrack.UpdatePageSize;
begin
 if FPageSize > (FMax - FMin) then
  FPageSize := FMax - FMin;
end;

procedure TmbTBXScrollbarTrack.UpdateThumbSize;
var
 size, sThumb:integer;
begin
  if (FKind = sbHorizontal) then
   size := Width
  else
   size := Height;

  Dec(size, 2 * FBtnSize);

  if size <= 0 then
   sThumb := 0
  else
   begin
    if FPageSize = 0 then
     sThumb := 7
    else
     sThumb := size div ((FMax - FMin) div FPageSize);

    if sThumb < 7 then
      sThumb := 7
    else
     if sThumb > size then
      sThumb := size;
   end;

 if FKind = sbHorizontal then
  FThumb.Width := sThumb
 else
  FThumb.Height := sThumb;
end;

procedure TmbTBXScrollbarTrack.SetMin(Value: Integer);
begin
 if Value >= FMax then
  Value := FMax  - 1;
 if Value >= FMax - FPageSize then
  Value := FMax  - FPageSize;
 if (Value <> FMin) and (Value > -1) then
  begin
   FMin := Value;
   UpdatePageSize;
   UpdateThumbSize;
   FThumb.Top := ThumbFromPosition;
  end;
end;

procedure TmbTBXScrollbarTrack.SetMax(Value: Integer);
begin
 if Value <= FMin then
  Value := FMin + 1;
 if Value <= FMin + FPageSize then
  Value := FMin  + FPageSize;
 if (Value <> FMax) and (Value > -1) then
  begin
   FMax := Value;
   UpdatePageSize;
   UpdateThumbSize;
   FThumb.Top := ThumbFromPosition;
  end;
end;

procedure TmbTBXScrollbarTrack.SetPosition(Value: Integer);
begin
  FPosition := Value;
  if Position > Max then Position := Max;
  if Position < Min then Position := Min;
  case FKind of
   sbVertical: FThumb.Top := ThumbFromPosition;
   sbHorizontal: FThumb.Left := ThumbFromPosition;
  end;
  DoPositionChange;
end;

procedure TmbTBXScrollbarTrack.SetKind(Value: TScrollBarKind);
begin
 if Value <> FKind then
  begin
   FKind:= Value;
   case FKind of
    sbVertical: FThumb.Height := FThumb.Width;
    sbHorizontal: FThumb.Width := FThumb.Height;
   end;
   UpdateThumbSize;
  end;
 Position := FPosition;
end;

procedure TmbTBXScrollbarTrack.WMSize(var Message: TMessage);
begin
  if FKind = sbVertical then
   FThumb.Width := Width
  else
   FThumb.Height := Height;
end;

function TmbTBXScrollbarTrack.ThumbFromPosition: Integer;
var
 size: Integer;
begin
 size := 0;
 case FKind of
  sbVertical: size := Height - FThumb.Height;
  sbHorizontal: size := Width - FThumb.Width;
 end;
 Result := Round((size / (FMax - FMin)) * FPosition);
end;

function TmbTBXScrollbarTrack.PositionFromThumb: Integer;
var
 size, pos: Integer;
begin
 size := 0;
 case FKind of
  sbVertical: size :=  Height - FThumb.Height;
  sbHorizontal: size := Width - FThumb.Width;
 end;
 pos := 0;
 case FKind of
  sbVertical: pos := FThumb.Top;
  sbHorizontal: pos := FThumb.Left;
 end;
 if size > 0 then
  Result := Round(pos / size * (FMax - FMin))
 else
  Result := pos;
end;

procedure TmbTBXScrollbarTrack.DoPositionChange;
begin
 if Assigned(Parent) then
  begin
   TmbTBXScrollBar(Parent).FPosition := Position;
   TmbTBXScrollBar(Parent).DoScroll;
  end;
end;

procedure TmbTBXScrollbarTrack.DoHScroll(var Message: TWMScroll);
var
  iPosition: Integer;
begin
  case Message.ScrollCode of
    SB_BOTTOM: Position := Max;
    SB_LINELEFT:
     begin
      iPosition := Position;
      Dec(iPosition,SmallChange);
      Position := iPosition;
     end;
    SB_LINERIGHT:
     begin
      iPosition := Position;
      Inc(iPosition,SmallChange);
      Position := iPosition;
     end;
    SB_PAGELEFT:
     begin
      iPosition := Position;
      Dec(iPosition,LargeChange);
      Position := iPosition;
     end;
    SB_PAGERIGHT:
     begin
      iPosition := Position;
      Inc(iPosition,LargeChange);
      Position := iPosition;
     end;
    SB_THUMBPOSITION, SB_THUMBTRACK: Position := Message.Pos;
    SB_TOP: Position := Min;
  end;
  Message.Result := 0;
end;

procedure TmbTBXScrollbarTrack.DoVScroll(var Message: TWMScroll);
var
  iPosition: Integer;
begin
  case Message.ScrollCode of
    SB_BOTTOM: Position := Max;
    SB_LINEUP:
     begin
      iPosition := Position;
      Dec(iPosition,SmallChange);
      Position := iPosition;
     end;
    SB_LINEDOWN:
     begin
      iPosition := Position;
      Inc(iPosition,SmallChange);
      Position := iPosition;
     end;
    SB_PAGEUP:
     begin
      iPosition := Position;
      Dec(iPosition,LargeChange);
      Position := iPosition;
     end;
    SB_PAGEDOWN:
     begin
      iPosition := Position;
      Inc(iPosition,LargeChange);
      Position := iPosition;
     end;
    SB_THUMBPOSITION, SB_THUMBTRACK: Position := Message.Pos;
    SB_TOP: Position := Min;
  end;
  Message.Result := 0;
end;

procedure TmbTBXScrollbarTrack.DoEnableArrows(var Message: TMessage);
begin
  if Message.WParam = ESB_DISABLE_BOTH then
  begin
    TmbTBXScrollBar(Parent).EnableBtnOne(False);
    TmbTBXScrollBar(Parent).EnableBtnTwo(False);
  end;
  if Message.WParam = ESB_DISABLE_DOWN then
   if FKind = sbVertical then TmbTBXScrollBar(Parent).EnableBtnTwo(False);
  if Message.WParam = ESB_DISABLE_LTUP then TmbTBXScrollBar(Parent).EnableBtnOne(False);
  if Message.WParam = ESB_DISABLE_LEFT then
   if FKind = sbHorizontal then TmbTBXScrollBar(Parent).EnableBtnOne(False);
  if Message.WParam = ESB_DISABLE_RTDN then TmbTBXScrollBar(Parent).EnableBtnTwo(False);
  if Message.WParam = ESB_DISABLE_UP then
   if FKind = sbVertical then TmbTBXScrollBar(Parent).EnableBtnOne(False);
  if Message.WParam = ESB_ENABLE_BOTH then
   begin
    TmbTBXScrollBar(Parent).EnableBtnOne(True);
    TmbTBXScrollBar(Parent).EnableBtnTwo(True);
   end;
  Message.Result := 1;
end;

procedure TmbTBXScrollbarTrack.DoGetPos(var Message: TMessage);
begin
  Message.Result := Position;
end;

procedure TmbTBXScrollbarTrack.DoGetRange(var Message: TMessage);
begin
  Message.WParam := Min;
  Message.LParam := Max;
end;

procedure TmbTBXScrollbarTrack.DoSetPos(var Message: TMessage);
begin
  Position := Message.WParam;
end;

procedure TmbTBXScrollbarTrack.DoSetRange(var Message: TMessage);
begin
  Min := Message.WParam;
  Max := Message.LParam;
end;

procedure TmbTBXScrollbarTrack.DoKeyDown(var Message: TWMKeyDown);
var
  iPosition: Integer;
begin
  iPosition := Position;
  case Message.CharCode of
    VK_PRIOR: Dec(iPosition,LargeChange);
    VK_NEXT: Inc(iPosition,LargeChange);
    VK_UP: if FKind = sbVertical then Dec(iPosition,SmallChange);
    VK_DOWN: if FKind = sbVertical then Inc(iPosition,SmallChange);
    VK_LEFT: if FKind = sbHorizontal then Dec(iPosition,SmallChange);
    VK_RIGHT: if FKind = sbHorizontal then Inc(iPosition,SmallChange);
  end;
  Position := iPosition;
end;

{ TmbTBXScrollbarButton }

constructor TmbTBXScrollbarButton.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
 DoubleBuffered := true;
 ShowFocus := false;
 FInterval := 20;
end;

procedure PaintArrow(Canvas: TCanvas; Rect: TRect; FDirection: TScrollDirection; FStyle: TScrollArrowStyle; AColor: TColor);
var
 R: TRect;
begin
 //this procedure automaticaly centers the arrow, so just give it the whole button rect
 Canvas.Pen.Color := AColor;
 case FStyle of
  asFlat:
   begin
    case FDirection of
     sdUp:
      begin
       R.Left := Rect.Left + ((Rect.Right - Rect.Left) - 10) div 2;
       R.Top := Rect.Top + ((Rect.Bottom - Rect.Top) - 9) div 2;
       R.Right := Rect.Left + 10;
       R.Bottom := Rect.Top + 9;
       Canvas.Polyline([Point(R.Left + 5, R.Top + 2), Point(R.Left + 4, R.Top + 3),
       Point(R.Left + 3, R.Top + 4), Point(R.Left + 2, R.Top + 5),
       Point(R.Left + 3, R.Top + 5), Point(R.Left + 4, R.Top + 5),
       Point(R.Left + 4, R.Top + 6), Point(R.Left + 4, R.Top + 7),
       Point(R.Left + 4, R.Top + 8), Point(R.Left + 5, R.Top + 8),
       Point(R.Left + 6, R.Top + 8), Point(R.Left + 7, R.Top + 8),
       Point(R.Left + 7, R.Top + 7), Point(R.Left + 7, R.Top + 6),
       Point(R.Left + 7, R.Top + 5), Point(R.Left + 8, R.Top + 5),
       Point(R.Left + 9, R.Top + 5), Point(R.Left + 8, R.Top + 4),
       Point(R.Left + 7, R.Top + 3), Point(R.Left + 6, R.Top + 2),
       Point(R.Left + 6, R.Top + 3), Point(R.Left + 6, R.Top + 4),
       Point(R.Left + 7, R.Top + 4), Point(R.Left + 6, R.Top + 5),
       Point(R.Left + 6, R.Top + 6), Point(R.Left + 6, R.Top + 7),
       Point(R.Left + 5, R.Top + 7), Point(R.Left + 5, R.Top + 6),
       Point(R.Left + 5, R.Top + 5), Point(R.Left + 5, R.Top + 4),
       Point(R.Left + 4, R.Top + 4), Point(R.Left + 5, R.Top + 3),
       Point(R.Left + 5, R.Top + 4)]);
      end;
     sdDown:
      begin
       R.Right := Rect.Right - ((Rect.Right - Rect.Left) - 10) div 2;
       R.Bottom := Rect.Bottom - ((Rect.Bottom - Rect.Top) - 9) div 2;
       R.Left := Rect.Right - 10;
       R.Top := Rect.Bottom - 9;
       Canvas.Polyline([Point(R.Right - 5, R.Bottom - 2),
       Point(R.Right - 4, R.Bottom - 3), Point(R.Right - 3, R.Bottom - 4),
       Point(R.Right - 2, R.Bottom - 5), Point(R.Right - 3, R.Bottom - 5),
       Point(R.Right - 4, R.Bottom - 5), Point(R.Right - 4, R.Bottom - 6),
       Point(R.Right - 4, R.Bottom - 7), Point(R.Right - 4, R.Bottom - 8),
       Point(R.Right - 5, R.Bottom - 8), Point(R.Right - 6, R.Bottom - 8),
       Point(R.Right - 7, R.Bottom - 8), Point(R.Right - 7, R.Bottom - 7),
       Point(R.Right - 7, R.Bottom - 6), Point(R.Right - 7, R.Bottom - 5),
       Point(R.Right - 8, R.Bottom - 5), Point(R.Right - 9, R.Bottom - 5),
       Point(R.Right - 8, R.Bottom - 4), Point(R.Right - 7, R.Bottom - 3),
       Point(R.Right - 6, R.Bottom - 2), Point(R.Right - 6, R.Bottom - 3),
       Point(R.Right - 6, R.Bottom - 4), Point(R.Right - 7, R.Bottom - 4),
       Point(R.Right - 6, R.Bottom - 5), Point(R.Right - 6, R.Bottom - 6),
       Point(R.Right - 6, R.Bottom - 7), Point(R.Right - 5, R.Bottom - 7),
       Point(R.Right - 5, R.Bottom - 6), Point(R.Right - 5, R.Bottom - 5),
       Point(R.Right - 5, R.Bottom - 4), Point(R.Right - 4, R.Bottom - 4),
       Point(R.Right - 5, R.Bottom - 3), Point(R.Right - 5, R.Bottom - 4)]);
      end;
     sdLeft:
      begin
       R.Left := Rect.Left + ((Rect.Right - Rect.Left) - 10) div 2;
       R.Top := Rect.Top + ((Rect.Bottom - Rect.Top) - 9) div 2 - 1;
       R.Right := Rect.Left + 10;
       R.Bottom := Rect.Top + 9;
       Canvas.Polyline([Point(R.Left + 2, R.Top + 5),
       Point(R.Left + 3, R.Top + 4), Point(R.Left + 4, R.Top + 3),
       Point(R.Left + 5, R.Top + 2), Point(R.Left + 5, R.Top + 3),
       Point(R.Left + 5, R.Top + 4), Point(R.Left + 6, R.Top + 4),
       Point(R.Left + 7, R.Top + 4), Point(R.Left + 8, R.Top + 4),
       Point(R.Left + 8, R.Top + 5), Point(R.Left + 8, R.Top + 6),
       Point(R.Left + 8, R.Top + 7), Point(R.Left + 7, R.Top + 7),
       Point(R.Left + 6, R.Top + 7), Point(R.Left + 5, R.Top + 7),
       Point(R.Left + 5, R.Top + 8), Point(R.Left + 5, R.Top + 9),
       Point(R.Left + 4, R.Top + 8), Point(R.Left + 3, R.Top + 7),
       Point(R.Left + 2, R.Top + 6), Point(R.Left + 3, R.Top + 6),
       Point(R.Left + 4, R.Top + 6), Point(R.Left + 4, R.Top + 7),
       Point(R.Left + 5, R.Top + 6), Point(R.Left + 6, R.Top + 6),
       Point(R.Left + 7, R.Top + 6), Point(R.Left + 7, R.Top + 5),
       Point(R.Left + 6, R.Top + 5), Point(R.Left + 5, R.Top + 5),
       Point(R.Left + 4, R.Top + 5), Point(R.Left + 4, R.Top + 4),
       Point(R.Left + 3, R.Top + 5), Point(R.Left + 4, R.Top + 5)]);
      end;
     sdRight:
      begin
       R.Right := Rect.Right - ((Rect.Right - Rect.Left) - 10) div 2;
       R.Bottom := Rect.Bottom - ((Rect.Bottom - Rect.Top) - 9) div 2;
       R.Left := Rect.Right - 10;
       R.Top := Rect.Bottom - 9;
       Canvas.Polyline([Point(R.Right - 2, R.Bottom - 5),
       Point(R.Right - 3, R.Bottom - 4), Point(R.Right - 4, R.Bottom - 3),
       Point(R.Right - 5, R.Bottom - 2), Point(R.Right - 5, R.Bottom - 3),
       Point(R.Right - 5, R.Bottom - 4), Point(R.Right - 6, R.Bottom - 4),
       Point(R.Right - 7, R.Bottom - 4), Point(R.Right - 8, R.Bottom - 4),
       Point(R.Right - 8, R.Bottom - 5), Point(R.Right - 8, R.Bottom - 6),
       Point(R.Right - 8, R.Bottom - 7), Point(R.Right - 7, R.Bottom - 7),
       Point(R.Right - 6, R.Bottom - 7), Point(R.Right - 5, R.Bottom - 7),
       Point(R.Right - 5, R.Bottom - 8), Point(R.Right - 5, R.Bottom - 9),
       Point(R.Right - 4, R.Bottom - 8), Point(R.Right - 3, R.Bottom - 7),
       Point(R.Right - 2, R.Bottom - 6), Point(R.Right - 3, R.Bottom - 6),
       Point(R.Right - 4, R.Bottom - 6), Point(R.Right - 4, R.Bottom - 7),
       Point(R.Right - 5, R.Bottom - 6), Point(R.Right - 6, R.Bottom - 6),
       Point(R.Right - 7, R.Bottom - 6), Point(R.Right - 7, R.Bottom - 5),
       Point(R.Right - 6, R.Bottom - 5), Point(R.Right - 5, R.Bottom - 5),
       Point(R.Right - 4, R.Bottom - 5), Point(R.Right - 4, R.Bottom - 4),
       Point(R.Right - 3, R.Bottom - 5), Point(R.Right - 4, R.Bottom - 5)]);
      end;
    end;
   end;
  asXPStyle:
   begin
    case FDirection of
     sdUp:
      begin
       R.Left := Rect.Left + ((Rect.Right - Rect.Left) - 11) div 2;
       R.Top := Rect.Top + ((Rect.Bottom - Rect.Top) - 8) div 2;
       R.Right := Rect.Left + 11;
       R.Bottom := Rect.Top + 8;
       Canvas.Polyline([Point(R.Left + 6, R.Top + 2), Point(R.Left + 5, R.Top + 3),
       Point(R.Left + 4, R.Top + 4), Point(R.Left + 3, R.Top + 5),
       Point(R.Left + 2, R.Top + 6), Point(R.Left + 3, R.Top + 7),
       Point(R.Left + 4, R.Top + 6), Point(R.Left + 5, R.Top + 5),
       Point(R.Left + 6, R.Top + 4), Point(R.Left + 7, R.Top + 5),
       Point(R.Left + 8, R.Top + 6), Point(R.Left + 9, R.Top + 7),
       Point(R.Left + 10, R.Top + 6), Point(R.Left + 9, R.Top + 5),
       Point(R.Left + 8, R.Top + 4), Point(R.Left + 7, R.Top + 3),
       Point(R.Left + 9, R.Top + 6), Point(R.Left + 8, R.Top + 5),
       Point(R.Left + 7, R.Top + 4), Point(R.Left + 6, R.Top + 3),
       Point(R.Left + 5, R.Top + 4), Point(R.Left + 4, R.Top + 5),
       Point(R.Left + 3, R.Top + 6), Point(R.Left + 4, R.Top + 5)]);
      end;
     sdDown:
      begin
       R.Right := Rect.Right - ((Rect.Right - Rect.Left) - 11) div 2;
       R.Bottom := Rect.Bottom - ((Rect.Bottom - Rect.Top) - 8) div 2;
       R.Left := Rect.Right - 11;
       R.Top := Rect.Bottom - 8;
       Canvas.Polyline([Point(R.Right - 6, R.Bottom - 2), Point(R.Right - 5, R.Bottom - 3),
       Point(R.Right - 4, R.Bottom - 4), Point(R.Right - 3, R.Bottom - 5),
       Point(R.Right - 2, R.Bottom - 6), Point(R.Right - 3, R.Bottom - 7),
       Point(R.Right - 4, R.Bottom - 6), Point(R.Right - 5, R.Bottom - 5),
       Point(R.Right - 6, R.Bottom - 4), Point(R.Right - 7, R.Bottom - 5),
       Point(R.Right - 8, R.Bottom - 6), Point(R.Right - 9, R.Bottom - 7),
       Point(R.Right - 10, R.Bottom - 6), Point(R.Right - 9, R.Bottom - 5),
       Point(R.Right - 8, R.Bottom - 4), Point(R.Right - 7, R.Bottom - 3),
       Point(R.Right - 9, R.Bottom - 6), Point(R.Right - 8, R.Bottom - 5),
       Point(R.Right - 7, R.Bottom - 4), Point(R.Right - 6, R.Bottom - 3),
       Point(R.Right - 5, R.Bottom - 4), Point(R.Right - 4, R.Bottom - 5),
       Point(R.Right - 3, R.Bottom - 6), Point(R.Right - 4, R.Bottom - 5)]);
      end;
     sdLeft:
      begin
       R.Left := Rect.Left + ((Rect.Right - Rect.Left) - 11) div 2;
       R.Top := Rect.Top + ((Rect.Bottom - Rect.Top) - 8) div 2 - 2;
       R.Right := Rect.Left + 11;
       R.Bottom := Rect.Top + 8;
       Canvas.Polyline([Point(R.Left + 2, R.Top + 6), Point(R.Left + 3, R.Top + 5),
       Point(R.Left + 4, R.Top + 4), Point(R.Left + 5, R.Top + 3),
       Point(R.Left + 6, R.Top + 2), Point(R.Left + 7, R.Top + 3),
       Point(R.Left + 6, R.Top + 4), Point(R.Left + 5, R.Top + 5),
       Point(R.Left + 4, R.Top + 6), Point(R.Left + 5, R.Top + 7),
       Point(R.Left + 6, R.Top + 8), Point(R.Left + 7, R.Top + 9),
       Point(R.Left + 6, R.Top + 10), Point(R.Left + 5, R.Top + 9),
       Point(R.Left + 4, R.Top + 8), Point(R.Left + 3, R.Top + 7),
       Point(R.Left + 6, R.Top + 9), Point(R.Left + 5, R.Top + 8),
       Point(R.Left + 4, R.Top + 7), Point(R.Left + 3, R.Top + 6),
       Point(R.Left + 4, R.Top + 5), Point(R.Left + 5, R.Top + 4),
       Point(R.Left + 6, R.Top + 3), Point(R.Left + 5, R.Top + 4)]);
      end;
     sdRight:
      begin
       R.Right := Rect.Right - ((Rect.Right - Rect.Left) - 11) div 2;
       R.Bottom := Rect.Bottom - ((Rect.Bottom - Rect.Top) - 9) div 2;
       R.Left := Rect.Right - 11;
       R.Top := Rect.Bottom - 9;
       Canvas.Polyline([Point(R.Right - 2, R.Bottom - 6), Point(R.Right - 3, R.Bottom - 5),
       Point(R.Right - 4, R.Bottom - 4), Point(R.Right - 5, R.Bottom - 3),
       Point(R.Right - 6, R.Bottom - 2), Point(R.Right - 7, R.Bottom - 3),
       Point(R.Right - 6, R.Bottom - 4), Point(R.Right - 5, R.Bottom - 5),
       Point(R.Right - 4, R.Bottom - 6), Point(R.Right - 5, R.Bottom - 7),
       Point(R.Right - 6, R.Bottom - 8), Point(R.Right - 7, R.Bottom - 9),
       Point(R.Right - 6, R.Bottom - 10), Point(R.Right - 5, R.Bottom - 9),
       Point(R.Right - 4, R.Bottom - 8), Point(R.Right - 3, R.Bottom - 7),
       Point(R.Right - 6, R.Bottom - 9), Point(R.Right - 5, R.Bottom - 8),
       Point(R.Right - 4, R.Bottom - 7), Point(R.Right - 3, R.Bottom - 6),
       Point(R.Right - 4, R.Bottom - 5), Point(R.Right - 5, R.Bottom - 4),
       Point(R.Right - 6, R.Bottom - 3), Point(R.Right - 5, R.Bottom - 4)]);
      end;
    end;
   end;
 end;
end;

procedure TmbTBXScrollBarButton.WMPaint(var Msg: TWMPaint);
var
 DC: HDC;
 ii: TTBXItemInfo;
 R: TRect;
 c: TCanvas;
begin
 inherited;
 // this is inherited code - I have to use it here because of vertical scrollbars
  C := TCanvas.Create;
  DC := GetWindowDC(Handle);
  GetWindowRect(Handle, R);
  OffsetRect(R, -R.Left, -R.Top);
  C.Handle := DC;
  if (csDesigning in ComponentState) or (Enabled = false) then FMouseLoc := mlNone;
  case FMouseLoc of
   mlNone: SetItemInfo('normal', ii);
   mlOver: SetItemInfo('over', ii);
   mlDown: SetItemInfo('down', ii);
  end;
  ii.Enabled := Enabled;
  ii.IsVertical := (FKind = sbVertical);
  //paint image
  if FImages <> nil then
   begin
    with R do
     begin
      R.Top := R.Top + ((R.Bottom - R.Top) - FImages.Height) div 2;
      R.Bottom := R.Top + FImages.Height;
      R.Left := R.Left + ((R.Right - R.Left) - FImages.Width) div 2;
      R.Right := R.Left + FImages.Width;
     end;
    CurrentTheme.PaintImage(c, R, ii, FImages, FImageIndex);
   end
  else
   PaintArrow(c, R, FDirection, FStyle, CurrentTheme.GetItemTextColor(ii));
  //Clean up
  C.Handle := 0;
  ReleaseDC(Handle, DC);
  c.free;
end;

procedure TmbTBXScrollBarButton.SetImageIndex(i: TImageIndex);
begin
 FImageIndex := i;
 Invalidate;
end;

procedure TmbTBXScrollbarButton.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited MouseDown(Button,Shift,X,Y);
  FNewDown := True;
  FTimer := TTimer.Create(Self);
  FTimer.Interval := FInterval;
  FTimer.OnTimer := DoTimer;
  FTimer.Enabled := True;
end;

procedure TmbTBXScrollbarButton.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
  inherited MouseMove(Shift,X,Y);
end;

procedure TmbTBXScrollbarButton.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited MouseUp(Button,Shift,X,Y);
  FNewDown := False;
  FTimer.Enabled := False;
  FTimer.Free;
end;

procedure TmbTBXScrollbarButton.DoTimer(Sender: TObject);
begin
  if FNewDown = True then
  begin
    if Assigned(FOnDown) then FOnDown(Self);
     TmbTBXScrollBar(Parent).DoScroll;
  end;
end;

{ TmbTBXScrollBar }

constructor TmbTBXScrollBar.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
 AddThemeNotification(self);
 DoubleBuffered := true;
 FBtnSize := 18;
 Width := 150;
 Height := 18;
 FStyle := asXPStyle;
 //create & set Button1
 FBtnOne := TmbTBXScrollbarButton.Create(Self);
 FBtnOne.FKind := FKind;
 FBtnOne.FImages := FImages;
 FBtnOne.SetImageIndex(0);
 FBtnOne.OnDown := BtnOneClick;
 FBtnOne.FStyle := asXPStyle;
 InsertControl(FBtnOne);
 //create & set Button2
 FBtnTwo := TmbTBXScrollbarButton.Create(Self);
 FBtnTwo.FKind := FKind;
 FBtnTwo.FImages := FImages;
 FBtnTwo.SetImageIndex(1);
 FBtnTwo.OnDown := BtnTwoClick;
 FBtnTwo.FStyle := asXPStyle;
 InsertControl(FBtnTwo);
 //create & set the track
 FTrack := TmbTBXScrollbarTrack.Create(Self);
 FTrack.FImages := FImages;
 FTrack.SetBounds(0,0,Width,Height);
 FTrack.Kind := sbVertical;
 FTrack.OnScroll := TrackScroll;
 InsertControl(FTrack);
 //set other params
 Kind := sbVertical;
 Min := 0;
 Max := 100;
 Position := 0;
 SmallChange := 1;
 LargeChange := 10;
 FImageChangeLink := TChangeLink.Create;
 FImageChangeLink.OnChange := ImageListChange;
 FHotTrack := true;
 FExtraFrame := false;
 FPageSize := 1;
 case FKind of
  sbHorizontal:
   begin
    FBtnOne.Width := FBtnSize;
    FBtnTwo.Width := FBtnSize;
   end;
  sbVertical:
   begin
    FBtnOne.Height := FBtnSize;
    FBtnTwo.Height := FBtnSize;
   end;
 end;
 FSInterval := 20;
end;

destructor TmbTBXScrollBar.Destroy;
begin
  FImageChangeLink.Free;
  FTrack.Free;
  FBtnOne.Free;
  FBtnTwo.Free;
  RemoveThemeNotification(Self);
  inherited Destroy;
end;

procedure TmbTBXScrollBar.TBMThemeChange(var Message: TMessage);
begin
if Message.WParam = TSC_VIEWCHANGE then
 Invalidate;
end;

procedure TmbTBXScrollBar.SetBtnSize(Value: Integer);
begin
 if Value <> FBtnSize then
  begin
   FBtnSize := Value;
   case FKind of
    sbHorizontal:
     begin
      FBtnOne.Width := FBtnSize;
      FBtnTwo.Width := FBtnSize;
     end;
    sbVertical:
     begin
      FBtnOne.Height := FBtnSize;
      FBtnTwo.Height := FBtnSize;
     end;
   end;
   FTrack.SetBtnSize(Value);
   UpdateSize;
  end;
end;

procedure TmbTBXScrollBar.UpdateSize;
begin
  if FKind = sbVertical then
  begin
    SetBounds(Left, Top, Width, Height);
    FBtnOne.SetBounds(0, 0, Width, FBtnSize);
    FBtnTwo.SetBounds(0, Height - FBtnSize, Width, FBtnSize);
    FTrack.SetBounds(0, FBtnSize, Width, Height - 2*FBtnSize);
  end
  else
  begin
    SetBounds(Left, Top, Width, Height);
    FBtnOne.SetBounds(0, 0, FBtnSize, Height);
    FBtnTwo.SetBounds(Width - FBtnSize, 0, FBtnSize, Height);
    FTrack.SetBounds(FBtnSize, 0, Width - 2*FBtnSize, Height);
  end;
  Position := FPosition;
  UpdatePageSize;
  UpdateThumbSize;
end;

procedure TmbTBXScrollBar.SetSmallChange(Value: Integer);
begin
 if Value <> FSmallChange then
  begin
   if Value < 1 then Value := 1;
   FSmallChange := Value;
   FTrack.SmallChange := FSmallChange;
  end;
end;

procedure TmbTBXScrollBar.SetLargeChange(Value: Integer);
begin
 if Value <> FLargeChange then
  begin
   if Value < 1 then Value := 1;
   FLargeChange := Value;
   FTrack.LargeChange := FLargeChange;
  end;
end;

procedure TmbTBXScrollBar.SetMin(Value: Integer);
begin
  if (Value <> FMin) and (Value > -1) then
  begin
    FMin := Value;
    FTrack.Min := FMin;
  end;
end;

procedure TmbTBXScrollBar.SetMax(Value: Integer);
begin
  if (Value <> FMax) and (Value > -1) then
  begin
    FMax := Value;
    FTrack.Max := FMax;
  end;
end;

procedure TmbTBXScrollBar.SetPosition(Value: Integer);
begin
  FPosition := Value;
  if Position < Min then Position := Min;
  if Position > Max then Position := Max;
  FTrack.Position := FPosition;
end;

procedure TmbTBXScrollBar.SetKind(Value: TScrollBarKind);
var
  i: Integer;
begin
  if FKind <> Value then
   begin
    FKind := Value;
    FTrack.Kind := FKind;
    FBtnOne.SetImageIndex(0);
    FBtnTwo.SetImageIndex(1);
    case Value of
     sbVertical:
      begin
       FBtnOne.FDirection := sdUp;
       FBtnTwo.FDirection := sdDown;
      end;
     sbHorizontal:
      begin
       FBtnOne.FDirection := sdLeft;
       FBtnTwo.FDirection := sdRight;
      end;
    end;
    if (csDesigning in ComponentState) and not (csLoading in ComponentState) then
     begin
      i := Width;
      Width := Height;
      Height := i;
     end;
   end;
end;

procedure TmbTBXScrollBar.BtnOneClick(Sender: TObject);
var
  iPosition: Integer;
begin
  iPosition := Position;
  Dec(iPosition,SmallChange);
  Position := iPosition;
end;

procedure TmbTBXScrollBar.BtnTwoClick(Sender: TObject);
var
  iPosition: Integer;
begin
  iPosition := Position;
  Inc(iPosition,SmallChange);
  Position := iPosition;
end;

procedure TmbTBXScrollBar.EnableBtnOne(Value: Boolean);
begin
 FBtnOne.Enabled := Value;
 Invalidate;
end;

procedure TmbTBXScrollBar.EnableBtnTwo(Value: Boolean);
begin
 FBtnTwo.Enabled := Value;
 Invalidate;
end;

procedure TmbTBXScrollBar.WMSize(var Message: TWMSize);
begin
  if FKind = sbVertical then
  begin
    SetBounds(Left, Top, Width, Height);
    FBtnOne.SetBounds(0, 0, Width, FBtnSize);
    FBtnTwo.SetBounds(0, Height - FBtnSize, Width, FBtnSize);
    FTrack.SetBounds(0, FBtnSize, Width, Height - 2*FBtnSize);
  end
  else
  begin
    SetBounds(Left, Top, Width, Height);
    FBtnOne.SetBounds(0, 0, FBtnSize, Height);
    FBtnTwo.SetBounds(Width - FBtnSize, 0, FBtnSize, Height);
    FTrack.SetBounds(FBtnSize, 0, Width - 2*FBtnSize, Height);
  end;
  Position := FPosition;
  UpdatePageSize;
  UpdateThumbSize;
end;

procedure TmbTBXScrollBar.DoScroll;
begin
  if Assigned(FOnScroll) then FOnScroll(Self,Position);
end;

procedure TmbTBXScrollBar.CNHScroll(var Message: TWMScroll);
begin
  FTrack.DoHScroll(Message);
end;

procedure TmbTBXScrollBar.CNVScroll(var Message: TWMScroll);
begin
  FTrack.DoVScroll(Message);
end;

procedure TmbTBXScrollBar.SBMEnableArrows(var Message: TMessage);
begin
  FTrack.DoEnableArrows(Message);
end;

procedure TmbTBXScrollBar.SBMGetPos(var Message: TMessage);
begin
  FTrack.DoGetPos(Message);
end;

procedure TmbTBXScrollBar.SBMGetRange(var Message: TMessage);
begin
  FTrack.DoGetRange(Message);
end;

procedure TmbTBXScrollBar.SBMSetPos(var Message: TMessage);
begin
  FTrack.DoSetPos(Message);
end;

procedure TmbTBXScrollBar.SBMSetRange(var Message: TMessage);
begin
  FTrack.DoSetRange(Message);
end;

procedure TmbTBXScrollBar.WMKeyDown(var Message: TWMKeyDown);
begin
  FTrack.DoKeyDown(Message);
end;

procedure TmbTBXScrollBar.Notification(AComponent: TComponent; Operation: TOperation);
begin
 inherited Notification(AComponent, Operation);
 if Operation = opRemove then
  if AComponent = Images then
   SetImages(nil);
end;

procedure TmbTBXScrollBar.SetImages(i: TCustomImageList);
begin
 if FImages <> nil then
  FImages.UnRegisterChanges(FImageChangeLink);
 FImages := i;
 FTrack.FThumb.FImages := i;
 FTrack.FThumb.Invalidate;
 FBtnOne.FImages := i;
 FBtnOne.Invalidate;
 FBtnTwo.FImages := i;
 FBtnTwo.Invalidate;
 if FImages <> nil then
  begin
   FImages.RegisterChanges(FImageChangeLink);
   FImages.FreeNotification(Self);
  end;
 Invalidate;
end;

procedure TmbTBXScrollBar.ImageListChange(Sender: TObject);
begin
 if Sender = Images then
  Invalidate;
end;

procedure TmbTBXScrollBar.SetStyle(s: TScrollArrowStyle);
begin
 FStyle := s;
 FBtnOne.FStyle := s;
 FBtnTwo.FStyle := s;
 invalidate;
end;

procedure TmbTBXScrollBar.CMEnabledChanged(var Message: TMessage);
begin
 inherited;
 Invalidate;
end;

procedure TmbTBXScrollBar.SetHotTrack(Value: boolean);
begin
 if FHotTrack <> Value then
  begin
   FHotTrack := Value;
   FBtnOne.HotTracking := Value;
   FBtnTwo.HotTracking := Value;
   FTrack.FThumb.HotTracking := Value;
  end;
end;

procedure TmbTBXScrollBar.SetExtraFrame(Value: boolean);
begin
 if FExtraFrame <> Value then
  begin
   FExtraFrame := Value;
   FBtnOne.ExtraFrame := Value;
   FBtnTwo.ExtraFrame := Value;
   FTrack.FThumb.ExtraFrame := Value;
  end;
end;

procedure TmbTBXScrollBar.Center;
begin
  SetPosition((FMin + FMax - FPageSize) div 2);
end;

procedure TmbTBXScrollBar.UpdatePageSize;
begin
 if FPageSize > (FMax - FMin) then
  SetPageSize(FMax - FMin);
end;

procedure TmbTBXScrollBar.SetPageSize(Value: integer);
begin
 if Value < 1 then
  Value := 1;
 if Value > (FMax - FMin) then
  Value := FMax - FMin;
 if FPageSize <> Value then
  begin
   FPageSize := Value;
   FTrack.SetPageSize(Value);
   UpdatePageSize;
   UpdateThumbSize;
  end;
end;

procedure TmbTBXScrollBar.UpdateThumbSize;
begin
 FTrack.UpdateThumbSize;
end;

procedure TmbTBXScrollBar.SetSInterval(Value: integer);
begin
 if FSInterval <> Value then
  begin
   FSInterval := Value;
   FBtnOne.FInterval := Value;
   FBtnTwo.FInterval := Value;
   FTrack.FTimer.Interval := Value;
  end;
end;

procedure TmbTBXScrollBar.TrackScroll;
begin
 if Assigned(FOnScroll) then FOnScroll(Self, Position);
end;

end.
