unit mbTBXScrollBox;

{$I mxs.inc}

interface

uses
  SysUtils, Classes, Controls, StdCtrls, Windows, Messages, Graphics, TBX,
  TBXUtils, TBXThemes, Forms;

type
  TmbTBXScrollBox = class(TScrollingWinControl)
  protected
   FAfterPaint: TNotifyEvent;

   procedure WMNCHitTest(var Message: TMessage); message WM_NCHITTEST;
   procedure WMHScroll(var Msg: TWMHScroll); message WM_HSCROLL;
   procedure WMVScroll(var Msg: TWMVScroll); message WM_VSCROLL;
   procedure WMPaint(var Msg: TWMPaint); message WM_PAINT;
   procedure TBMThemeChange(var Message: TMessage); message TBM_THEMECHANGE;
   procedure Resize; override;
   procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
   property OnAfterPaint: TNotifyEvent read FAfterPaint write FAfterPaint;
  public
   constructor Create(AOwner: TComponent); override;
   destructor Destroy; override;
  published
   property Align;
   property Anchors;
   property AutoScroll;
   property AutoSize;
   property BiDiMode;
   property Color;
   property Constraints;
   property DockSite;
   property DragCursor;
   property DragKind;
   property DragMode;
   property Enabled;
   property Font;
   property ParentColor;
   property ParentBiDiMode;
   {$IFDEF DELPHI_7_UP}
   property ParentBackground default False;
   {$ENDIF}
   property ParentFont;
   property ParentShowHint;
   property PopupMenu;
   property ShowHint;
   property TabOrder;
   property TabStop;
   property Visible;

   property OnCanResize;
   property OnClick;
   property OnConstrainedResize;
   property OnContextPopup;
   property OnDblClick;
   property OnDockDrop;
   property OnDockOver;
   property OnDragDrop;
   property OnDragOver;
   property OnEndDock;
   property OnEndDrag;
   property OnEnter;
   property OnExit;
   property OnGetSiteInfo;
   property OnMouseDown;
   property OnMouseMove;
   property OnMouseUp;
   property OnMouseWheel;
   property OnMouseWheelDown;
   property OnMouseWheelUp;
   property OnResize;
   property OnStartDock;
   property OnStartDrag;
   property OnUnDock;
  end;
  
implementation

constructor TmbTBXScrollBox.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
 AddThemeNotification(Self);
 DoubleBuffered := true;
 ControlStyle := [csAcceptsControls, csCaptureMouse, csClickEvents, csSetCaption, csDoubleClicks];
 Width := 50;
 Height := 50;
end;

destructor TmbTBXScrollBox.Destroy;
begin
 inherited Destroy;
 RemoveThemeNotification(Self);
end;

procedure TmbTBXScrollBox.TBMThemeChange(var Message: TMessage);
begin
if Message.WParam = TSC_VIEWCHANGE then
 Invalidate;
end;

procedure SetItemInfo(var IInfo: TTBXItemInfo);
begin
   FillChar(IInfo, SizeOf(TTBXItemInfo), 0);
   IInfo.ViewType := TVT_NORMALTOOLBAR;
   IInfo.ItemOptions := IO_APPACTIVE or IO_TOOLBARSTYLE;
   IInfo.Enabled := True;
   IInfo.Pushed := false;
   IInfo.Selected := false;
   IInfo.HoverKind := hkNone;
end;

procedure TmbTBXScrollBox.WMPaint(var Msg: TWMPaint);
var
 DC: HDC;
 ii: TTBXItemInfo;
 R: TRect;
 C: TCanvas;
begin
 inherited;
 C := TCanvas.Create;
 DC := GetWindowDC(Handle);
 GetWindowRect(Handle, R);
 OffsetRect(R, -R.Left, -R.Top);
 C.Handle := DC;
 ExcludeClipRect(DC, 2, 2, R.Right - 1, R.Bottom - 1); // Important
 // set info
 SetItemInfo(ii);
 ii.Enabled := Enabled;
 // paint the border
 CurrentTheme.PaintFrame(c, R, ii);
 CurrentTheme.PaintBackgnd(c, R, R, R, CurrentTheme.GetViewColor(TVT_NORMALTOOLBAR), false, TVT_NORMALTOOLBAR);
 CurrentTheme.PaintDock(c, R, R, DP_TOP);
 if CurrentTheme.PaintDockBackground then
  CurrentTheme.PaintBackgnd(c, R, R, R, clBtnShadow{CurrentTheme.GetViewColor(TVT_NORMALTOOLBAR)}, false, TVT_NORMALTOOLBAR)
 else
  CurrentTheme.PaintDock(c, R, R, DP_TOP);
 CurrentTheme.PaintButton(c, R, ii);
  //hide double border
 C.Brush.Color := Color;
 InflateRect(R, -1, -1);
 C.FillRect(R);
 // clean up
 C.Handle := 0;
 ReleaseDC(Handle, DC);
 C.free;
 // do after paint
 if Assigned(FAfterPaint) then
  FAfterPaint(self);
end;

procedure TmbTBXScrollBox.Resize;
begin
 Invalidate;
 if Assigned(OnResize) then
  OnResize(Self);
end;

procedure TmbTBXScrollBox.CMEnabledChanged(var Message: TMessage);
begin
 inherited;
 Invalidate;
end;

procedure TmbTBXScrollBox.WMNCHitTest(var Message: TMessage);
begin
 DefaultHandler(Message);
end;

procedure TmbTBXScrollBox.WMHScroll(var Msg: TWMHScroll);
begin
 inherited;
 Invalidate;
end;

procedure TmbTBXScrollBox.WMVScroll(var Msg: TWMVScroll);
begin
 inherited;
 Invalidate;
end;

end.
