unit mbTBXShellTreeView;

{$I mxs.inc}

interface

uses
  SysUtils, Classes, Controls, ShellCtrls, Windows, Messages, Graphics, TBX,
  TBXUtils, TBXThemes;

type
  TMouseLoc = (mlNone, mlOver, mlDown);

  TmbTBXCustomShellTreeView = class(TCustomShellTreeView)
  private
   c: TCanvas;
   FMouseOver: boolean;
   FThemed: boolean;
   FHotTrack: boolean;

   procedure SetHotTrack(Value: boolean);
   procedure SetThemed(Value: boolean);
  protected
   FMouseLoc: TMouseLoc;
   FAfterPaint: TNotifyEvent;

   procedure WMNCPaint (var Msg: TWMNCPaint); message WM_NCPAINT;
   procedure TBMThemeChange(var Message: TMessage); message TBM_THEMECHANGE;
   procedure Resize; override;
   procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
   procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
   procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
   procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
   procedure CMGotFocus(var Message: TCMGotFocus); message CM_ENTER;
   procedure CMLostFocus(var Message: TCMLostFocus); message CM_EXIT;
   procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
   procedure RPaint;
   property OnAfterPaint: TNotifyEvent read FAfterPaint write FAfterPaint;
  public
   constructor Create(AOwner: TComponent); override;
   destructor Destroy; override;

   property Canvas: TCanvas read c write c;
   property Themed: boolean read FThemed write SetThemed default true;
   property HotTracking: boolean read FHotTrack write SetHotTrack default true;
  end;

 TmbTBXShellTreeView = class(TmbTBXCustomShellTreeView)
  published
   property AutoContextMenus;
   property ObjectTypes;
   property Root;
   property ShellComboBox;
   property ShellListView;
   property UseShellImages;
   property OnAddFolder;
   property Align;
   property Anchors;
   property AutoRefresh;
   property ChangeDelay;
   property Color;
   property Cursor;
   property DragCursor;
   property DragMode;
   property Enabled;
   property Font;
   property HideSelection;
   property Images;
   property Indent;
   property ParentFont;
   property ParentColor default false;
   property ParentShowHint;
   property PopupMenu;
   property RightClickSelect;
   property ShowButtons;
   property ShowHint;
   property ShowLines;
   property ShowRoot;
   property StateImages;
   property TabOrder;
   property TabStop default True;
   property Visible;
   property Themed;
   property HotTracking;

   property OnClick;
   property OnEnter;
   property OnExit;
   property OnDragDrop;
   property OnDragOver;
   property OnStartDrag;
   property OnEndDrag;
   property OnMouseDown;
   property OnMouseMove;
   property OnMouseUp;
   property OnDblClick;
   property OnKeyDown;
   property OnKeyPress;
   property OnKeyUp;
   property OnChanging;
   property OnChange;
   property OnExpanding;
   property OnCollapsing;
   property OnCollapsed;
   property OnExpanded;
   property OnEditing;
   property OnEdited;
   property OnGetImageIndex;
   property OnGetSelectedIndex;
  end;

implementation

constructor TmbTBXCustomShellTreeView.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
ParentColor := false;
 AddThemeNotification(Self);
 c := TCanvas.Create;
 FMouseLoc := mlNone;
 FMouseOver := false;
 DoubleBuffered := true;
 FThemed := true;
 FHotTrack := true;
end;

destructor TmbTBXCustomShellTreeView.Destroy;
begin
 FreeAndNil(c);
 RemoveThemeNotification(Self);
 inherited Destroy;
end;

procedure TmbTBXCustomShellTreeView.TBMThemeChange(var Message: TMessage);
begin
if Message.WParam = TSC_VIEWCHANGE then
 RPaint;
end;

procedure TmbTBXCustomShellTreeView.RPaint;
var
 r: TRect;
begin
 r := ClientRect;
 RedrawWindow(Handle, @R, 0, RDW_INVALIDATE or RDW_UPDATENOW or RDW_FRAME);
end;

procedure SetItemInfo(state: string; var IInfo: TTBXItemInfo; var EInfo: TTBXEditInfo);
begin
   FillChar(IInfo, SizeOf(TTBXItemInfo), 0);
   IInfo.ViewType := TVT_NORMALTOOLBAR;
   IInfo.ItemOptions := IO_APPACTIVE or IO_TOOLBARSTYLE;
   IInfo.Enabled := True;

   FillChar(EInfo, SizeOf(TTBXEditInfo), 0);
   EInfo.RightBtnWidth := CurrentTheme.GetIntegerMetrics(TMI_EDIT_BTNWIDTH);
   EInfo.RightBtnInfo.ButtonType := EBT_DROPDOWN;

   if state = 'normal' then
   begin
     IInfo.Pushed := false;
     IInfo.Selected := false;
     IInfo.HoverKind := hkNone;
     EInfo.RightBtnInfo.ButtonState := 0;
   end;
   if state = 'over' then
   begin
     IInfo.Pushed := false;
     IInfo.Selected := false;
     IInfo.HoverKind := hkMouseHover;
     EInfo.RightBtnInfo.ButtonState := EBDS_HOT;
   end;
   if state = 'down' then
   begin
     IInfo.Pushed := true;
     IInfo.Selected := false;
     IInfo.HoverKind := hkMouseHover;
     EInfo.RightBtnInfo.ButtonState := EBDS_PRESSED;
   end;
end;

procedure TmbTBXCustomShellTreeView.WMNCPaint(var Msg: TWMNCPaint);
var
 DC: HDC;
 ii: TTBXItemInfo;
 ed: TTBXEditInfo;
 R: TRect;
begin
 inherited;
 if not FThemed then Exit;
 DC := GetWindowDC(Handle);
 GetWindowRect(Handle, R);
 OffsetRect(R, -R.Left, -R.Top);
 C.Handle := DC;
 ExcludeClipRect(DC, 2, 2, R.Right - 1, R.Bottom - 1); // Important
 // set info
 case FMouseLoc of
  mlNone: SetItemInfo('normal', ii, ed);
  mlOver:
   if FHotTrack then
    SetItemInfo('over', ii, ed)
   else
    SetItemInfo('normal', ii, ed);
  mlDown: SetItemInfo('down', ii, ed);
 end;
 if Focused then SetItemInfo('over', ii, ed);
 if (csDesigning in ComponentState) or (Enabled = false) then SetItemInfo('normal', ii, ed);
 ii.Enabled := Enabled;
 // paint the border
 CurrentTheme.PaintFrame(c, R, ii);
 CurrentTheme.PaintBackgnd(c, R, R, R, CurrentTheme.GetViewColor(TVT_NORMALTOOLBAR), false, TVT_NORMALTOOLBAR);
 CurrentTheme.PaintDock(c, R, R, DP_TOP);
 if CurrentTheme.PaintDockBackground then
  CurrentTheme.PaintBackgnd(c, R, R, R, CurrentTheme.GetViewColor(TVT_NORMALTOOLBAR), false, TVT_NORMALTOOLBAR)
 else
  CurrentTheme.PaintDock(c, R, R, DP_TOP);
 CurrentTheme.PaintButton(c, R, ii);
  //hide double border
 C.Brush.Color := Color;
 InflateRect(R, -1, -1);
 C.FillRect(R);
 // clean up
 C.Handle := 0;
 ReleaseDC(Handle, DC);
 // do after paint
 if Assigned(FAfterPaint) then
  FAfterPaint(self);
end;

procedure TmbTBXCustomShellTreeView.Resize;
begin
 inherited;
 RPaint;
 if Assigned(OnResize) then
  OnResize(Self);
end;

procedure TmbTBXCustomShellTreeView.CMMouseEnter(var Message: TMessage);
begin
  if not Focused then
   begin
    FMouseOver := true;
    FMouseLoc := mlOver;
    if FHotTrack then
    RPaint;
   end;
  inherited;
end;

procedure TmbTBXCustomShellTreeView.CMMouseLeave(var Message: TMessage);
begin
 if not Focused then
  begin
   FMouseOver := false;
   FMouseLoc := mlNone;
   if FHotTrack then
   RPaint;
  end;
 inherited;
end;

procedure TmbTBXCustomShellTreeView.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 begin
  FMouseLoc := mlDown;
  RPaint;
 end;
 if Assigned(OnMouseDown) then
  OnMouseDown(Self, Button, Shift, x, y);
 inherited;
end;

procedure TmbTBXCustomShellTreeView.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 if not Focused then
  begin
   if FMouseOver then
    FMouseLoc := mlOver
   else
    FMouseLoc := mlNone;
   RPaint;
  end;
 if Assigned(OnMouseUp) then
  OnMouseUp(Self, Button, Shift, x, y);
 inherited;
end;

procedure TmbTBXCustomShellTreeView.CMGotFocus(var Message: TCMGotFocus);
begin
 inherited;
 FMouseLoc := mlOver;
 RPaint;
end;

procedure TmbTBXCustomShellTreeView.CMLostFocus(var Message: TCMLostFocus);
begin
 inherited;
 FMouseLoc := mlNone;
 RPaint;
end;

procedure TmbTBXCustomShellTreeView.CMEnabledChanged(var Message: TMessage);
begin
 inherited;
 RPaint;
end;

procedure TmbTBXCustomShellTreeView.SetThemed(Value: boolean);
begin
 FThemed := Value;
 RPaint;
end;

procedure TmbTBXCustomShellTreeView.SetHotTrack(Value: boolean);
begin
 if FHotTrack <> Value then
  begin
   FHotTrack := Value;
   RPaint;
  end;
end;

end.
