unit mbTBXSpinButton;

{$I mxs.inc}

interface

uses
  SysUtils, Classes, Controls, Windows, Messages, Graphics, TBX, TBXUtils,
  TBXThemes, ExtCtrls;

type
  TMouseLoc = (mlNone, mlOver, mlDown);
  TSpinButState = (ssDown, ssUp, ssNone);

  TmbTBXSpinButton = class(TWinControl)
  private
   FMouseLoc: TMouseLoc;
   FMouseOver, FMouseDown: boolean;
   FSbut: TSpinButState;
   FOnUpClick: TNotifyEvent;
   FOnDownClick: TNotifyEvent;
   FTimer: TTimer;
   FClicked: boolean;
   FHotTrack: boolean;

   procedure SetHotTrack(Value: boolean);
   procedure SetItemInfo(state: string; var IInfo: TTBXItemInfo; var EInfo: TTBXEditInfo);
   procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
   procedure TBMThemeChange(var Message: TMessage); message TBM_THEMECHANGE;
   procedure UpClick;
   procedure DownClick;
   procedure DoTimer(Sender: TObject);
  protected
   procedure WMEraseBkgnd(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
   procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
   procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
   procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
   procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
   procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
   procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
  public
   constructor Create(AOwner: TComponent); override;
   destructor Destroy; override;
  published
   property Align;
   property Anchors;
   property Constraints;
   property DragCursor;
   property DragKind;
   property DragMode;
   property Enabled;
   property ParentShowHint;
   property PopupMenu;
   property ShowHint;
   property TabOrder;
   property Visible;
   property HotTracking: boolean read FHotTrack write SetHotTrack default true;
   property TabStop default true;

   property OnDownClick: TNotifyEvent read FOnDownClick write FOnDownClick;
   property OnDragDrop;
   property OnDragOver;
   property OnEndDock;
   property OnEndDrag;
   property OnEnter;
   property OnExit;
   property OnStartDock;
   property OnStartDrag;
   property OnUpClick: TNotifyEvent read FOnUpClick write FOnUpClick;
  end;

implementation

constructor TmbTBXSpinButton.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
 ControlStyle := ControlStyle + [csOpaque];
 ParentColor := false;
 AddThemeNotification(Self);
 FMouseLoc := mlNone;
 FMouseOver := false;
 FSbut := ssNone;
 Width := GetSystemMetrics(SM_CXHSCROLL);
 Height := 22;
 FTimer := TTimer.Create(Self);
 FTimer.OnTimer := DoTimer;
 FTimer.Interval := 120;
 DoubleBuffered := true;
 FClicked := false;
 FMouseDown := false;
 FHotTrack := true;
 TabStop := true;
end;

destructor TmbTBXSpinButton.Destroy;
begin
 FTimer.Free;
 inherited Destroy;
 RemoveThemeNotification(Self);
end;

procedure TmbTBXSpinButton.WMPaint(var Message: TWMPaint);
var
 C: TControlCanvas;
 ii: TTBXItemInfo;
 ed: TTBXEditInfo;
 R: TRect;
 b: TBitmap;
begin
 inherited;
 if csDesigning in ComponentState then FMouseLoc := mlNone;
 C := TControlCanvas.Create;
 C.Control := Self;
 b := TBitmap.Create;
 b.Width := Width+20;
 b.Height := Height;
 R := b.Canvas.ClipRect;
  case FMouseLoc of
   mlNone: SetItemInfo('normal', ii, ed);
   mlOver:
    if FHotTrack or Focused then
    SetItemInfo('over', ii, ed)
   else
    SetItemInfo('normal', ii, ed);
   mlDown: SetItemInfo('down', ii, ed);
  end;
 ii.Enabled := Enabled;
 CurrentTheme.PaintButton(b.Canvas, R, ii);
 CurrentTheme.PaintBackgnd(b.Canvas, R, R, R, clBtnFace, false, VT_TOOLBAR);
 CurrentTheme.PaintDock(b.Canvas, R, R, DP_TOP);
 if SameText(CurrentTheme.Name, 'Default') then
  begin
   Dec(R.Top, 2);
   Inc(R.Bottom);
  end
 else
  inflateRect(R, -1, 0);
 CurrentTheme.PaintEditFrame(b.Canvas, R, ii, ed);
 if SameText(CurrentTheme.Name, 'Default') then
  C.Draw(-18, 0, b)
 else
  if FMouseLoc = mlNone then
   C.Draw(-19, 0, b)
  else
   begin
    C.Draw(-19, 0, b);
    R := ClientRect;
    Dec(R.Bottom);
    ExcludeClipRect(C.Handle, 2, 2, R.Right - 2, R.Bottom-1);
    CurrentTheme.PaintFrame(C, R, ii);
   end;
 b.Free;
 c.free;
end;

procedure TmbTBXSpinButton.SetItemInfo(state: string; var IInfo: TTBXItemInfo; var EInfo: TTBXEditInfo);
begin
   FillChar(IInfo, SizeOf(TTBXItemInfo), 0);
   IInfo.ViewType := TVT_NORMALTOOLBAR;
   IInfo.ItemOptions := IO_APPACTIVE or IO_TOOLBARSTYLE;
   IInfo.Enabled := True;

   FillChar(EInfo, SizeOf(TTBXEditInfo), 0);
   EInfo.RightBtnWidth := Width;
   EInfo.RightBtnInfo.ButtonType := EBT_SPIN;

   if state = 'normal' then
    begin
     IInfo.Pushed := false;
     IInfo.Selected := false;
     IInfo.HoverKind := hkNone;
     EInfo.RightBtnInfo.ButtonState := 0;
    end;
   if state = 'over' then
    begin
     IInfo.Pushed := false;
     IInfo.Selected := false;
     IInfo.HoverKind := hkMouseHover;
     EInfo.RightBtnInfo.ButtonState := EBSS_HOT;
    end;
   if state = 'down' then
    begin
     IInfo.Pushed := true;
     IInfo.Selected := false;
     IInfo.HoverKind := hkMouseHover;
     case FSbut of
      ssDown: EInfo.RightBtnInfo.ButtonState := EBSS_DOWN;
      ssUp: EInfo.RightBtnInfo.ButtonState := EBSS_UP;
      ssNone: EInfo.RightBtnInfo.ButtonState := 0;
     end;
    end;
end;

procedure TmbTBXSpinButton.TBMThemeChange(var Message: TMessage);
begin
if Message.WParam = TSC_VIEWCHANGE then
 begin
  FMouseLoc := mlNone;
  Invalidate;
 end;
end;

procedure TmbTBXSpinButton.CMMouseEnter(var Message: TMessage);
begin
 FMouseOver := true;
 FMouseLoc := mlOver;
 if FHotTrack then
 Invalidate;
 inherited;
end;

procedure TmbTBXSpinButton.CMMouseLeave(var Message: TMessage);
begin
 FMouseOver := false;
 if FMouseDown then
  FMouseLoc := mlDown
 else
  FMouseLoc := mlNone;
  if FHotTrack then
 Invalidate;
 inherited;
end;

procedure TmbTBXSpinButton.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
 FMouseOver := true;
 if FMouseDown then
  FMouseLoc := mlDown
 else
  FMouseLoc := mlOver;
 if ssLeft in Shift then
  begin
   if y > Height div 2 then
    FSbut := ssDown
   else
    FSbut := ssUp;
   if not FTimer.Enabled then
    FTimer.Enabled := true;
  end;
  if FHotTrack then
 Invalidate;
 inherited;
end;

procedure TmbTBXSpinButton.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 FMouseDown := true;
 FMouseLoc := mlDown;
 if y > Height div 2 then
  FSbut := ssDown
 else
  FSbut := ssUp;
 FClicked := false;
 if not FTimer.Enabled then
  FTimer.Enabled := true;
 Invalidate;
 inherited;
end;

procedure TmbTBXSpinButton.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 FMouseDown := false;
 FTimer.Enabled := false;
 if not FClicked then
  DoTimer(Self);
 if FMouseOver then
  FMouseLoc := mlOver
 else
  FMouseLoc := mlNone;
 FSbut := ssNone;
{ if y > Height div 2 then
  DownClick
 else
  UpClick;}
 FSbut := ssNone;
 Invalidate;
 inherited;
end;

procedure TmbTBXSpinButton.UpClick;
begin
 if Assigned(FOnUpClick) then
  FOnUpClick(self);
end;

procedure TmbTBXSpinButton.DownClick;
begin
 if Assigned(FOnDownClick) then
  FOnDownClick(self);
end;

procedure TmbTBXSpinButton.DoTimer(Sender: TObject);
begin
 case FSbut of
  ssUp: UpClick;
  ssDown: DownClick;
 end;
 if FSbut <> ssNone then
  FClicked := true;
end;

procedure TmbTBXSpinButton.WMEraseBkgnd(var Message: TWMEraseBkgnd);
begin
 Message.Result := 1;
end;

procedure TmbTBXSpinButton.CMEnabledChanged(var Message: TMessage);
begin
 inherited;
 Invalidate;
end;

procedure TmbTBXSpinButton.SetHotTrack(Value: boolean);
begin
 if FHotTrack <> Value then
  begin
   FHotTrack := Value;
   Invalidate;
  end;
end;

end.
