unit mbTBXValidateEdit;

{$I mxs.inc}

interface

uses
  SysUtils, Classes, mbTBXEdit, StdCtrls, Windows, Messages, Graphics, TBX,
  TBXUtils, TBXThemes;

type
  TValidCharSet = (vcsAll, vcsAlphabet,  vcsCustom, vcsNumbers);

  TmbTBXValidateEdit = class(TmbTBXEdit)
  private
   FCheckChars: string;
   FValidCharSet: TValidCharSet;

   function MakeValid(P: string): string;
   procedure SetValidCharSet(s: TValidCharSet);
   procedure SetCheckChars(c: string);
  protected
   procedure KeyPress(var Key: Char); override;
   procedure Change; override;
  public
   constructor Create(AOwner: TComponent); override;
   destructor Destroy; override;
  published
   property ValidChars: string read FCheckChars write SetCheckChars;
   property ValidCharSet: TValidCharSet read FValidCharSet write SetValidCharSet default vcsAll;
  end;

implementation

constructor TmbTBXValidateEdit.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
ParentColor := false;
 FCheckChars := '';
 FValidCharSet := vcsAll;
end;

destructor TmbTBXValidateEdit.Destroy;
begin
 inherited Destroy;
end;

function IsValidChar(const S: string; Key: Char): boolean;
begin
 Result := Pos(Key, s) > 0
end;

procedure TmbTBXValidateEdit.KeyPress(var Key: Char);
var
 c: string;
begin
 c := '';
 if FValidCharSet <> vcsAll then
  begin
   case FValidCharSet of
    vcsAlphabet: c := 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
    vcsNumbers: c := '0123456789-+';
    vcsCustom: c := FCheckChars;
   end;
   if (not IsValidChar(c, Key)) and (Key >= #32) then Key := #0;
  end;
 if Assigned(OnKeyPress) then
  OnKeyPress(Self, Key);
end;

function TmbTBXValidateEdit.MakeValid(P: string): string;
var
  S, t: string;
  I: Integer;
begin
  S := '';
  case FValidCharSet of
   vcsAlphabet: t := 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
   vcsNumbers: t := '0123456789-+';
   vcsCustom: t := FCheckChars;
  end;
  for I := 1 to Length(P) do
    if IsValidChar(t, P[I]) then
      S := S + P[I];
  Result := S;
end;

procedure TmbTBXValidateEdit.Change;
begin
 if FValidCharSet <> vcsAll then
  Text := MakeValid(Text);
 if Assigned(OnChange) then
  OnChange(Self);
end;

procedure TmbTBXValidateEdit.SetValidCharSet(s: TValidCharSet);
begin
 FValidCharSet := s;
 Text := MakeValid(Text);
 RPaint;
end;

procedure TmbTBXValidateEdit.SetCheckChars(c: string);
begin
 FCheckChars := c;
 if FValidCharSet = vcsCustom then
  begin
   Text := MakeValid(Text);
   RPaint;
  end;
end;

end.
